<?php
/**
 * @package     Joomla.Administrator
 * @subpackage  com_admin
 *
 * @copyright   Copyright (C) 2005 - 2020 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

use Joomla\CMS\Extension\ExtensionHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Installer\Installer;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Log\Log;
use Joomla\CMS\Table\Table;
use Joomla\Component\Fields\Administrator\Model\FieldModel;
use Joomla\Database\ParameterType;

/**
 * Script file of Joomla CMS
 *
 * @since  1.6.4
 */
class JoomlaInstallerScript
{
	/**
	 * The Joomla Version we are updating from
	 *
	 * @var    string
	 * @since  3.7
	 */
	protected $fromVersion = null;

	/**
	 * Function to act prior to installation process begins
	 *
	 * @param   string     $action     Which action is happening (install|uninstall|discover_install|update)
	 * @param   Installer  $installer  The class calling this method
	 *
	 * @return  boolean  True on success
	 *
	 * @since   3.7.0
	 */
	public function preflight($action, $installer)
	{
		if ($action === 'update')
		{
			// Get the version we are updating from
			if (!empty($installer->extension->manifest_cache))
			{
				$manifestValues = json_decode($installer->extension->manifest_cache, true);

				if ((array_key_exists('version', $manifestValues)))
				{
					$this->fromVersion = $manifestValues['version'];

					return true;
				}
			}

			return false;
		}

		return true;
	}

	/**
	 * Method to update Joomla!
	 *
	 * @param   Installer  $installer  The class calling this method
	 *
	 * @return  void
	 */
	public function update($installer)
	{
		$options['format']    = '{DATE}\t{TIME}\t{LEVEL}\t{CODE}\t{MESSAGE}';
		$options['text_file'] = 'joomla_update.php';

		Log::addLogger($options, Log::INFO, array('Update', 'databasequery', 'jerror'));

		try
		{
			Log::add(Text::_('COM_JOOMLAUPDATE_UPDATE_LOG_DELETE_FILES'), Log::INFO, 'Update');
		}
		catch (RuntimeException $exception)
		{
			// Informational log only
		}

		// Ensure we delete the repeatable fields plugin before we remove its files
		$this->uninstallRepeatableFieldsPlugin();

		// This needs to stay for 2.5 update compatibility
		$this->deleteUnexistingFiles();
		$this->updateManifestCaches();
		$this->updateDatabase();
		$this->updateAssets($installer);
		$this->clearStatsCache();
		$this->convertTablesToUtf8mb4(true);
		$this->cleanJoomlaCache();

		// VERY IMPORTANT! THIS METHOD SHOULD BE CALLED LAST, SINCE IT COULD
		// LOGOUT ALL THE USERS
		$this->flushSessions();
	}

	/**
	 * Method to clear our stats plugin cache to ensure we get fresh data on Joomla Update
	 *
	 * @return  void
	 *
	 * @since   3.5
	 */
	protected function clearStatsCache()
	{
		$db = Factory::getDbo();

		try
		{
			// Get the params for the stats plugin
			$params = $db->setQuery(
				$db->getQuery(true)
					->select($db->quoteName('params'))
					->from($db->quoteName('#__extensions'))
					->where($db->quoteName('type') . ' = ' . $db->quote('plugin'))
					->where($db->quoteName('folder') . ' = ' . $db->quote('system'))
					->where($db->quoteName('element') . ' = ' . $db->quote('stats'))
			)->loadResult();
		}
		catch (Exception $e)
		{
			echo Text::sprintf('JLIB_DATABASE_ERROR_FUNCTION_FAILED', $e->getCode(), $e->getMessage()) . '<br>';

			return;
		}

		$params = json_decode($params, true);

		// Reset the last run parameter
		if (isset($params['lastrun']))
		{
			$params['lastrun'] = '';
		}

		$params = json_encode($params);

		$query = $db->getQuery(true)
			->update($db->quoteName('#__extensions'))
			->set($db->quoteName('params') . ' = ' . $db->quote($params))
			->where($db->quoteName('type') . ' = ' . $db->quote('plugin'))
			->where($db->quoteName('folder') . ' = ' . $db->quote('system'))
			->where($db->quoteName('element') . ' = ' . $db->quote('stats'));

		try
		{
			$db->setQuery($query)->execute();
		}
		catch (Exception $e)
		{
			echo Text::sprintf('JLIB_DATABASE_ERROR_FUNCTION_FAILED', $e->getCode(), $e->getMessage()) . '<br>';

			return;
		}
	}

	/**
	 * Method to update Database
	 *
	 * @return  void
	 */
	protected function updateDatabase()
	{
		if (Factory::getDbo()->getServerType() === 'mysql')
		{
			$this->updateDatabaseMysql();
		}
	}

	/**
	 * Method to update MySQL Database
	 *
	 * @return  void
	 */
	protected function updateDatabaseMysql()
	{
		$db = Factory::getDbo();

		$db->setQuery('SHOW ENGINES');

		try
		{
			$results = $db->loadObjectList();
		}
		catch (Exception $e)
		{
			echo Text::sprintf('JLIB_DATABASE_ERROR_FUNCTION_FAILED', $e->getCode(), $e->getMessage()) . '<br>';

			return;
		}

		foreach ($results as $result)
		{
			if ($result->Support != 'DEFAULT')
			{
				continue;
			}

			$db->setQuery('ALTER TABLE #__update_sites_extensions ENGINE = ' . $result->Engine);

			try
			{
				$db->execute();
			}
			catch (Exception $e)
			{
				echo Text::sprintf('JLIB_DATABASE_ERROR_FUNCTION_FAILED', $e->getCode(), $e->getMessage()) . '<br>';

				return;
			}

			break;
		}
	}

	/**
	 * Uninstalls the plg_fields_repeatable plugin and transforms its custom field instances
	 * to instances of the plg_fields_subfields plugin.
	 *
	 * @return  void
	 *
	 * @since   4.0.0
	 */
	protected function uninstallRepeatableFieldsPlugin()
	{
		$app = Factory::getApplication();
		$db = Factory::getDbo();

		// Check if the plg_fields_repeatable plugin is present
		$extensionId = $db->setQuery(
			$db->getQuery(true)
				->select('extension_id')
				->from('#__extensions')
				->where('name = ' . $db->quote('plg_fields_repeatable'))
		)->loadResult();

		// Skip uninstalling when it doesn't exist
		if (!$extensionId)
		{
			return;
		}

		try
		{
			$db->transactionStart();

			// Get the FieldsModelField, we need it in a sec
			$fieldModel = $app->bootComponent('com_fields')->getMVCFactory()->createModel('Field', 'Administrator', ['ignore_request' => true]);
			/** @var FieldModel $fieldModel */

			// Now get a list of all `repeatable` custom field instances
			$db->setQuery(
				$db->getQuery(true)
					->select('*')
					->from('#__fields')
					->where($db->quoteName('type') . ' = ' . $db->quote('repeatable'))
			);

			// Execute the query and iterate over the `repeatable` instances
			foreach ($db->loadObjectList() as $row)
			{
				// Skip broken rows - just a security measure, should not happen
				if (!isset($row->fieldparams) || !($oldFieldparams = json_decode($row->fieldparams)) || !is_object($oldFieldparams))
				{
					continue;
				}

				/**
				 * We basically want to transform this `repeatable` type into a `subfields` type. While $oldFieldparams
				 * holds the `fieldparams` of the `repeatable` type, $newFieldparams shall hold the `fieldparams`
				 * of the `subfields` type.
				 */
				$newFieldparams = array(
					'repeat'  => '1',
					'options' => array(),
				);

				// If this repeatable fields actually had child-fields (normally this is always the case)
				if (isset($oldFieldparams->fields) && is_object($oldFieldparams->fields))
				{
					// Small counter for the child-fields (aka sub fields)
					$newFieldCount = 0;

					// Iterate over the sub fields
					foreach (get_object_vars($oldFieldparams->fields) as $oldField)
					{
						// Used for field name collision prevention
						$fieldname_prefix = '';
						$fieldname_suffix = 0;

						// Try to save the new sub field in a loop because of field name collisions
						while (true)
						{
							/**
							 * We basically want to create a completely new custom fields instance for every sub field
							 * of the `repeatable` instance. This is what we use $data for, we create a new custom field
							 * for each of the sub fields of the `repeatable` instance.
							 */
							$data = array(
								'context'       => $row->context,
								'group_id'      => $row->group_id,
								'title'         => $oldField->fieldname,
								'name'          => (
									$fieldname_prefix
									. $oldField->fieldname
									. ($fieldname_suffix > 0 ? ('_' . $fieldname_suffix) : '')
								),
								'label'         => $oldField->fieldname,
								'default_value' => $row->default_value,
								'type'          => $oldField->fieldtype,
								'description'   => $row->description,
								'state'         => '1',
								'params'        => $row->params,
								'language'      => '*',
								'assigned_cat_ids' => [-1],
							);

							// `number` is not a valid custom field type, so use `text` instead.
							if ($data['type'] == 'number')
							{
								$data['type'] = 'text';
							}

							// Reset the state because else \Joomla\CMS\MVC\Model\AdminModel will take an already
							// existing value (e.g. from previous save) and do an UPDATE instead of INSERT.
							$fieldModel->setState('field.id', 0);

							// If an error occurred when trying to save this.
							if (!$fieldModel->save($data))
							{
								// If the error is, that the name collided, increase the collision prevention
								$error = $fieldModel->getError();

								if ($error == 'COM_FIELDS_ERROR_UNIQUE_NAME')
								{
									// If this is the first time this error occurs, set only the prefix
									if ($fieldname_prefix == '')
									{
										$fieldname_prefix = ($row->name . '_');
									}
									else
									{
										// Else increase the suffix
										$fieldname_suffix++;
									}

									// And start again with the while loop.
									continue 1;
								}

								// Else bail out with the error. Something is totally wrong.
								throw new \Exception($error);
							}

							// Break out of the while loop, saving was successful.
							break 1;
						}

						// Get the newly created id
						$subfield_id = $fieldModel->getState('field.id');

						// Really check that it is valid
						if (!is_numeric($subfield_id) || $subfield_id < 1)
						{
							throw new \Exception('Something went wrong.');
						}

						// And tell our new `subfields` field about his child
						$newFieldparams['options'][('option' . $newFieldCount)] = array(
							'customfield'   => $subfield_id,
							'render_values' => '1',
						);

						$newFieldCount++;
					}
				}

				// Write back the changed stuff to the database
				$db->setQuery(
					$db->getQuery(true)
						->update('#__fields')
						->set($db->quoteName('type') . ' = ' . $db->quote('subfields'))
						->set($db->quoteName('fieldparams') . ' = ' . $db->quote(json_encode($newFieldparams)))
						->where($db->quoteName('id') . ' = ' . $db->quote($row->id))
				)->execute();
			}

			// Now, unprotect the plugin so we can uninstall it
			$db->setQuery(
				$db->getQuery(true)
					->update('#__extensions')
					->set('protected = 0')
					->where($db->quoteName('extension_id') . ' = ' . $extensionId)
			)->execute();

			// And now uninstall the plugin
			$installer = new Installer;
			$installer->uninstall('plugin', $extensionId);

			$db->transactionCommit();
		}
		catch (\Exception $e)
		{
			$db->transactionRollback();
			throw $e;
		}
	}

	/**
	 * Update the manifest caches
	 *
	 * @return  void
	 */
	protected function updateManifestCaches()
	{
		$extensions = ExtensionHelper::getCoreExtensions();

		// Attempt to refresh manifest caches
		$db    = Factory::getDbo();
		$query = $db->getQuery(true)
			->select('*')
			->from('#__extensions');

		foreach ($extensions as $extension)
		{
			$query->where(
				'type=' . $db->quote($extension[0])
				. ' AND element=' . $db->quote($extension[1])
				. ' AND folder=' . $db->quote($extension[2])
				. ' AND client_id=' . $extension[3], 'OR'
			);
		}

		$db->setQuery($query);

		try
		{
			$extensions = $db->loadObjectList();
		}
		catch (Exception $e)
		{
			echo Text::sprintf('JLIB_DATABASE_ERROR_FUNCTION_FAILED', $e->getCode(), $e->getMessage()) . '<br>';

			return;
		}

		$installer = new Installer;

		foreach ($extensions as $extension)
		{
			if (!$installer->refreshManifestCache($extension->extension_id))
			{
				echo Text::sprintf('FILES_JOOMLA_ERROR_MANIFEST', $extension->type, $extension->element, $extension->name, $extension->client_id) . '<br>';
			}
		}
	}

	/**
	 * Delete files that should not exist
	 *
	 * @return  void
	 */
	public function deleteUnexistingFiles()
	{
		$files = array(
			// Joomla 4.0
			'/administrator/components/com_actionlogs/actionlogs.php',
			'/administrator/components/com_actionlogs/controller.php',
			'/administrator/components/com_actionlogs/controllers/actionlogs.php',
			'/administrator/components/com_actionlogs/helpers/actionlogs.php',
			'/administrator/components/com_actionlogs/helpers/actionlogsphp55.php',
			'/administrator/components/com_actionlogs/libraries/actionlogplugin.php',
			'/administrator/components/com_actionlogs/models/actionlog.php',
			'/administrator/components/com_actionlogs/models/actionlogs.php',
			'/administrator/components/com_actionlogs/models/fields/extension.php',
			'/administrator/components/com_actionlogs/models/fields/logcreator.php',
			'/administrator/components/com_actionlogs/models/fields/logsdaterange.php',
			'/administrator/components/com_actionlogs/models/fields/logtype.php',
			'/administrator/components/com_actionlogs/models/fields/plugininfo.php',
			'/administrator/components/com_actionlogs/models/forms/filter_actionlogs.xml',
			'/administrator/components/com_actionlogs/views/actionlogs/tmpl/default.php',
			'/administrator/components/com_actionlogs/views/actionlogs/tmpl/default.xml',
			'/administrator/components/com_actionlogs/views/actionlogs/view.html.php',
			'/administrator/components/com_admin/admin.php',
			'/administrator/components/com_admin/controller.php',
			'/administrator/components/com_admin/controllers/profile.php',
			'/administrator/components/com_admin/helpers/html/directory.php',
			'/administrator/components/com_admin/helpers/html/phpsetting.php',
			'/administrator/components/com_admin/helpers/html/system.php',
			'/administrator/components/com_admin/models/forms/profile.xml',
			'/administrator/components/com_admin/models/help.php',
			'/administrator/components/com_admin/models/profile.php',
			'/administrator/components/com_admin/models/sysinfo.php',
			'/administrator/components/com_admin/postinstall/eaccelerator.php',
			'/administrator/components/com_admin/postinstall/joomla40checks.php',
			'/administrator/components/com_admin/sql/others/mysql/utf8mb4-conversion-01.sql',
			'/administrator/components/com_admin/sql/others/mysql/utf8mb4-conversion-02.sql',
			'/administrator/components/com_admin/sql/others/mysql/utf8mb4-conversion-03.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.0-2011-12-06.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.0-2011-12-16.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.0-2011-12-19.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.0-2011-12-20.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.0-2011-12-21-1.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.0-2011-12-21-2.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.0-2011-12-22.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.0-2011-12-23.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.0-2011-12-24.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.0-2012-01-10.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.0-2012-01-14.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.1-2012-01-26.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.2-2012-03-05.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.3-2012-03-13.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.4-2012-03-18.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.4-2012-03-19.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.5.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.6.sql',
			'/administrator/components/com_admin/sql/updates/mysql/2.5.7.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.0.0.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.0.1.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.0.2.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.0.3.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.1.0.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.1.1.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.1.2.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.1.3.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.1.4.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.1.5.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.2.0.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.2.1.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.2.2-2013-12-22.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.2.2-2013-12-28.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.2.2-2014-01-08.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.2.2-2014-01-15.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.2.2-2014-01-18.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.2.2-2014-01-23.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.2.3-2014-02-20.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.3.0-2014-02-16.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.3.0-2014-04-02.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.3.4-2014-08-03.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.3.6-2014-09-30.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.4.0-2014-08-24.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.4.0-2014-09-01.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.4.0-2014-09-16.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.4.0-2014-10-20.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.4.0-2014-12-03.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.4.0-2015-01-21.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.4.0-2015-02-26.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.5.0-2015-07-01.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.5.0-2015-10-13.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.5.0-2015-10-26.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.5.0-2015-10-30.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.5.0-2015-11-04.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.5.0-2015-11-05.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.5.0-2016-02-26.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.5.0-2016-03-01.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.5.1-2016-03-25.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.5.1-2016-03-29.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.6.0-2016-04-01.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.6.0-2016-04-06.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.6.0-2016-04-08.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.6.0-2016-04-09.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.6.0-2016-05-06.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.6.0-2016-06-01.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.6.0-2016-06-05.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.6.3-2016-08-15.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.6.3-2016-08-16.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2016-08-06.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2016-08-22.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2016-08-29.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2016-09-29.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2016-10-01.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2016-10-02.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2016-11-04.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2016-11-19.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2016-11-21.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2016-11-24.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2016-11-27.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2017-01-08.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2017-01-09.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2017-01-15.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2017-01-17.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2017-01-31.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2017-02-02.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2017-02-15.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2017-02-17.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2017-03-03.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2017-03-09.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2017-03-19.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2017-04-10.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.0-2017-04-19.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.3-2017-06-03.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.7.4-2017-07-05.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.8.0-2017-07-28.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.8.0-2017-07-31.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.8.2-2017-10-14.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.8.4-2018-01-16.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.8.6-2018-02-14.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.8.8-2018-05-18.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.8.9-2018-06-19.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-05-02.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-05-03.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-05-05.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-05-19.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-05-20.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-05-24.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-05-27.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-06-02.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-06-12.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-06-13.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-06-14.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-06-17.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-07-09.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-07-10.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-07-11.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-08-12.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-08-28.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-08-29.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-09-04.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-10-15.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-10-20.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.0-2018-10-21.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.10-2019-07-09.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.16-2020-02-15.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.16-2020-03-04.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.19-2020-05-16.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.3-2019-01-12.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.3-2019-02-07.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.7-2019-04-23.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.7-2019-04-26.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.7-2019-05-16.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.8-2019-06-11.sql',
			'/administrator/components/com_admin/sql/updates/mysql/3.9.8-2019-06-15.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.0.0.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.0.1.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.0.2.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.0.3.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.1.0.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.1.1.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.1.2.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.1.3.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.1.4.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.1.5.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.2.0.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.2.1.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.2.2-2013-12-22.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.2.2-2013-12-28.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.2.2-2014-01-08.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.2.2-2014-01-15.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.2.2-2014-01-18.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.2.2-2014-01-23.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.2.3-2014-02-20.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.3.0-2013-12-21.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.3.0-2014-02-16.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.3.0-2014-04-02.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.3.4-2014-08-03.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.3.6-2014-09-30.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.4.0-2014-08-24.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.4.0-2014-09-01.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.4.0-2014-09-16.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.4.0-2014-10-20.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.4.0-2014-12-03.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.4.0-2015-01-21.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.4.0-2015-02-26.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.4.4-2015-07-11.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.5.0-2015-10-13.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.5.0-2015-10-26.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.5.0-2015-10-30.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.5.0-2015-11-04.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.5.0-2015-11-05.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.5.0-2016-03-01.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.6.0-2016-04-01.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.6.0-2016-04-08.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.6.0-2016-04-09.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.6.0-2016-05-06.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.6.0-2016-06-01.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.6.0-2016-06-05.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.6.3-2016-08-15.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.6.3-2016-08-16.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.6.3-2016-10-04.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2016-08-06.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2016-08-22.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2016-08-29.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2016-09-29.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2016-10-01.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2016-10-02.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2016-11-04.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2016-11-19.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2016-11-21.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2016-11-24.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2017-01-08.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2017-01-09.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2017-01-15.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2017-01-17.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2017-01-31.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2017-02-02.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2017-02-15.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2017-02-17.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2017-03-03.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2017-03-09.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2017-04-10.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.0-2017-04-19.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.7.4-2017-07-05.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.8.0-2017-07-28.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.8.0-2017-07-31.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.8.2-2017-10-14.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.8.4-2018-01-16.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.8.6-2018-02-14.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.8.8-2018-05-18.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.8.9-2018-06-19.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-05-02.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-05-03.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-05-05.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-05-19.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-05-20.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-05-24.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-05-27.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-06-02.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-06-12.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-06-13.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-06-14.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-06-17.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-07-09.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-07-10.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-07-11.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-08-12.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-08-28.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-08-29.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-09-04.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-10-15.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-10-20.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.0-2018-10-21.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.10-2019-07-09.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.16-2020-02-15.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.16-2020-03-04.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.3-2019-01-12.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.3-2019-02-07.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.7-2019-04-23.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.7-2019-04-26.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.7-2019-05-16.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.8-2019-06-11.sql',
			'/administrator/components/com_admin/sql/updates/postgresql/3.9.8-2019-06-15.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/2.5.2-2012-03-05.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/2.5.3-2012-03-13.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/2.5.4-2012-03-18.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/2.5.4-2012-03-19.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/2.5.5.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/2.5.6.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/2.5.7.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.0.0.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.0.1.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.0.2.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.0.3.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.1.0.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.1.1.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.1.2.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.1.3.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.1.4.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.1.5.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.2.0.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.2.1.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.2.2-2013-12-22.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.2.2-2013-12-28.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.2.2-2014-01-08.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.2.2-2014-01-15.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.2.2-2014-01-18.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.2.2-2014-01-23.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.2.3-2014-02-20.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.3.0-2014-02-16.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.3.0-2014-04-02.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.3.4-2014-08-03.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.3.6-2014-09-30.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.4.0-2014-08-24.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.4.0-2014-09-01.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.4.0-2014-09-16.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.4.0-2014-10-20.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.4.0-2014-12-03.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.4.0-2015-01-21.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.4.0-2015-02-26.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.4.4-2015-07-11.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.5.0-2015-10-13.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.5.0-2015-10-26.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.5.0-2015-10-30.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.5.0-2015-11-04.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.5.0-2015-11-05.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.5.0-2016-03-01.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.6.0-2016-04-01.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.6.0-2016-04-06.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.6.0-2016-04-08.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.6.0-2016-04-09.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.6.0-2016-05-06.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.6.0-2016-06-01.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.6.0-2016-06-05.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.6.3-2016-08-15.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.6.3-2016-08-16.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2016-08-06.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2016-08-22.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2016-08-29.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2016-09-29.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2016-10-01.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2016-10-02.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2016-11-04.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2016-11-19.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2016-11-24.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2017-01-08.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2017-01-09.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2017-01-15.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2017-01-17.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2017-01-31.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2017-02-02.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2017-02-15.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2017-02-16.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2017-02-17.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2017-03-03.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2017-03-09.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2017-04-10.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.0-2017-04-19.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.7.4-2017-07-05.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.8.0-2017-07-28.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.8.0-2017-07-31.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.8.2-2017-10-14.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.8.4-2018-01-16.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.8.6-2018-02-14.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.8.8-2018-05-18.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.8.9-2018-06-19.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-05-02.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-05-03.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-05-05.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-05-19.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-05-20.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-05-24.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-05-27.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-06-02.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-06-12.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-06-13.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-06-14.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-06-17.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-07-09.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-07-10.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-07-11.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-08-12.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-08-28.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-08-29.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-09-04.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-10-15.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-10-20.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.0-2018-10-21.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.10-2019-07-09.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.16-2020-03-04.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.3-2019-01-12.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.3-2019-02-07.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.4-2019-03-06.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.7-2019-04-23.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.7-2019-04-26.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.7-2019-05-16.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.8-2019-06-11.sql',
			'/administrator/components/com_admin/sql/updates/sqlazure/3.9.8-2019-06-15.sql',
			'/administrator/components/com_admin/views/help/tmpl/default.php',
			'/administrator/components/com_admin/views/help/tmpl/default.xml',
			'/administrator/components/com_admin/views/help/tmpl/langforum.php',
			'/administrator/components/com_admin/views/help/view.html.php',
			'/administrator/components/com_admin/views/profile/tmpl/edit.php',
			'/administrator/components/com_admin/views/profile/view.html.php',
			'/administrator/components/com_admin/views/sysinfo/tmpl/default.php',
			'/administrator/components/com_admin/views/sysinfo/tmpl/default.xml',
			'/administrator/components/com_admin/views/sysinfo/tmpl/default_config.php',
			'/administrator/components/com_admin/views/sysinfo/tmpl/default_directory.php',
			'/administrator/components/com_admin/views/sysinfo/tmpl/default_phpinfo.php',
			'/administrator/components/com_admin/views/sysinfo/tmpl/default_phpsettings.php',
			'/administrator/components/com_admin/views/sysinfo/tmpl/default_system.php',
			'/administrator/components/com_admin/views/sysinfo/view.html.php',
			'/administrator/components/com_admin/views/sysinfo/view.json.php',
			'/administrator/components/com_admin/views/sysinfo/view.text.php',
			'/administrator/components/com_associations/associations.php',
			'/administrator/components/com_associations/controller.php',
			'/administrator/components/com_associations/controllers/association.php',
			'/administrator/components/com_associations/controllers/associations.php',
			'/administrator/components/com_associations/helpers/associations.php',
			'/administrator/components/com_associations/layouts/joomla/searchtools/default/bar.php',
			'/administrator/components/com_associations/models/association.php',
			'/administrator/components/com_associations/models/associations.php',
			'/administrator/components/com_associations/models/fields/itemlanguage.php',
			'/administrator/components/com_associations/models/fields/itemtype.php',
			'/administrator/components/com_associations/models/fields/modalassociation.php',
			'/administrator/components/com_associations/models/forms/association.xml',
			'/administrator/components/com_associations/models/forms/filter_associations.xml',
			'/administrator/components/com_associations/views/association/tmpl/edit.php',
			'/administrator/components/com_associations/views/association/view.html.php',
			'/administrator/components/com_associations/views/associations/tmpl/default.php',
			'/administrator/components/com_associations/views/associations/tmpl/default.xml',
			'/administrator/components/com_associations/views/associations/tmpl/modal.php',
			'/administrator/components/com_associations/views/associations/view.html.php',
			'/administrator/components/com_banners/banners.php',
			'/administrator/components/com_banners/controller.php',
			'/administrator/components/com_banners/controllers/banner.php',
			'/administrator/components/com_banners/controllers/banners.php',
			'/administrator/components/com_banners/controllers/client.php',
			'/administrator/components/com_banners/controllers/clients.php',
			'/administrator/components/com_banners/controllers/tracks.php',
			'/administrator/components/com_banners/controllers/tracks.raw.php',
			'/administrator/components/com_banners/helpers/html/banner.php',
			'/administrator/components/com_banners/models/banner.php',
			'/administrator/components/com_banners/models/banners.php',
			'/administrator/components/com_banners/models/client.php',
			'/administrator/components/com_banners/models/clients.php',
			'/administrator/components/com_banners/models/download.php',
			'/administrator/components/com_banners/models/fields/bannerclient.php',
			'/administrator/components/com_banners/models/fields/clicks.php',
			'/administrator/components/com_banners/models/fields/impmade.php',
			'/administrator/components/com_banners/models/fields/imptotal.php',
			'/administrator/components/com_banners/models/forms/banner.xml',
			'/administrator/components/com_banners/models/forms/client.xml',
			'/administrator/components/com_banners/models/forms/download.xml',
			'/administrator/components/com_banners/models/forms/filter_banners.xml',
			'/administrator/components/com_banners/models/forms/filter_clients.xml',
			'/administrator/components/com_banners/models/forms/filter_tracks.xml',
			'/administrator/components/com_banners/models/tracks.php',
			'/administrator/components/com_banners/tables/banner.php',
			'/administrator/components/com_banners/tables/client.php',
			'/administrator/components/com_banners/views/banner/tmpl/edit.php',
			'/administrator/components/com_banners/views/banner/view.html.php',
			'/administrator/components/com_banners/views/banners/tmpl/default.php',
			'/administrator/components/com_banners/views/banners/tmpl/default_batch_body.php',
			'/administrator/components/com_banners/views/banners/tmpl/default_batch_footer.php',
			'/administrator/components/com_banners/views/banners/view.html.php',
			'/administrator/components/com_banners/views/client/tmpl/edit.php',
			'/administrator/components/com_banners/views/client/view.html.php',
			'/administrator/components/com_banners/views/clients/tmpl/default.php',
			'/administrator/components/com_banners/views/clients/view.html.php',
			'/administrator/components/com_banners/views/download/tmpl/default.php',
			'/administrator/components/com_banners/views/download/view.html.php',
			'/administrator/components/com_banners/views/tracks/tmpl/default.php',
			'/administrator/components/com_banners/views/tracks/view.html.php',
			'/administrator/components/com_banners/views/tracks/view.raw.php',
			'/administrator/components/com_cache/cache.php',
			'/administrator/components/com_cache/controller.php',
			'/administrator/components/com_cache/helpers/cache.php',
			'/administrator/components/com_cache/models/cache.php',
			'/administrator/components/com_cache/models/forms/filter_cache.xml',
			'/administrator/components/com_cache/views/cache/tmpl/default.php',
			'/administrator/components/com_cache/views/cache/tmpl/default.xml',
			'/administrator/components/com_cache/views/cache/view.html.php',
			'/administrator/components/com_cache/views/purge/tmpl/default.php',
			'/administrator/components/com_cache/views/purge/tmpl/default.xml',
			'/administrator/components/com_cache/views/purge/view.html.php',
			'/administrator/components/com_categories/categories.php',
			'/administrator/components/com_categories/controller.php',
			'/administrator/components/com_categories/controllers/ajax.json.php',
			'/administrator/components/com_categories/controllers/categories.php',
			'/administrator/components/com_categories/controllers/category.php',
			'/administrator/components/com_categories/helpers/association.php',
			'/administrator/components/com_categories/helpers/html/categoriesadministrator.php',
			'/administrator/components/com_categories/models/categories.php',
			'/administrator/components/com_categories/models/category.php',
			'/administrator/components/com_categories/models/fields/categoryedit.php',
			'/administrator/components/com_categories/models/fields/categoryparent.php',
			'/administrator/components/com_categories/models/fields/modal/category.php',
			'/administrator/components/com_categories/models/forms/category.xml',
			'/administrator/components/com_categories/models/forms/filter_categories.xml',
			'/administrator/components/com_categories/tables/category.php',
			'/administrator/components/com_categories/views/categories/tmpl/default.php',
			'/administrator/components/com_categories/views/categories/tmpl/default.xml',
			'/administrator/components/com_categories/views/categories/tmpl/default_batch_body.php',
			'/administrator/components/com_categories/views/categories/tmpl/default_batch_footer.php',
			'/administrator/components/com_categories/views/categories/tmpl/modal.php',
			'/administrator/components/com_categories/views/categories/view.html.php',
			'/administrator/components/com_categories/views/category/tmpl/edit.php',
			'/administrator/components/com_categories/views/category/tmpl/edit.xml',
			'/administrator/components/com_categories/views/category/tmpl/edit_associations.php',
			'/administrator/components/com_categories/views/category/tmpl/edit_metadata.php',
			'/administrator/components/com_categories/views/category/tmpl/modal.php',
			'/administrator/components/com_categories/views/category/tmpl/modal_associations.php',
			'/administrator/components/com_categories/views/category/tmpl/modal_extrafields.php',
			'/administrator/components/com_categories/views/category/tmpl/modal_metadata.php',
			'/administrator/components/com_categories/views/category/tmpl/modal_options.php',
			'/administrator/components/com_categories/views/category/view.html.php',
			'/administrator/components/com_checkin/checkin.php',
			'/administrator/components/com_checkin/controller.php',
			'/administrator/components/com_checkin/models/checkin.php',
			'/administrator/components/com_checkin/models/forms/filter_checkin.xml',
			'/administrator/components/com_checkin/views/checkin/tmpl/default.php',
			'/administrator/components/com_checkin/views/checkin/tmpl/default.xml',
			'/administrator/components/com_checkin/views/checkin/view.html.php',
			'/administrator/components/com_config/config.php',
			'/administrator/components/com_config/controller.php',
			'/administrator/components/com_config/controller/application/cancel.php',
			'/administrator/components/com_config/controller/application/display.php',
			'/administrator/components/com_config/controller/application/removeroot.php',
			'/administrator/components/com_config/controller/application/save.php',
			'/administrator/components/com_config/controller/application/sendtestmail.php',
			'/administrator/components/com_config/controller/application/store.php',
			'/administrator/components/com_config/controller/component/cancel.php',
			'/administrator/components/com_config/controller/component/display.php',
			'/administrator/components/com_config/controller/component/save.php',
			'/administrator/components/com_config/controllers/application.php',
			'/administrator/components/com_config/controllers/component.php',
			'/administrator/components/com_config/helper/config.php',
			'/administrator/components/com_config/model/application.php',
			'/administrator/components/com_config/model/component.php',
			'/administrator/components/com_config/model/field/configcomponents.php',
			'/administrator/components/com_config/model/field/filters.php',
			'/administrator/components/com_config/model/form/application.xml',
			'/administrator/components/com_config/models/application.php',
			'/administrator/components/com_config/models/component.php',
			'/administrator/components/com_config/view/application/html.php',
			'/administrator/components/com_config/view/application/json.php',
			'/administrator/components/com_config/view/application/tmpl/default.php',
			'/administrator/components/com_config/view/application/tmpl/default.xml',
			'/administrator/components/com_config/view/application/tmpl/default_cache.php',
			'/administrator/components/com_config/view/application/tmpl/default_cookie.php',
			'/administrator/components/com_config/view/application/tmpl/default_database.php',
			'/administrator/components/com_config/view/application/tmpl/default_debug.php',
			'/administrator/components/com_config/view/application/tmpl/default_filters.php',
			'/administrator/components/com_config/view/application/tmpl/default_ftp.php',
			'/administrator/components/com_config/view/application/tmpl/default_ftplogin.php',
			'/administrator/components/com_config/view/application/tmpl/default_locale.php',
			'/administrator/components/com_config/view/application/tmpl/default_mail.php',
			'/administrator/components/com_config/view/application/tmpl/default_metadata.php',
			'/administrator/components/com_config/view/application/tmpl/default_navigation.php',
			'/administrator/components/com_config/view/application/tmpl/default_permissions.php',
			'/administrator/components/com_config/view/application/tmpl/default_proxy.php',
			'/administrator/components/com_config/view/application/tmpl/default_seo.php',
			'/administrator/components/com_config/view/application/tmpl/default_server.php',
			'/administrator/components/com_config/view/application/tmpl/default_session.php',
			'/administrator/components/com_config/view/application/tmpl/default_site.php',
			'/administrator/components/com_config/view/application/tmpl/default_system.php',
			'/administrator/components/com_config/view/component/html.php',
			'/administrator/components/com_config/view/component/tmpl/default.php',
			'/administrator/components/com_config/view/component/tmpl/default.xml',
			'/administrator/components/com_config/view/component/tmpl/default_navigation.php',
			'/administrator/components/com_contact/contact.php',
			'/administrator/components/com_contact/controller.php',
			'/administrator/components/com_contact/controllers/ajax.json.php',
			'/administrator/components/com_contact/controllers/contact.php',
			'/administrator/components/com_contact/controllers/contacts.php',
			'/administrator/components/com_contact/helpers/associations.php',
			'/administrator/components/com_contact/helpers/html/contact.php',
			'/administrator/components/com_contact/models/contact.php',
			'/administrator/components/com_contact/models/contacts.php',
			'/administrator/components/com_contact/models/fields/modal/contact.php',
			'/administrator/components/com_contact/models/forms/contact.xml',
			'/administrator/components/com_contact/models/forms/fields/mail.xml',
			'/administrator/components/com_contact/models/forms/filter_contacts.xml',
			'/administrator/components/com_contact/tables/contact.php',
			'/administrator/components/com_contact/views/contact/tmpl/edit.php',
			'/administrator/components/com_contact/views/contact/tmpl/edit_associations.php',
			'/administrator/components/com_contact/views/contact/tmpl/edit_metadata.php',
			'/administrator/components/com_contact/views/contact/tmpl/edit_params.php',
			'/administrator/components/com_contact/views/contact/tmpl/modal.php',
			'/administrator/components/com_contact/views/contact/tmpl/modal_associations.php',
			'/administrator/components/com_contact/views/contact/tmpl/modal_metadata.php',
			'/administrator/components/com_contact/views/contact/tmpl/modal_params.php',
			'/administrator/components/com_contact/views/contact/view.html.php',
			'/administrator/components/com_contact/views/contacts/tmpl/default.php',
			'/administrator/components/com_contact/views/contacts/tmpl/default_batch.php',
			'/administrator/components/com_contact/views/contacts/tmpl/default_batch_body.php',
			'/administrator/components/com_contact/views/contacts/tmpl/default_batch_footer.php',
			'/administrator/components/com_contact/views/contacts/tmpl/modal.php',
			'/administrator/components/com_contact/views/contacts/view.html.php',
			'/administrator/components/com_content/content.php',
			'/administrator/components/com_content/controller.php',
			'/administrator/components/com_content/controllers/ajax.json.php',
			'/administrator/components/com_content/controllers/article.php',
			'/administrator/components/com_content/controllers/articles.php',
			'/administrator/components/com_content/controllers/featured.php',
			'/administrator/components/com_content/helpers/associations.php',
			'/administrator/components/com_content/helpers/html/contentadministrator.php',
			'/administrator/components/com_content/models/article.php',
			'/administrator/components/com_content/models/articles.php',
			'/administrator/components/com_content/models/feature.php',
			'/administrator/components/com_content/models/featured.php',
			'/administrator/components/com_content/models/fields/modal/article.php',
			'/administrator/components/com_content/models/fields/voteradio.php',
			'/administrator/components/com_content/models/forms/article.xml',
			'/administrator/components/com_content/models/forms/filter_articles.xml',
			'/administrator/components/com_content/models/forms/filter_featured.xml',
			'/administrator/components/com_content/tables/featured.php',
			'/administrator/components/com_content/views/article/tmpl/edit.php',
			'/administrator/components/com_content/views/article/tmpl/edit.xml',
			'/administrator/components/com_content/views/article/tmpl/edit_associations.php',
			'/administrator/components/com_content/views/article/tmpl/edit_metadata.php',
			'/administrator/components/com_content/views/article/tmpl/modal.php',
			'/administrator/components/com_content/views/article/tmpl/modal_associations.php',
			'/administrator/components/com_content/views/article/tmpl/modal_metadata.php',
			'/administrator/components/com_content/views/article/tmpl/pagebreak.php',
			'/administrator/components/com_content/views/article/view.html.php',
			'/administrator/components/com_content/views/articles/tmpl/default.php',
			'/administrator/components/com_content/views/articles/tmpl/default.xml',
			'/administrator/components/com_content/views/articles/tmpl/default_batch_body.php',
			'/administrator/components/com_content/views/articles/tmpl/default_batch_footer.php',
			'/administrator/components/com_content/views/articles/tmpl/modal.php',
			'/administrator/components/com_content/views/articles/view.html.php',
			'/administrator/components/com_content/views/featured/tmpl/default.php',
			'/administrator/components/com_content/views/featured/tmpl/default.xml',
			'/administrator/components/com_content/views/featured/view.html.php',
			'/administrator/components/com_contenthistory/contenthistory.php',
			'/administrator/components/com_contenthistory/controller.php',
			'/administrator/components/com_contenthistory/controllers/history.php',
			'/administrator/components/com_contenthistory/controllers/preview.php',
			'/administrator/components/com_contenthistory/helpers/html/textdiff.php',
			'/administrator/components/com_contenthistory/models/compare.php',
			'/administrator/components/com_contenthistory/models/history.php',
			'/administrator/components/com_contenthistory/models/preview.php',
			'/administrator/components/com_contenthistory/views/compare/tmpl/compare.php',
			'/administrator/components/com_contenthistory/views/compare/view.html.php',
			'/administrator/components/com_contenthistory/views/history/tmpl/modal.php',
			'/administrator/components/com_contenthistory/views/history/view.html.php',
			'/administrator/components/com_contenthistory/views/preview/tmpl/preview.php',
			'/administrator/components/com_contenthistory/views/preview/view.html.php',
			'/administrator/components/com_cpanel/controller.php',
			'/administrator/components/com_cpanel/cpanel.php',
			'/administrator/components/com_cpanel/views/cpanel/tmpl/default.php',
			'/administrator/components/com_cpanel/views/cpanel/tmpl/default.xml',
			'/administrator/components/com_cpanel/views/cpanel/view.html.php',
			'/administrator/components/com_fields/controller.php',
			'/administrator/components/com_fields/controllers/field.php',
			'/administrator/components/com_fields/controllers/fields.php',
			'/administrator/components/com_fields/controllers/group.php',
			'/administrator/components/com_fields/controllers/groups.php',
			'/administrator/components/com_fields/fields.php',
			'/administrator/components/com_fields/libraries/fieldslistplugin.php',
			'/administrator/components/com_fields/libraries/fieldsplugin.php',
			'/administrator/components/com_fields/models/field.php',
			'/administrator/components/com_fields/models/fields.php',
			'/administrator/components/com_fields/models/fields/fieldcontexts.php',
			'/administrator/components/com_fields/models/fields/fieldgroups.php',
			'/administrator/components/com_fields/models/fields/fieldlayout.php',
			'/administrator/components/com_fields/models/fields/section.php',
			'/administrator/components/com_fields/models/fields/type.php',
			'/administrator/components/com_fields/models/forms/field.xml',
			'/administrator/components/com_fields/models/forms/filter_fields.xml',
			'/administrator/components/com_fields/models/forms/filter_groups.xml',
			'/administrator/components/com_fields/models/forms/group.xml',
			'/administrator/components/com_fields/models/group.php',
			'/administrator/components/com_fields/models/groups.php',
			'/administrator/components/com_fields/tables/field.php',
			'/administrator/components/com_fields/tables/group.php',
			'/administrator/components/com_fields/views/field/tmpl/edit.php',
			'/administrator/components/com_fields/views/field/view.html.php',
			'/administrator/components/com_fields/views/fields/tmpl/default.php',
			'/administrator/components/com_fields/views/fields/tmpl/default_batch_body.php',
			'/administrator/components/com_fields/views/fields/tmpl/default_batch_footer.php',
			'/administrator/components/com_fields/views/fields/tmpl/modal.php',
			'/administrator/components/com_fields/views/fields/view.html.php',
			'/administrator/components/com_fields/views/group/tmpl/edit.php',
			'/administrator/components/com_fields/views/group/view.html.php',
			'/administrator/components/com_fields/views/groups/tmpl/default.php',
			'/administrator/components/com_fields/views/groups/tmpl/default_batch_body.php',
			'/administrator/components/com_fields/views/groups/tmpl/default_batch_footer.php',
			'/administrator/components/com_fields/views/groups/view.html.php',
			'/administrator/components/com_finder/controller.php',
			'/administrator/components/com_finder/controllers/filter.php',
			'/administrator/components/com_finder/controllers/filters.php',
			'/administrator/components/com_finder/controllers/index.php',
			'/administrator/components/com_finder/controllers/indexer.json.php',
			'/administrator/components/com_finder/controllers/maps.php',
			'/administrator/components/com_finder/finder.php',
			'/administrator/components/com_finder/helpers/finder.php',
			'/administrator/components/com_finder/helpers/html/finder.php',
			'/administrator/components/com_finder/helpers/indexer/driver/mysql.php',
			'/administrator/components/com_finder/helpers/indexer/driver/postgresql.php',
			'/administrator/components/com_finder/helpers/indexer/driver/sqlsrv.php',
			'/administrator/components/com_finder/helpers/indexer/parser/html.php',
			'/administrator/components/com_finder/helpers/indexer/parser/rtf.php',
			'/administrator/components/com_finder/helpers/indexer/parser/txt.php',
			'/administrator/components/com_finder/helpers/indexer/stemmer.php',
			'/administrator/components/com_finder/helpers/indexer/stemmer/fr.php',
			'/administrator/components/com_finder/helpers/indexer/stemmer/porter_en.php',
			'/administrator/components/com_finder/helpers/indexer/stemmer/snowball.php',
			'/administrator/components/com_finder/models/fields/branches.php',
			'/administrator/components/com_finder/models/fields/contentmap.php',
			'/administrator/components/com_finder/models/fields/contenttypes.php',
			'/administrator/components/com_finder/models/fields/directories.php',
			'/administrator/components/com_finder/models/fields/searchfilter.php',
			'/administrator/components/com_finder/models/filter.php',
			'/administrator/components/com_finder/models/filters.php',
			'/administrator/components/com_finder/models/forms/filter.xml',
			'/administrator/components/com_finder/models/forms/filter_filters.xml',
			'/administrator/components/com_finder/models/forms/filter_index.xml',
			'/administrator/components/com_finder/models/forms/filter_maps.xml',
			'/administrator/components/com_finder/models/index.php',
			'/administrator/components/com_finder/models/indexer.php',
			'/administrator/components/com_finder/models/maps.php',
			'/administrator/components/com_finder/models/statistics.php',
			'/administrator/components/com_finder/tables/filter.php',
			'/administrator/components/com_finder/tables/link.php',
			'/administrator/components/com_finder/tables/map.php',
			'/administrator/components/com_finder/views/filter/tmpl/edit.php',
			'/administrator/components/com_finder/views/filter/view.html.php',
			'/administrator/components/com_finder/views/filters/tmpl/default.php',
			'/administrator/components/com_finder/views/filters/view.html.php',
			'/administrator/components/com_finder/views/index/tmpl/default.php',
			'/administrator/components/com_finder/views/index/view.html.php',
			'/administrator/components/com_finder/views/indexer/tmpl/default.php',
			'/administrator/components/com_finder/views/indexer/view.html.php',
			'/administrator/components/com_finder/views/maps/tmpl/default.php',
			'/administrator/components/com_finder/views/maps/view.html.php',
			'/administrator/components/com_finder/views/statistics/tmpl/default.php',
			'/administrator/components/com_finder/views/statistics/view.html.php',
			'/administrator/components/com_installer/controller.php',
			'/administrator/components/com_installer/controllers/database.php',
			'/administrator/components/com_installer/controllers/discover.php',
			'/administrator/components/com_installer/controllers/install.php',
			'/administrator/components/com_installer/controllers/manage.php',
			'/administrator/components/com_installer/controllers/update.php',
			'/administrator/components/com_installer/controllers/updatesites.php',
			'/administrator/components/com_installer/helpers/html/manage.php',
			'/administrator/components/com_installer/helpers/html/updatesites.php',
			'/administrator/components/com_installer/installer.php',
			'/administrator/components/com_installer/models/database.php',
			'/administrator/components/com_installer/models/discover.php',
			'/administrator/components/com_installer/models/extension.php',
			'/administrator/components/com_installer/models/fields/extensionstatus.php',
			'/administrator/components/com_installer/models/fields/folder.php',
			'/administrator/components/com_installer/models/fields/location.php',
			'/administrator/components/com_installer/models/fields/type.php',
			'/administrator/components/com_installer/models/forms/filter_discover.xml',
			'/administrator/components/com_installer/models/forms/filter_languages.xml',
			'/administrator/components/com_installer/models/forms/filter_manage.xml',
			'/administrator/components/com_installer/models/forms/filter_update.xml',
			'/administrator/components/com_installer/models/forms/filter_updatesites.xml',
			'/administrator/components/com_installer/models/install.php',
			'/administrator/components/com_installer/models/languages.php',
			'/administrator/components/com_installer/models/manage.php',
			'/administrator/components/com_installer/models/update.php',
			'/administrator/components/com_installer/models/updatesites.php',
			'/administrator/components/com_installer/models/warnings.php',
			'/administrator/components/com_installer/views/database/tmpl/default.php',
			'/administrator/components/com_installer/views/database/tmpl/default.xml',
			'/administrator/components/com_installer/views/database/view.html.php',
			'/administrator/components/com_installer/views/default/tmpl/default_ftp.php',
			'/administrator/components/com_installer/views/default/tmpl/default_message.php',
			'/administrator/components/com_installer/views/default/view.php',
			'/administrator/components/com_installer/views/discover/tmpl/default.php',
			'/administrator/components/com_installer/views/discover/tmpl/default.xml',
			'/administrator/components/com_installer/views/discover/tmpl/default_item.php',
			'/administrator/components/com_installer/views/discover/view.html.php',
			'/administrator/components/com_installer/views/install/tmpl/default.php',
			'/administrator/components/com_installer/views/install/tmpl/default.xml',
			'/administrator/components/com_installer/views/install/view.html.php',
			'/administrator/components/com_installer/views/languages/tmpl/default.php',
			'/administrator/components/com_installer/views/languages/tmpl/default.xml',
			'/administrator/components/com_installer/views/languages/view.html.php',
			'/administrator/components/com_installer/views/manage/tmpl/default.php',
			'/administrator/components/com_installer/views/manage/tmpl/default.xml',
			'/administrator/components/com_installer/views/manage/view.html.php',
			'/administrator/components/com_installer/views/update/tmpl/default.php',
			'/administrator/components/com_installer/views/update/tmpl/default.xml',
			'/administrator/components/com_installer/views/update/view.html.php',
			'/administrator/components/com_installer/views/updatesites/tmpl/default.php',
			'/administrator/components/com_installer/views/updatesites/tmpl/default.xml',
			'/administrator/components/com_installer/views/updatesites/view.html.php',
			'/administrator/components/com_installer/views/warnings/tmpl/default.php',
			'/administrator/components/com_installer/views/warnings/tmpl/default.xml',
			'/administrator/components/com_installer/views/warnings/view.html.php',
			'/administrator/components/com_joomlaupdate/controller.php',
			'/administrator/components/com_joomlaupdate/controllers/update.php',
			'/administrator/components/com_joomlaupdate/helpers/joomlaupdate.php',
			'/administrator/components/com_joomlaupdate/helpers/select.php',
			'/administrator/components/com_joomlaupdate/joomlaupdate.php',
			'/administrator/components/com_joomlaupdate/models/default.php',
			'/administrator/components/com_joomlaupdate/views/default/tmpl/complete.php',
			'/administrator/components/com_joomlaupdate/views/default/tmpl/default.php',
			'/administrator/components/com_joomlaupdate/views/default/tmpl/default.xml',
			'/administrator/components/com_joomlaupdate/views/default/tmpl/default_nodownload.php',
			'/administrator/components/com_joomlaupdate/views/default/tmpl/default_noupdate.php',
			'/administrator/components/com_joomlaupdate/views/default/tmpl/default_preupdatecheck.php',
			'/administrator/components/com_joomlaupdate/views/default/tmpl/default_reinstall.php',
			'/administrator/components/com_joomlaupdate/views/default/tmpl/default_update.php',
			'/administrator/components/com_joomlaupdate/views/default/tmpl/default_updatemefirst.php',
			'/administrator/components/com_joomlaupdate/views/default/tmpl/default_upload.php',
			'/administrator/components/com_joomlaupdate/views/default/view.html.php',
			'/administrator/components/com_joomlaupdate/views/update/tmpl/default.php',
			'/administrator/components/com_joomlaupdate/views/update/tmpl/finaliseconfirm.php',
			'/administrator/components/com_joomlaupdate/views/update/view.html.php',
			'/administrator/components/com_joomlaupdate/views/upload/tmpl/captive.php',
			'/administrator/components/com_joomlaupdate/views/upload/view.html.php',
			'/administrator/components/com_languages/controller.php',
			'/administrator/components/com_languages/controllers/installed.php',
			'/administrator/components/com_languages/controllers/language.php',
			'/administrator/components/com_languages/controllers/languages.php',
			'/administrator/components/com_languages/controllers/override.php',
			'/administrator/components/com_languages/controllers/overrides.php',
			'/administrator/components/com_languages/controllers/strings.json.php',
			'/administrator/components/com_languages/helpers/html/languages.php',
			'/administrator/components/com_languages/helpers/jsonresponse.php',
			'/administrator/components/com_languages/helpers/languages.php',
			'/administrator/components/com_languages/helpers/multilangstatus.php',
			'/administrator/components/com_languages/languages.php',
			'/administrator/components/com_languages/layouts/joomla/searchtools/default/bar.php',
			'/administrator/components/com_languages/models/fields/languageclient.php',
			'/administrator/components/com_languages/models/forms/filter_installed.xml',
			'/administrator/components/com_languages/models/forms/filter_languages.xml',
			'/administrator/components/com_languages/models/forms/filter_overrides.xml',
			'/administrator/components/com_languages/models/forms/language.xml',
			'/administrator/components/com_languages/models/forms/override.xml',
			'/administrator/components/com_languages/models/installed.php',
			'/administrator/components/com_languages/models/language.php',
			'/administrator/components/com_languages/models/languages.php',
			'/administrator/components/com_languages/models/override.php',
			'/administrator/components/com_languages/models/overrides.php',
			'/administrator/components/com_languages/models/strings.php',
			'/administrator/components/com_languages/views/installed/tmpl/default.php',
			'/administrator/components/com_languages/views/installed/tmpl/default.xml',
			'/administrator/components/com_languages/views/installed/view.html.php',
			'/administrator/components/com_languages/views/language/tmpl/edit.php',
			'/administrator/components/com_languages/views/language/view.html.php',
			'/administrator/components/com_languages/views/languages/tmpl/default.php',
			'/administrator/components/com_languages/views/languages/tmpl/default.xml',
			'/administrator/components/com_languages/views/languages/view.html.php',
			'/administrator/components/com_languages/views/multilangstatus/tmpl/default.php',
			'/administrator/components/com_languages/views/multilangstatus/view.html.php',
			'/administrator/components/com_languages/views/override/tmpl/edit.php',
			'/administrator/components/com_languages/views/override/view.html.php',
			'/administrator/components/com_languages/views/overrides/tmpl/default.php',
			'/administrator/components/com_languages/views/overrides/tmpl/default.xml',
			'/administrator/components/com_languages/views/overrides/view.html.php',
			'/administrator/components/com_login/controller.php',
			'/administrator/components/com_login/login.php',
			'/administrator/components/com_login/models/login.php',
			'/administrator/components/com_login/views/login/tmpl/default.php',
			'/administrator/components/com_login/views/login/view.html.php',
			'/administrator/components/com_media/controller.php',
			'/administrator/components/com_media/controllers/file.json.php',
			'/administrator/components/com_media/controllers/file.php',
			'/administrator/components/com_media/controllers/folder.php',
			'/administrator/components/com_media/helpers/media.php',
			'/administrator/components/com_media/layouts/toolbar/deletemedia.php',
			'/administrator/components/com_media/layouts/toolbar/newfolder.php',
			'/administrator/components/com_media/layouts/toolbar/uploadmedia.php',
			'/administrator/components/com_media/media.php',
			'/administrator/components/com_media/models/list.php',
			'/administrator/components/com_media/models/manager.php',
			'/administrator/components/com_media/views/images/tmpl/default.php',
			'/administrator/components/com_media/views/images/view.html.php',
			'/administrator/components/com_media/views/imageslist/tmpl/default.php',
			'/administrator/components/com_media/views/imageslist/tmpl/default_folder.php',
			'/administrator/components/com_media/views/imageslist/tmpl/default_image.php',
			'/administrator/components/com_media/views/imageslist/view.html.php',
			'/administrator/components/com_media/views/media/tmpl/default.php',
			'/administrator/components/com_media/views/media/tmpl/default.xml',
			'/administrator/components/com_media/views/media/tmpl/default_folders.php',
			'/administrator/components/com_media/views/media/tmpl/default_navigation.php',
			'/administrator/components/com_media/views/media/view.html.php',
			'/administrator/components/com_media/views/medialist/tmpl/default.php',
			'/administrator/components/com_media/views/medialist/tmpl/details.php',
			'/administrator/components/com_media/views/medialist/tmpl/details_doc.php',
			'/administrator/components/com_media/views/medialist/tmpl/details_docs.php',
			'/administrator/components/com_media/views/medialist/tmpl/details_folder.php',
			'/administrator/components/com_media/views/medialist/tmpl/details_folders.php',
			'/administrator/components/com_media/views/medialist/tmpl/details_img.php',
			'/administrator/components/com_media/views/medialist/tmpl/details_imgs.php',
			'/administrator/components/com_media/views/medialist/tmpl/details_up.php',
			'/administrator/components/com_media/views/medialist/tmpl/details_video.php',
			'/administrator/components/com_media/views/medialist/tmpl/details_videos.php',
			'/administrator/components/com_media/views/medialist/tmpl/thumbs.php',
			'/administrator/components/com_media/views/medialist/tmpl/thumbs_docs.php',
			'/administrator/components/com_media/views/medialist/tmpl/thumbs_folders.php',
			'/administrator/components/com_media/views/medialist/tmpl/thumbs_imgs.php',
			'/administrator/components/com_media/views/medialist/tmpl/thumbs_up.php',
			'/administrator/components/com_media/views/medialist/tmpl/thumbs_videos.php',
			'/administrator/components/com_media/views/medialist/view.html.php',
			'/administrator/components/com_menus/controller.php',
			'/administrator/components/com_menus/controllers/ajax.json.php',
			'/administrator/components/com_menus/controllers/item.php',
			'/administrator/components/com_menus/controllers/items.php',
			'/administrator/components/com_menus/controllers/menu.php',
			'/administrator/components/com_menus/controllers/menus.php',
			'/administrator/components/com_menus/helpers/associations.php',
			'/administrator/components/com_menus/helpers/html/menus.php',
			'/administrator/components/com_menus/layouts/joomla/searchtools/default/bar.php',
			'/administrator/components/com_menus/menus.php',
			'/administrator/components/com_menus/models/fields/componentscategory.php',
			'/administrator/components/com_menus/models/fields/menuitembytype.php',
			'/administrator/components/com_menus/models/fields/menuordering.php',
			'/administrator/components/com_menus/models/fields/menuparent.php',
			'/administrator/components/com_menus/models/fields/menupreset.php',
			'/administrator/components/com_menus/models/fields/menutype.php',
			'/administrator/components/com_menus/models/fields/modal/menu.php',
			'/administrator/components/com_menus/models/forms/filter_items.xml',
			'/administrator/components/com_menus/models/forms/filter_itemsadmin.xml',
			'/administrator/components/com_menus/models/forms/filter_menus.xml',
			'/administrator/components/com_menus/models/forms/item.xml',
			'/administrator/components/com_menus/models/forms/item_alias.xml',
			'/administrator/components/com_menus/models/forms/item_component.xml',
			'/administrator/components/com_menus/models/forms/item_heading.xml',
			'/administrator/components/com_menus/models/forms/item_separator.xml',
			'/administrator/components/com_menus/models/forms/item_url.xml',
			'/administrator/components/com_menus/models/forms/itemadmin.xml',
			'/administrator/components/com_menus/models/forms/itemadmin_alias.xml',
			'/administrator/components/com_menus/models/forms/itemadmin_component.xml',
			'/administrator/components/com_menus/models/forms/itemadmin_container.xml',
			'/administrator/components/com_menus/models/forms/itemadmin_heading.xml',
			'/administrator/components/com_menus/models/forms/itemadmin_separator.xml',
			'/administrator/components/com_menus/models/forms/itemadmin_url.xml',
			'/administrator/components/com_menus/models/forms/menu.xml',
			'/administrator/components/com_menus/models/item.php',
			'/administrator/components/com_menus/models/items.php',
			'/administrator/components/com_menus/models/menu.php',
			'/administrator/components/com_menus/models/menus.php',
			'/administrator/components/com_menus/models/menutypes.php',
			'/administrator/components/com_menus/presets/joomla.xml',
			'/administrator/components/com_menus/presets/modern.xml',
			'/administrator/components/com_menus/tables/menu.php',
			'/administrator/components/com_menus/views/item/tmpl/edit.php',
			'/administrator/components/com_menus/views/item/tmpl/edit.xml',
			'/administrator/components/com_menus/views/item/tmpl/edit_associations.php',
			'/administrator/components/com_menus/views/item/tmpl/edit_container.php',
			'/administrator/components/com_menus/views/item/tmpl/edit_modules.php',
			'/administrator/components/com_menus/views/item/tmpl/edit_options.php',
			'/administrator/components/com_menus/views/item/tmpl/modal.php',
			'/administrator/components/com_menus/views/item/tmpl/modal_associations.php',
			'/administrator/components/com_menus/views/item/tmpl/modal_options.php',
			'/administrator/components/com_menus/views/item/view.html.php',
			'/administrator/components/com_menus/views/items/tmpl/default.php',
			'/administrator/components/com_menus/views/items/tmpl/default.xml',
			'/administrator/components/com_menus/views/items/tmpl/default_batch_body.php',
			'/administrator/components/com_menus/views/items/tmpl/default_batch_footer.php',
			'/administrator/components/com_menus/views/items/tmpl/modal.php',
			'/administrator/components/com_menus/views/items/view.html.php',
			'/administrator/components/com_menus/views/menu/tmpl/edit.php',
			'/administrator/components/com_menus/views/menu/tmpl/edit.xml',
			'/administrator/components/com_menus/views/menu/view.html.php',
			'/administrator/components/com_menus/views/menu/view.xml.php',
			'/administrator/components/com_menus/views/menus/tmpl/default.php',
			'/administrator/components/com_menus/views/menus/tmpl/default.xml',
			'/administrator/components/com_menus/views/menus/view.html.php',
			'/administrator/components/com_menus/views/menutypes/tmpl/default.php',
			'/administrator/components/com_menus/views/menutypes/view.html.php',
			'/administrator/components/com_messages/controller.php',
			'/administrator/components/com_messages/controllers/config.php',
			'/administrator/components/com_messages/controllers/message.php',
			'/administrator/components/com_messages/controllers/messages.php',
			'/administrator/components/com_messages/helpers/html/messages.php',
			'/administrator/components/com_messages/helpers/messages.php',
			'/administrator/components/com_messages/messages.php',
			'/administrator/components/com_messages/models/config.php',
			'/administrator/components/com_messages/models/fields/messagestates.php',
			'/administrator/components/com_messages/models/fields/usermessages.php',
			'/administrator/components/com_messages/models/forms/config.xml',
			'/administrator/components/com_messages/models/forms/filter_messages.xml',
			'/administrator/components/com_messages/models/forms/message.xml',
			'/administrator/components/com_messages/models/message.php',
			'/administrator/components/com_messages/models/messages.php',
			'/administrator/components/com_messages/tables/message.php',
			'/administrator/components/com_messages/views/config/tmpl/default.php',
			'/administrator/components/com_messages/views/config/view.html.php',
			'/administrator/components/com_messages/views/message/tmpl/default.php',
			'/administrator/components/com_messages/views/message/tmpl/edit.php',
			'/administrator/components/com_messages/views/message/view.html.php',
			'/administrator/components/com_messages/views/messages/tmpl/default.php',
			'/administrator/components/com_messages/views/messages/view.html.php',
			'/administrator/components/com_modules/controller.php',
			'/administrator/components/com_modules/controllers/module.php',
			'/administrator/components/com_modules/controllers/modules.php',
			'/administrator/components/com_modules/helpers/html/modules.php',
			'/administrator/components/com_modules/helpers/xml.php',
			'/administrator/components/com_modules/layouts/toolbar/newmodule.php',
			'/administrator/components/com_modules/models/fields/modulesmodule.php',
			'/administrator/components/com_modules/models/fields/modulesposition.php',
			'/administrator/components/com_modules/models/forms/advanced.xml',
			'/administrator/components/com_modules/models/forms/filter_modules.xml',
			'/administrator/components/com_modules/models/forms/filter_modulesadmin.xml',
			'/administrator/components/com_modules/models/forms/module.xml',
			'/administrator/components/com_modules/models/forms/moduleadmin.xml',
			'/administrator/components/com_modules/models/module.php',
			'/administrator/components/com_modules/models/modules.php',
			'/administrator/components/com_modules/models/positions.php',
			'/administrator/components/com_modules/models/select.php',
			'/administrator/components/com_modules/modules.php',
			'/administrator/components/com_modules/views/module/tmpl/edit.php',
			'/administrator/components/com_modules/views/module/tmpl/edit_assignment.php',
			'/administrator/components/com_modules/views/module/tmpl/edit_options.php',
			'/administrator/components/com_modules/views/module/tmpl/edit_positions.php',
			'/administrator/components/com_modules/views/module/tmpl/modal.php',
			'/administrator/components/com_modules/views/module/view.html.php',
			'/administrator/components/com_modules/views/module/view.json.php',
			'/administrator/components/com_modules/views/modules/tmpl/default.php',
			'/administrator/components/com_modules/views/modules/tmpl/default.xml',
			'/administrator/components/com_modules/views/modules/tmpl/default_batch_body.php',
			'/administrator/components/com_modules/views/modules/tmpl/default_batch_footer.php',
			'/administrator/components/com_modules/views/modules/tmpl/modal.php',
			'/administrator/components/com_modules/views/modules/view.html.php',
			'/administrator/components/com_modules/views/positions/tmpl/modal.php',
			'/administrator/components/com_modules/views/positions/view.html.php',
			'/administrator/components/com_modules/views/preview/tmpl/default.php',
			'/administrator/components/com_modules/views/preview/view.html.php',
			'/administrator/components/com_modules/views/select/tmpl/default.php',
			'/administrator/components/com_modules/views/select/view.html.php',
			'/administrator/components/com_newsfeeds/controller.php',
			'/administrator/components/com_newsfeeds/controllers/ajax.json.php',
			'/administrator/components/com_newsfeeds/controllers/newsfeed.php',
			'/administrator/components/com_newsfeeds/controllers/newsfeeds.php',
			'/administrator/components/com_newsfeeds/helpers/associations.php',
			'/administrator/components/com_newsfeeds/helpers/html/newsfeed.php',
			'/administrator/components/com_newsfeeds/models/fields/modal/newsfeed.php',
			'/administrator/components/com_newsfeeds/models/fields/newsfeeds.php',
			'/administrator/components/com_newsfeeds/models/forms/filter_newsfeeds.xml',
			'/administrator/components/com_newsfeeds/models/forms/newsfeed.xml',
			'/administrator/components/com_newsfeeds/models/newsfeed.php',
			'/administrator/components/com_newsfeeds/models/newsfeeds.php',
			'/administrator/components/com_newsfeeds/newsfeeds.php',
			'/administrator/components/com_newsfeeds/tables/newsfeed.php',
			'/administrator/components/com_newsfeeds/views/newsfeed/tmpl/edit.php',
			'/administrator/components/com_newsfeeds/views/newsfeed/tmpl/edit_associations.php',
			'/administrator/components/com_newsfeeds/views/newsfeed/tmpl/edit_display.php',
			'/administrator/components/com_newsfeeds/views/newsfeed/tmpl/edit_metadata.php',
			'/administrator/components/com_newsfeeds/views/newsfeed/tmpl/edit_params.php',
			'/administrator/components/com_newsfeeds/views/newsfeed/tmpl/modal.php',
			'/administrator/components/com_newsfeeds/views/newsfeed/tmpl/modal_associations.php',
			'/administrator/components/com_newsfeeds/views/newsfeed/tmpl/modal_display.php',
			'/administrator/components/com_newsfeeds/views/newsfeed/tmpl/modal_metadata.php',
			'/administrator/components/com_newsfeeds/views/newsfeed/tmpl/modal_params.php',
			'/administrator/components/com_newsfeeds/views/newsfeed/view.html.php',
			'/administrator/components/com_newsfeeds/views/newsfeeds/tmpl/default.php',
			'/administrator/components/com_newsfeeds/views/newsfeeds/tmpl/default_batch_body.php',
			'/administrator/components/com_newsfeeds/views/newsfeeds/tmpl/default_batch_footer.php',
			'/administrator/components/com_newsfeeds/views/newsfeeds/tmpl/modal.php',
			'/administrator/components/com_newsfeeds/views/newsfeeds/view.html.php',
			'/administrator/components/com_plugins/controller.php',
			'/administrator/components/com_plugins/controllers/plugin.php',
			'/administrator/components/com_plugins/controllers/plugins.php',
			'/administrator/components/com_plugins/models/fields/pluginelement.php',
			'/administrator/components/com_plugins/models/fields/pluginordering.php',
			'/administrator/components/com_plugins/models/fields/plugintype.php',
			'/administrator/components/com_plugins/models/forms/filter_plugins.xml',
			'/administrator/components/com_plugins/models/forms/plugin.xml',
			'/administrator/components/com_plugins/models/plugin.php',
			'/administrator/components/com_plugins/models/plugins.php',
			'/administrator/components/com_plugins/plugins.php',
			'/administrator/components/com_plugins/views/plugin/tmpl/edit.php',
			'/administrator/components/com_plugins/views/plugin/tmpl/edit_options.php',
			'/administrator/components/com_plugins/views/plugin/tmpl/modal.php',
			'/administrator/components/com_plugins/views/plugin/view.html.php',
			'/administrator/components/com_plugins/views/plugins/tmpl/default.php',
			'/administrator/components/com_plugins/views/plugins/tmpl/default.xml',
			'/administrator/components/com_plugins/views/plugins/view.html.php',
			'/administrator/components/com_postinstall/controllers/message.php',
			'/administrator/components/com_postinstall/fof.xml',
			'/administrator/components/com_postinstall/models/messages.php',
			'/administrator/components/com_postinstall/postinstall.php',
			'/administrator/components/com_postinstall/toolbar.php',
			'/administrator/components/com_postinstall/views/messages/tmpl/default.php',
			'/administrator/components/com_postinstall/views/messages/tmpl/default.xml',
			'/administrator/components/com_postinstall/views/messages/view.html.php',
			'/administrator/components/com_privacy/controller.php',
			'/administrator/components/com_privacy/controllers/consents.php',
			'/administrator/components/com_privacy/controllers/request.php',
			'/administrator/components/com_privacy/controllers/request.xml.php',
			'/administrator/components/com_privacy/controllers/requests.php',
			'/administrator/components/com_privacy/helpers/export/domain.php',
			'/administrator/components/com_privacy/helpers/export/field.php',
			'/administrator/components/com_privacy/helpers/export/item.php',
			'/administrator/components/com_privacy/helpers/html/helper.php',
			'/administrator/components/com_privacy/helpers/plugin.php',
			'/administrator/components/com_privacy/helpers/privacy.php',
			'/administrator/components/com_privacy/helpers/removal/status.php',
			'/administrator/components/com_privacy/models/capabilities.php',
			'/administrator/components/com_privacy/models/consents.php',
			'/administrator/components/com_privacy/models/dashboard.php',
			'/administrator/components/com_privacy/models/export.php',
			'/administrator/components/com_privacy/models/fields/requeststatus.php',
			'/administrator/components/com_privacy/models/fields/requesttype.php',
			'/administrator/components/com_privacy/models/forms/filter_consents.xml',
			'/administrator/components/com_privacy/models/forms/filter_requests.xml',
			'/administrator/components/com_privacy/models/forms/request.xml',
			'/administrator/components/com_privacy/models/remove.php',
			'/administrator/components/com_privacy/models/request.php',
			'/administrator/components/com_privacy/models/requests.php',
			'/administrator/components/com_privacy/privacy.php',
			'/administrator/components/com_privacy/tables/consent.php',
			'/administrator/components/com_privacy/tables/request.php',
			'/administrator/components/com_privacy/views/capabilities/tmpl/default.php',
			'/administrator/components/com_privacy/views/capabilities/view.html.php',
			'/administrator/components/com_privacy/views/consents/tmpl/default.php',
			'/administrator/components/com_privacy/views/consents/tmpl/default.xml',
			'/administrator/components/com_privacy/views/consents/view.html.php',
			'/administrator/components/com_privacy/views/dashboard/tmpl/default.php',
			'/administrator/components/com_privacy/views/dashboard/tmpl/default.xml',
			'/administrator/components/com_privacy/views/dashboard/view.html.php',
			'/administrator/components/com_privacy/views/export/view.xml.php',
			'/administrator/components/com_privacy/views/request/tmpl/default.php',
			'/administrator/components/com_privacy/views/request/tmpl/edit.php',
			'/administrator/components/com_privacy/views/request/view.html.php',
			'/administrator/components/com_privacy/views/requests/tmpl/default.php',
			'/administrator/components/com_privacy/views/requests/tmpl/default.xml',
			'/administrator/components/com_privacy/views/requests/view.html.php',
			'/administrator/components/com_redirect/controller.php',
			'/administrator/components/com_redirect/controllers/link.php',
			'/administrator/components/com_redirect/controllers/links.php',
			'/administrator/components/com_redirect/helpers/html/redirect.php',
			'/administrator/components/com_redirect/models/fields/redirect.php',
			'/administrator/components/com_redirect/models/forms/filter_links.xml',
			'/administrator/components/com_redirect/models/forms/link.xml',
			'/administrator/components/com_redirect/models/link.php',
			'/administrator/components/com_redirect/models/links.php',
			'/administrator/components/com_redirect/redirect.php',
			'/administrator/components/com_redirect/tables/link.php',
			'/administrator/components/com_redirect/views/link/tmpl/edit.php',
			'/administrator/components/com_redirect/views/link/view.html.php',
			'/administrator/components/com_redirect/views/links/tmpl/default.php',
			'/administrator/components/com_redirect/views/links/tmpl/default.xml',
			'/administrator/components/com_redirect/views/links/tmpl/default_addform.php',
			'/administrator/components/com_redirect/views/links/tmpl/default_batch_body.php',
			'/administrator/components/com_redirect/views/links/tmpl/default_batch_footer.php',
			'/administrator/components/com_redirect/views/links/view.html.php',
			'/administrator/components/com_tags/controller.php',
			'/administrator/components/com_tags/controllers/tag.php',
			'/administrator/components/com_tags/controllers/tags.php',
			'/administrator/components/com_tags/helpers/tags.php',
			'/administrator/components/com_tags/models/forms/filter_tags.xml',
			'/administrator/components/com_tags/models/forms/tag.xml',
			'/administrator/components/com_tags/models/tag.php',
			'/administrator/components/com_tags/models/tags.php',
			'/administrator/components/com_tags/tables/tag.php',
			'/administrator/components/com_tags/tags.php',
			'/administrator/components/com_tags/views/tag/tmpl/edit.php',
			'/administrator/components/com_tags/views/tag/tmpl/edit_metadata.php',
			'/administrator/components/com_tags/views/tag/tmpl/edit_options.php',
			'/administrator/components/com_tags/views/tag/view.html.php',
			'/administrator/components/com_tags/views/tags/tmpl/default.php',
			'/administrator/components/com_tags/views/tags/tmpl/default.xml',
			'/administrator/components/com_tags/views/tags/tmpl/default_batch_body.php',
			'/administrator/components/com_tags/views/tags/tmpl/default_batch_footer.php',
			'/administrator/components/com_tags/views/tags/view.html.php',
			'/administrator/components/com_templates/controller.php',
			'/administrator/components/com_templates/controllers/style.php',
			'/administrator/components/com_templates/controllers/styles.php',
			'/administrator/components/com_templates/controllers/template.php',
			'/administrator/components/com_templates/helpers/html/templates.php',
			'/administrator/components/com_templates/models/fields/templatelocation.php',
			'/administrator/components/com_templates/models/fields/templatename.php',
			'/administrator/components/com_templates/models/forms/filter_styles.xml',
			'/administrator/components/com_templates/models/forms/filter_templates.xml',
			'/administrator/components/com_templates/models/forms/source.xml',
			'/administrator/components/com_templates/models/forms/style.xml',
			'/administrator/components/com_templates/models/forms/style_administrator.xml',
			'/administrator/components/com_templates/models/forms/style_site.xml',
			'/administrator/components/com_templates/models/style.php',
			'/administrator/components/com_templates/models/styles.php',
			'/administrator/components/com_templates/models/template.php',
			'/administrator/components/com_templates/models/templates.php',
			'/administrator/components/com_templates/tables/style.php',
			'/administrator/components/com_templates/templates.php',
			'/administrator/components/com_templates/views/style/tmpl/edit.php',
			'/administrator/components/com_templates/views/style/tmpl/edit_assignment.php',
			'/administrator/components/com_templates/views/style/tmpl/edit_options.php',
			'/administrator/components/com_templates/views/style/view.html.php',
			'/administrator/components/com_templates/views/style/view.json.php',
			'/administrator/components/com_templates/views/styles/tmpl/default.php',
			'/administrator/components/com_templates/views/styles/tmpl/default.xml',
			'/administrator/components/com_templates/views/styles/view.html.php',
			'/administrator/components/com_templates/views/template/tmpl/default.php',
			'/administrator/components/com_templates/views/template/tmpl/default_description.php',
			'/administrator/components/com_templates/views/template/tmpl/default_folders.php',
			'/administrator/components/com_templates/views/template/tmpl/default_modal_copy_body.php',
			'/administrator/components/com_templates/views/template/tmpl/default_modal_copy_footer.php',
			'/administrator/components/com_templates/views/template/tmpl/default_modal_delete_body.php',
			'/administrator/components/com_templates/views/template/tmpl/default_modal_delete_footer.php',
			'/administrator/components/com_templates/views/template/tmpl/default_modal_file_body.php',
			'/administrator/components/com_templates/views/template/tmpl/default_modal_file_footer.php',
			'/administrator/components/com_templates/views/template/tmpl/default_modal_folder_body.php',
			'/administrator/components/com_templates/views/template/tmpl/default_modal_folder_footer.php',
			'/administrator/components/com_templates/views/template/tmpl/default_modal_rename_body.php',
			'/administrator/components/com_templates/views/template/tmpl/default_modal_rename_footer.php',
			'/administrator/components/com_templates/views/template/tmpl/default_modal_resize_body.php',
			'/administrator/components/com_templates/views/template/tmpl/default_modal_resize_footer.php',
			'/administrator/components/com_templates/views/template/tmpl/default_tree.php',
			'/administrator/components/com_templates/views/template/tmpl/readonly.php',
			'/administrator/components/com_templates/views/template/view.html.php',
			'/administrator/components/com_templates/views/templates/tmpl/default.php',
			'/administrator/components/com_templates/views/templates/tmpl/default.xml',
			'/administrator/components/com_templates/views/templates/view.html.php',
			'/administrator/components/com_users/controller.php',
			'/administrator/components/com_users/controllers/group.php',
			'/administrator/components/com_users/controllers/groups.php',
			'/administrator/components/com_users/controllers/level.php',
			'/administrator/components/com_users/controllers/levels.php',
			'/administrator/components/com_users/controllers/mail.php',
			'/administrator/components/com_users/controllers/note.php',
			'/administrator/components/com_users/controllers/notes.php',
			'/administrator/components/com_users/controllers/user.php',
			'/administrator/components/com_users/controllers/users.php',
			'/administrator/components/com_users/helpers/html/users.php',
			'/administrator/components/com_users/models/debuggroup.php',
			'/administrator/components/com_users/models/debuguser.php',
			'/administrator/components/com_users/models/fields/groupparent.php',
			'/administrator/components/com_users/models/fields/levels.php',
			'/administrator/components/com_users/models/forms/config_domain.xml',
			'/administrator/components/com_users/models/forms/fields/user.xml',
			'/administrator/components/com_users/models/forms/filter_debuggroup.xml',
			'/administrator/components/com_users/models/forms/filter_debuguser.xml',
			'/administrator/components/com_users/models/forms/filter_groups.xml',
			'/administrator/components/com_users/models/forms/filter_levels.xml',
			'/administrator/components/com_users/models/forms/filter_notes.xml',
			'/administrator/components/com_users/models/forms/filter_users.xml',
			'/administrator/components/com_users/models/forms/group.xml',
			'/administrator/components/com_users/models/forms/level.xml',
			'/administrator/components/com_users/models/forms/mail.xml',
			'/administrator/components/com_users/models/forms/note.xml',
			'/administrator/components/com_users/models/forms/user.xml',
			'/administrator/components/com_users/models/group.php',
			'/administrator/components/com_users/models/groups.php',
			'/administrator/components/com_users/models/level.php',
			'/administrator/components/com_users/models/levels.php',
			'/administrator/components/com_users/models/mail.php',
			'/administrator/components/com_users/models/note.php',
			'/administrator/components/com_users/models/notes.php',
			'/administrator/components/com_users/models/user.php',
			'/administrator/components/com_users/models/users.php',
			'/administrator/components/com_users/tables/note.php',
			'/administrator/components/com_users/users.php',
			'/administrator/components/com_users/views/debuggroup/tmpl/default.php',
			'/administrator/components/com_users/views/debuggroup/view.html.php',
			'/administrator/components/com_users/views/debuguser/tmpl/default.php',
			'/administrator/components/com_users/views/debuguser/view.html.php',
			'/administrator/components/com_users/views/group/tmpl/edit.php',
			'/administrator/components/com_users/views/group/tmpl/edit.xml',
			'/administrator/components/com_users/views/group/view.html.php',
			'/administrator/components/com_users/views/groups/tmpl/default.php',
			'/administrator/components/com_users/views/groups/tmpl/default.xml',
			'/administrator/components/com_users/views/groups/view.html.php',
			'/administrator/components/com_users/views/level/tmpl/edit.php',
			'/administrator/components/com_users/views/level/tmpl/edit.xml',
			'/administrator/components/com_users/views/level/view.html.php',
			'/administrator/components/com_users/views/levels/tmpl/default.php',
			'/administrator/components/com_users/views/levels/tmpl/default.xml',
			'/administrator/components/com_users/views/levels/view.html.php',
			'/administrator/components/com_users/views/mail/tmpl/default.php',
			'/administrator/components/com_users/views/mail/tmpl/default.xml',
			'/administrator/components/com_users/views/mail/view.html.php',
			'/administrator/components/com_users/views/note/tmpl/edit.php',
			'/administrator/components/com_users/views/note/tmpl/edit.xml',
			'/administrator/components/com_users/views/note/view.html.php',
			'/administrator/components/com_users/views/notes/tmpl/default.php',
			'/administrator/components/com_users/views/notes/tmpl/default.xml',
			'/administrator/components/com_users/views/notes/tmpl/modal.php',
			'/administrator/components/com_users/views/notes/view.html.php',
			'/administrator/components/com_users/views/user/tmpl/edit.php',
			'/administrator/components/com_users/views/user/tmpl/edit.xml',
			'/administrator/components/com_users/views/user/tmpl/edit_groups.php',
			'/administrator/components/com_users/views/user/view.html.php',
			'/administrator/components/com_users/views/users/tmpl/default.php',
			'/administrator/components/com_users/views/users/tmpl/default.xml',
			'/administrator/components/com_users/views/users/tmpl/default_batch_body.php',
			'/administrator/components/com_users/views/users/tmpl/default_batch_footer.php',
			'/administrator/components/com_users/views/users/tmpl/modal.php',
			'/administrator/components/com_users/views/users/view.html.php',
			'/administrator/help/helpsites.xml',
			'/administrator/includes/helper.php',
			'/administrator/includes/subtoolbar.php',
			'/administrator/language/en-GB/en-GB.com_actionlogs.ini',
			'/administrator/language/en-GB/en-GB.com_actionlogs.sys.ini',
			'/administrator/language/en-GB/en-GB.com_admin.ini',
			'/administrator/language/en-GB/en-GB.com_admin.sys.ini',
			'/administrator/language/en-GB/en-GB.com_ajax.ini',
			'/administrator/language/en-GB/en-GB.com_ajax.sys.ini',
			'/administrator/language/en-GB/en-GB.com_associations.ini',
			'/administrator/language/en-GB/en-GB.com_associations.sys.ini',
			'/administrator/language/en-GB/en-GB.com_banners.ini',
			'/administrator/language/en-GB/en-GB.com_banners.sys.ini',
			'/administrator/language/en-GB/en-GB.com_cache.ini',
			'/administrator/language/en-GB/en-GB.com_cache.sys.ini',
			'/administrator/language/en-GB/en-GB.com_categories.ini',
			'/administrator/language/en-GB/en-GB.com_categories.sys.ini',
			'/administrator/language/en-GB/en-GB.com_checkin.ini',
			'/administrator/language/en-GB/en-GB.com_checkin.sys.ini',
			'/administrator/language/en-GB/en-GB.com_config.ini',
			'/administrator/language/en-GB/en-GB.com_config.sys.ini',
			'/administrator/language/en-GB/en-GB.com_contact.ini',
			'/administrator/language/en-GB/en-GB.com_contact.sys.ini',
			'/administrator/language/en-GB/en-GB.com_content.ini',
			'/administrator/language/en-GB/en-GB.com_content.sys.ini',
			'/administrator/language/en-GB/en-GB.com_contenthistory.ini',
			'/administrator/language/en-GB/en-GB.com_contenthistory.sys.ini',
			'/administrator/language/en-GB/en-GB.com_cpanel.ini',
			'/administrator/language/en-GB/en-GB.com_cpanel.sys.ini',
			'/administrator/language/en-GB/en-GB.com_fields.ini',
			'/administrator/language/en-GB/en-GB.com_fields.sys.ini',
			'/administrator/language/en-GB/en-GB.com_finder.ini',
			'/administrator/language/en-GB/en-GB.com_finder.sys.ini',
			'/administrator/language/en-GB/en-GB.com_installer.ini',
			'/administrator/language/en-GB/en-GB.com_installer.sys.ini',
			'/administrator/language/en-GB/en-GB.com_joomlaupdate.ini',
			'/administrator/language/en-GB/en-GB.com_joomlaupdate.sys.ini',
			'/administrator/language/en-GB/en-GB.com_languages.ini',
			'/administrator/language/en-GB/en-GB.com_languages.sys.ini',
			'/administrator/language/en-GB/en-GB.com_login.ini',
			'/administrator/language/en-GB/en-GB.com_login.sys.ini',
			'/administrator/language/en-GB/en-GB.com_mailto.sys.ini',
			'/administrator/language/en-GB/en-GB.com_media.ini',
			'/administrator/language/en-GB/en-GB.com_media.sys.ini',
			'/administrator/language/en-GB/en-GB.com_menus.ini',
			'/administrator/language/en-GB/en-GB.com_menus.sys.ini',
			'/administrator/language/en-GB/en-GB.com_messages.ini',
			'/administrator/language/en-GB/en-GB.com_messages.sys.ini',
			'/administrator/language/en-GB/en-GB.com_modules.ini',
			'/administrator/language/en-GB/en-GB.com_modules.sys.ini',
			'/administrator/language/en-GB/en-GB.com_newsfeeds.ini',
			'/administrator/language/en-GB/en-GB.com_newsfeeds.sys.ini',
			'/administrator/language/en-GB/en-GB.com_plugins.ini',
			'/administrator/language/en-GB/en-GB.com_plugins.sys.ini',
			'/administrator/language/en-GB/en-GB.com_postinstall.ini',
			'/administrator/language/en-GB/en-GB.com_postinstall.sys.ini',
			'/administrator/language/en-GB/en-GB.com_privacy.ini',
			'/administrator/language/en-GB/en-GB.com_privacy.sys.ini',
			'/administrator/language/en-GB/en-GB.com_redirect.ini',
			'/administrator/language/en-GB/en-GB.com_redirect.sys.ini',
			'/administrator/language/en-GB/en-GB.com_tags.ini',
			'/administrator/language/en-GB/en-GB.com_tags.sys.ini',
			'/administrator/language/en-GB/en-GB.com_templates.ini',
			'/administrator/language/en-GB/en-GB.com_templates.sys.ini',
			'/administrator/language/en-GB/en-GB.com_users.ini',
			'/administrator/language/en-GB/en-GB.com_users.sys.ini',
			'/administrator/language/en-GB/en-GB.com_weblinks.ini',
			'/administrator/language/en-GB/en-GB.com_weblinks.sys.ini',
			'/administrator/language/en-GB/en-GB.com_wrapper.ini',
			'/administrator/language/en-GB/en-GB.com_wrapper.sys.ini',
			'/administrator/language/en-GB/en-GB.ini',
			'/administrator/language/en-GB/en-GB.lib_joomla.ini',
			'/administrator/language/en-GB/en-GB.localise.php',
			'/administrator/language/en-GB/en-GB.mod_custom.ini',
			'/administrator/language/en-GB/en-GB.mod_custom.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_feed.ini',
			'/administrator/language/en-GB/en-GB.mod_feed.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_latest.ini',
			'/administrator/language/en-GB/en-GB.mod_latest.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_latestactions.ini',
			'/administrator/language/en-GB/en-GB.mod_latestactions.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_logged.ini',
			'/administrator/language/en-GB/en-GB.mod_logged.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_login.ini',
			'/administrator/language/en-GB/en-GB.mod_login.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_menu.ini',
			'/administrator/language/en-GB/en-GB.mod_menu.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_multilangstatus.ini',
			'/administrator/language/en-GB/en-GB.mod_multilangstatus.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_popular.ini',
			'/administrator/language/en-GB/en-GB.mod_popular.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_privacy_dashboard.ini',
			'/administrator/language/en-GB/en-GB.mod_privacy_dashboard.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_quickicon.ini',
			'/administrator/language/en-GB/en-GB.mod_quickicon.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_sampledata.ini',
			'/administrator/language/en-GB/en-GB.mod_sampledata.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_stats_admin.ini',
			'/administrator/language/en-GB/en-GB.mod_stats_admin.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_status.ini',
			'/administrator/language/en-GB/en-GB.mod_status.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_submenu.ini',
			'/administrator/language/en-GB/en-GB.mod_submenu.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_title.ini',
			'/administrator/language/en-GB/en-GB.mod_title.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_toolbar.ini',
			'/administrator/language/en-GB/en-GB.mod_toolbar.sys.ini',
			'/administrator/language/en-GB/en-GB.mod_version.ini',
			'/administrator/language/en-GB/en-GB.mod_version.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_actionlog_joomla.ini',
			'/administrator/language/en-GB/en-GB.plg_actionlog_joomla.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_authentication_cookie.ini',
			'/administrator/language/en-GB/en-GB.plg_authentication_cookie.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_authentication_gmail.ini',
			'/administrator/language/en-GB/en-GB.plg_authentication_gmail.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_authentication_joomla.ini',
			'/administrator/language/en-GB/en-GB.plg_authentication_joomla.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_authentication_ldap.ini',
			'/administrator/language/en-GB/en-GB.plg_authentication_ldap.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_captcha_recaptcha.ini',
			'/administrator/language/en-GB/en-GB.plg_captcha_recaptcha.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_captcha_recaptcha_invisible.ini',
			'/administrator/language/en-GB/en-GB.plg_captcha_recaptcha_invisible.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_content_confirmconsent.ini',
			'/administrator/language/en-GB/en-GB.plg_content_confirmconsent.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_content_contact.ini',
			'/administrator/language/en-GB/en-GB.plg_content_contact.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_content_emailcloak.ini',
			'/administrator/language/en-GB/en-GB.plg_content_emailcloak.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_content_fields.ini',
			'/administrator/language/en-GB/en-GB.plg_content_fields.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_content_finder.ini',
			'/administrator/language/en-GB/en-GB.plg_content_finder.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_content_joomla.ini',
			'/administrator/language/en-GB/en-GB.plg_content_joomla.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_content_loadmodule.ini',
			'/administrator/language/en-GB/en-GB.plg_content_loadmodule.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_content_pagebreak.ini',
			'/administrator/language/en-GB/en-GB.plg_content_pagebreak.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_content_pagenavigation.ini',
			'/administrator/language/en-GB/en-GB.plg_content_pagenavigation.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_content_vote.ini',
			'/administrator/language/en-GB/en-GB.plg_content_vote.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_article.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_article.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_contact.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_contact.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_fields.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_fields.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_image.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_image.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_menu.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_menu.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_module.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_module.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_pagebreak.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_pagebreak.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_readmore.ini',
			'/administrator/language/en-GB/en-GB.plg_editors-xtd_readmore.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_editors_codemirror.ini',
			'/administrator/language/en-GB/en-GB.plg_editors_codemirror.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_editors_none.ini',
			'/administrator/language/en-GB/en-GB.plg_editors_none.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_editors_tinymce.ini',
			'/administrator/language/en-GB/en-GB.plg_editors_tinymce.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_extension_joomla.ini',
			'/administrator/language/en-GB/en-GB.plg_extension_joomla.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_calendar.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_calendar.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_checkboxes.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_checkboxes.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_color.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_color.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_editor.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_editor.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_image.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_image.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_imagelist.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_imagelist.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_integer.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_integer.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_list.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_list.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_media.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_media.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_radio.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_radio.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_sql.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_sql.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_text.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_text.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_textarea.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_textarea.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_url.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_url.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_user.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_user.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_usergrouplist.ini',
			'/administrator/language/en-GB/en-GB.plg_fields_usergrouplist.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_finder_categories.ini',
			'/administrator/language/en-GB/en-GB.plg_finder_categories.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_finder_contacts.ini',
			'/administrator/language/en-GB/en-GB.plg_finder_contacts.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_finder_content.ini',
			'/administrator/language/en-GB/en-GB.plg_finder_content.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_finder_newsfeeds.ini',
			'/administrator/language/en-GB/en-GB.plg_finder_newsfeeds.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_finder_tags.ini',
			'/administrator/language/en-GB/en-GB.plg_finder_tags.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_finder_weblinks.ini',
			'/administrator/language/en-GB/en-GB.plg_finder_weblinks.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_installer_folderinstaller.ini',
			'/administrator/language/en-GB/en-GB.plg_installer_folderinstaller.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_installer_packageinstaller.ini',
			'/administrator/language/en-GB/en-GB.plg_installer_packageinstaller.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_installer_urlinstaller.ini',
			'/administrator/language/en-GB/en-GB.plg_installer_urlinstaller.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_installer_webinstaller.ini',
			'/administrator/language/en-GB/en-GB.plg_installer_webinstaller.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_privacy_actionlogs.ini',
			'/administrator/language/en-GB/en-GB.plg_privacy_actionlogs.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_privacy_consents.ini',
			'/administrator/language/en-GB/en-GB.plg_privacy_consents.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_privacy_contact.ini',
			'/administrator/language/en-GB/en-GB.plg_privacy_contact.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_privacy_content.ini',
			'/administrator/language/en-GB/en-GB.plg_privacy_content.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_privacy_message.ini',
			'/administrator/language/en-GB/en-GB.plg_privacy_message.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_privacy_user.ini',
			'/administrator/language/en-GB/en-GB.plg_privacy_user.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_quickicon_extensionupdate.ini',
			'/administrator/language/en-GB/en-GB.plg_quickicon_extensionupdate.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_quickicon_joomlaupdate.ini',
			'/administrator/language/en-GB/en-GB.plg_quickicon_joomlaupdate.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_quickicon_phpversioncheck.ini',
			'/administrator/language/en-GB/en-GB.plg_quickicon_phpversioncheck.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_quickicon_privacycheck.ini',
			'/administrator/language/en-GB/en-GB.plg_quickicon_privacycheck.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_sampledata_blog.ini',
			'/administrator/language/en-GB/en-GB.plg_sampledata_blog.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_actionlogs.ini',
			'/administrator/language/en-GB/en-GB.plg_system_actionlogs.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_cache.ini',
			'/administrator/language/en-GB/en-GB.plg_system_cache.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_debug.ini',
			'/administrator/language/en-GB/en-GB.plg_system_debug.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_fields.ini',
			'/administrator/language/en-GB/en-GB.plg_system_fields.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_highlight.ini',
			'/administrator/language/en-GB/en-GB.plg_system_highlight.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_languagecode.ini',
			'/administrator/language/en-GB/en-GB.plg_system_languagecode.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_languagefilter.ini',
			'/administrator/language/en-GB/en-GB.plg_system_languagefilter.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_log.ini',
			'/administrator/language/en-GB/en-GB.plg_system_log.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_logout.ini',
			'/administrator/language/en-GB/en-GB.plg_system_logout.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_logrotation.ini',
			'/administrator/language/en-GB/en-GB.plg_system_logrotation.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_p3p.ini',
			'/administrator/language/en-GB/en-GB.plg_system_p3p.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_privacyconsent.ini',
			'/administrator/language/en-GB/en-GB.plg_system_privacyconsent.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_redirect.ini',
			'/administrator/language/en-GB/en-GB.plg_system_redirect.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_remember.ini',
			'/administrator/language/en-GB/en-GB.plg_system_remember.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_sef.ini',
			'/administrator/language/en-GB/en-GB.plg_system_sef.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_sessiongc.ini',
			'/administrator/language/en-GB/en-GB.plg_system_sessiongc.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_stats.ini',
			'/administrator/language/en-GB/en-GB.plg_system_stats.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_system_updatenotification.ini',
			'/administrator/language/en-GB/en-GB.plg_system_updatenotification.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_twofactorauth_totp.ini',
			'/administrator/language/en-GB/en-GB.plg_twofactorauth_totp.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_twofactorauth_yubikey.ini',
			'/administrator/language/en-GB/en-GB.plg_twofactorauth_yubikey.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_user_contactcreator.ini',
			'/administrator/language/en-GB/en-GB.plg_user_contactcreator.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_user_joomla.ini',
			'/administrator/language/en-GB/en-GB.plg_user_joomla.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_user_profile.ini',
			'/administrator/language/en-GB/en-GB.plg_user_profile.sys.ini',
			'/administrator/language/en-GB/en-GB.plg_user_terms.ini',
			'/administrator/language/en-GB/en-GB.plg_user_terms.sys.ini',
			'/administrator/language/en-GB/en-GB.tpl_hathor.ini',
			'/administrator/language/en-GB/en-GB.tpl_hathor.sys.ini',
			'/administrator/language/en-GB/en-GB.tpl_isis.ini',
			'/administrator/language/en-GB/en-GB.tpl_isis.sys.ini',
			'/administrator/language/en-GB/en-GB.xml',
			'/administrator/manifests/libraries/fof.xml',
			'/administrator/manifests/libraries/idna_convert.xml',
			'/administrator/manifests/libraries/phputf8.xml',
			'/administrator/modules/mod_feed/helper.php',
			'/administrator/modules/mod_latest/helper.php',
			'/administrator/modules/mod_latestactions/helper.php',
			'/administrator/modules/mod_logged/helper.php',
			'/administrator/modules/mod_login/helper.php',
			'/administrator/modules/mod_menu/helper.php',
			'/administrator/modules/mod_menu/menu.php',
			'/administrator/modules/mod_multilangstatus/language/en-GB/en-GB.mod_multilangstatus.ini',
			'/administrator/modules/mod_multilangstatus/language/en-GB/en-GB.mod_multilangstatus.sys.ini',
			'/administrator/modules/mod_popular/helper.php',
			'/administrator/modules/mod_privacy_dashboard/helper.php',
			'/administrator/modules/mod_quickicon/helper.php',
			'/administrator/modules/mod_quickicon/mod_quickicon.php',
			'/administrator/modules/mod_sampledata/helper.php',
			'/administrator/modules/mod_stats_admin/helper.php',
			'/administrator/modules/mod_stats_admin/language/en-GB.mod_stats_admin.ini',
			'/administrator/modules/mod_stats_admin/language/en-GB.mod_stats_admin.sys.ini',
			'/administrator/modules/mod_status/mod_status.php',
			'/administrator/modules/mod_status/mod_status.xml',
			'/administrator/modules/mod_status/tmpl/default.php',
			'/administrator/modules/mod_version/helper.php',
			'/administrator/modules/mod_version/language/en-GB/en-GB.mod_version.ini',
			'/administrator/modules/mod_version/language/en-GB/en-GB.mod_version.sys.ini',
			'/administrator/templates/hathor/LICENSE.txt',
			'/administrator/templates/hathor/component.php',
			'/administrator/templates/hathor/cpanel.php',
			'/administrator/templates/hathor/css/boldtext.css',
			'/administrator/templates/hathor/css/colour_blue.css',
			'/administrator/templates/hathor/css/colour_blue_rtl.css',
			'/administrator/templates/hathor/css/colour_brown.css',
			'/administrator/templates/hathor/css/colour_brown_rtl.css',
			'/administrator/templates/hathor/css/colour_highcontrast.css',
			'/administrator/templates/hathor/css/colour_highcontrast_rtl.css',
			'/administrator/templates/hathor/css/colour_standard.css',
			'/administrator/templates/hathor/css/colour_standard_rtl.css',
			'/administrator/templates/hathor/css/error.css',
			'/administrator/templates/hathor/css/ie7.css',
			'/administrator/templates/hathor/css/ie8.css',
			'/administrator/templates/hathor/css/template.css',
			'/administrator/templates/hathor/css/template_rtl.css',
			'/administrator/templates/hathor/css/theme.css',
			'/administrator/templates/hathor/error.php',
			'/administrator/templates/hathor/favicon.ico',
			'/administrator/templates/hathor/html/com_admin/help/default.php',
			'/administrator/templates/hathor/html/com_admin/profile/edit.php',
			'/administrator/templates/hathor/html/com_admin/sysinfo/default.php',
			'/administrator/templates/hathor/html/com_admin/sysinfo/default_config.php',
			'/administrator/templates/hathor/html/com_admin/sysinfo/default_directory.php',
			'/administrator/templates/hathor/html/com_admin/sysinfo/default_navigation.php',
			'/administrator/templates/hathor/html/com_admin/sysinfo/default_phpsettings.php',
			'/administrator/templates/hathor/html/com_admin/sysinfo/default_system.php',
			'/administrator/templates/hathor/html/com_associations/associations/default.php',
			'/administrator/templates/hathor/html/com_banners/banner/edit.php',
			'/administrator/templates/hathor/html/com_banners/banners/default.php',
			'/administrator/templates/hathor/html/com_banners/client/edit.php',
			'/administrator/templates/hathor/html/com_banners/clients/default.php',
			'/administrator/templates/hathor/html/com_banners/download/default.php',
			'/administrator/templates/hathor/html/com_banners/tracks/default.php',
			'/administrator/templates/hathor/html/com_cache/cache/default.php',
			'/administrator/templates/hathor/html/com_cache/purge/default.php',
			'/administrator/templates/hathor/html/com_categories/categories/default.php',
			'/administrator/templates/hathor/html/com_categories/category/edit.php',
			'/administrator/templates/hathor/html/com_categories/category/edit_options.php',
			'/administrator/templates/hathor/html/com_checkin/checkin/default.php',
			'/administrator/templates/hathor/html/com_config/application/default.php',
			'/administrator/templates/hathor/html/com_config/application/default_cache.php',
			'/administrator/templates/hathor/html/com_config/application/default_cookie.php',
			'/administrator/templates/hathor/html/com_config/application/default_database.php',
			'/administrator/templates/hathor/html/com_config/application/default_debug.php',
			'/administrator/templates/hathor/html/com_config/application/default_filters.php',
			'/administrator/templates/hathor/html/com_config/application/default_ftp.php',
			'/administrator/templates/hathor/html/com_config/application/default_ftplogin.php',
			'/administrator/templates/hathor/html/com_config/application/default_locale.php',
			'/administrator/templates/hathor/html/com_config/application/default_mail.php',
			'/administrator/templates/hathor/html/com_config/application/default_metadata.php',
			'/administrator/templates/hathor/html/com_config/application/default_navigation.php',
			'/administrator/templates/hathor/html/com_config/application/default_permissions.php',
			'/administrator/templates/hathor/html/com_config/application/default_seo.php',
			'/administrator/templates/hathor/html/com_config/application/default_server.php',
			'/administrator/templates/hathor/html/com_config/application/default_session.php',
			'/administrator/templates/hathor/html/com_config/application/default_site.php',
			'/administrator/templates/hathor/html/com_config/application/default_system.php',
			'/administrator/templates/hathor/html/com_config/component/default.php',
			'/administrator/templates/hathor/html/com_contact/contact/edit.php',
			'/administrator/templates/hathor/html/com_contact/contact/edit_params.php',
			'/administrator/templates/hathor/html/com_contact/contacts/default.php',
			'/administrator/templates/hathor/html/com_contact/contacts/modal.php',
			'/administrator/templates/hathor/html/com_content/article/edit.php',
			'/administrator/templates/hathor/html/com_content/articles/default.php',
			'/administrator/templates/hathor/html/com_content/articles/modal.php',
			'/administrator/templates/hathor/html/com_content/featured/default.php',
			'/administrator/templates/hathor/html/com_contenthistory/history/modal.php',
			'/administrator/templates/hathor/html/com_cpanel/cpanel/default.php',
			'/administrator/templates/hathor/html/com_fields/field/edit.php',
			'/administrator/templates/hathor/html/com_fields/fields/default.php',
			'/administrator/templates/hathor/html/com_fields/group/edit.php',
			'/administrator/templates/hathor/html/com_fields/groups/default.php',
			'/administrator/templates/hathor/html/com_finder/filters/default.php',
			'/administrator/templates/hathor/html/com_finder/index/default.php',
			'/administrator/templates/hathor/html/com_finder/maps/default.php',
			'/administrator/templates/hathor/html/com_installer/database/default.php',
			'/administrator/templates/hathor/html/com_installer/default/default_ftp.php',
			'/administrator/templates/hathor/html/com_installer/discover/default.php',
			'/administrator/templates/hathor/html/com_installer/install/default.php',
			'/administrator/templates/hathor/html/com_installer/install/default_form.php',
			'/administrator/templates/hathor/html/com_installer/languages/default.php',
			'/administrator/templates/hathor/html/com_installer/languages/default_filter.php',
			'/administrator/templates/hathor/html/com_installer/manage/default.php',
			'/administrator/templates/hathor/html/com_installer/manage/default_filter.php',
			'/administrator/templates/hathor/html/com_installer/update/default.php',
			'/administrator/templates/hathor/html/com_installer/warnings/default.php',
			'/administrator/templates/hathor/html/com_joomlaupdate/default/default.php',
			'/administrator/templates/hathor/html/com_languages/installed/default.php',
			'/administrator/templates/hathor/html/com_languages/installed/default_ftp.php',
			'/administrator/templates/hathor/html/com_languages/languages/default.php',
			'/administrator/templates/hathor/html/com_languages/overrides/default.php',
			'/administrator/templates/hathor/html/com_menus/item/edit.php',
			'/administrator/templates/hathor/html/com_menus/item/edit_options.php',
			'/administrator/templates/hathor/html/com_menus/items/default.php',
			'/administrator/templates/hathor/html/com_menus/menu/edit.php',
			'/administrator/templates/hathor/html/com_menus/menus/default.php',
			'/administrator/templates/hathor/html/com_menus/menutypes/default.php',
			'/administrator/templates/hathor/html/com_messages/message/edit.php',
			'/administrator/templates/hathor/html/com_messages/messages/default.php',
			'/administrator/templates/hathor/html/com_modules/module/edit.php',
			'/administrator/templates/hathor/html/com_modules/module/edit_assignment.php',
			'/administrator/templates/hathor/html/com_modules/module/edit_options.php',
			'/administrator/templates/hathor/html/com_modules/modules/default.php',
			'/administrator/templates/hathor/html/com_modules/positions/modal.php',
			'/administrator/templates/hathor/html/com_newsfeeds/newsfeed/edit.php',
			'/administrator/templates/hathor/html/com_newsfeeds/newsfeed/edit_params.php',
			'/administrator/templates/hathor/html/com_newsfeeds/newsfeeds/default.php',
			'/administrator/templates/hathor/html/com_newsfeeds/newsfeeds/modal.php',
			'/administrator/templates/hathor/html/com_plugins/plugin/edit.php',
			'/administrator/templates/hathor/html/com_plugins/plugin/edit_options.php',
			'/administrator/templates/hathor/html/com_plugins/plugins/default.php',
			'/administrator/templates/hathor/html/com_postinstall/messages/default.php',
			'/administrator/templates/hathor/html/com_redirect/links/default.php',
			'/administrator/templates/hathor/html/com_search/searches/default.php',
			'/administrator/templates/hathor/html/com_tags/tag/edit.php',
			'/administrator/templates/hathor/html/com_tags/tag/edit_metadata.php',
			'/administrator/templates/hathor/html/com_tags/tag/edit_options.php',
			'/administrator/templates/hathor/html/com_tags/tags/default.php',
			'/administrator/templates/hathor/html/com_templates/style/edit.php',
			'/administrator/templates/hathor/html/com_templates/style/edit_assignment.php',
			'/administrator/templates/hathor/html/com_templates/style/edit_options.php',
			'/administrator/templates/hathor/html/com_templates/styles/default.php',
			'/administrator/templates/hathor/html/com_templates/template/default.php',
			'/administrator/templates/hathor/html/com_templates/template/default_description.php',
			'/administrator/templates/hathor/html/com_templates/template/default_folders.php',
			'/administrator/templates/hathor/html/com_templates/template/default_tree.php',
			'/administrator/templates/hathor/html/com_templates/templates/default.php',
			'/administrator/templates/hathor/html/com_users/debuggroup/default.php',
			'/administrator/templates/hathor/html/com_users/debuguser/default.php',
			'/administrator/templates/hathor/html/com_users/groups/default.php',
			'/administrator/templates/hathor/html/com_users/levels/default.php',
			'/administrator/templates/hathor/html/com_users/note/edit.php',
			'/administrator/templates/hathor/html/com_users/notes/default.php',
			'/administrator/templates/hathor/html/com_users/user/edit.php',
			'/administrator/templates/hathor/html/com_users/users/default.php',
			'/administrator/templates/hathor/html/com_users/users/modal.php',
			'/administrator/templates/hathor/html/com_weblinks/weblink/edit.php',
			'/administrator/templates/hathor/html/com_weblinks/weblink/edit_params.php',
			'/administrator/templates/hathor/html/com_weblinks/weblinks/default.php',
			'/administrator/templates/hathor/html/layouts/com_media/toolbar/deletemedia.php',
			'/administrator/templates/hathor/html/layouts/com_media/toolbar/newfolder.php',
			'/administrator/templates/hathor/html/layouts/com_media/toolbar/uploadmedia.php',
			'/administrator/templates/hathor/html/layouts/com_messages/toolbar/mysettings.php',
			'/administrator/templates/hathor/html/layouts/com_modules/toolbar/cancelselect.php',
			'/administrator/templates/hathor/html/layouts/com_modules/toolbar/newmodule.php',
			'/administrator/templates/hathor/html/layouts/joomla/edit/details.php',
			'/administrator/templates/hathor/html/layouts/joomla/edit/fieldset.php',
			'/administrator/templates/hathor/html/layouts/joomla/edit/global.php',
			'/administrator/templates/hathor/html/layouts/joomla/edit/metadata.php',
			'/administrator/templates/hathor/html/layouts/joomla/edit/params.php',
			'/administrator/templates/hathor/html/layouts/joomla/quickicons/icon.php',
			'/administrator/templates/hathor/html/layouts/joomla/sidebars/submenu.php',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar/base.php',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar/batch.php',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar/confirm.php',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar/containerclose.php',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar/containeropen.php',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar/help.php',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar/iconclass.php',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar/link.php',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar/modal.php',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar/popup.php',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar/separator.php',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar/slider.php',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar/standard.php',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar/title.php',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar/versions.php',
			'/administrator/templates/hathor/html/layouts/plugins/user/profile/fields/dob.php',
			'/administrator/templates/hathor/html/mod_login/default.php',
			'/administrator/templates/hathor/html/mod_quickicon/default.php',
			'/administrator/templates/hathor/html/modules.php',
			'/administrator/templates/hathor/html/pagination.php',
			'/administrator/templates/hathor/images/admin/blank.png',
			'/administrator/templates/hathor/images/admin/checked_out.png',
			'/administrator/templates/hathor/images/admin/collapseall.png',
			'/administrator/templates/hathor/images/admin/disabled.png',
			'/administrator/templates/hathor/images/admin/downarrow-1.png',
			'/administrator/templates/hathor/images/admin/downarrow.png',
			'/administrator/templates/hathor/images/admin/downarrow0.png',
			'/administrator/templates/hathor/images/admin/expandall.png',
			'/administrator/templates/hathor/images/admin/featured.png',
			'/administrator/templates/hathor/images/admin/filesave.png',
			'/administrator/templates/hathor/images/admin/filter_16.png',
			'/administrator/templates/hathor/images/admin/icon-16-allow.png',
			'/administrator/templates/hathor/images/admin/icon-16-allowinactive.png',
			'/administrator/templates/hathor/images/admin/icon-16-deny.png',
			'/administrator/templates/hathor/images/admin/icon-16-denyinactive.png',
			'/administrator/templates/hathor/images/admin/icon-16-links.png',
			'/administrator/templates/hathor/images/admin/icon-16-notice-note.png',
			'/administrator/templates/hathor/images/admin/icon-16-protected.png',
			'/administrator/templates/hathor/images/admin/menu_divider.png',
			'/administrator/templates/hathor/images/admin/note_add_16.png',
			'/administrator/templates/hathor/images/admin/publish_g.png',
			'/administrator/templates/hathor/images/admin/publish_r.png',
			'/administrator/templates/hathor/images/admin/publish_x.png',
			'/administrator/templates/hathor/images/admin/publish_y.png',
			'/administrator/templates/hathor/images/admin/sort_asc.png',
			'/administrator/templates/hathor/images/admin/sort_desc.png',
			'/administrator/templates/hathor/images/admin/tick.png',
			'/administrator/templates/hathor/images/admin/trash.png',
			'/administrator/templates/hathor/images/admin/uparrow-1.png',
			'/administrator/templates/hathor/images/admin/uparrow.png',
			'/administrator/templates/hathor/images/admin/uparrow0.png',
			'/administrator/templates/hathor/images/arrow.png',
			'/administrator/templates/hathor/images/bg-menu.gif',
			'/administrator/templates/hathor/images/calendar.png',
			'/administrator/templates/hathor/images/header/icon-48-alert.png',
			'/administrator/templates/hathor/images/header/icon-48-apply.png',
			'/administrator/templates/hathor/images/header/icon-48-archive.png',
			'/administrator/templates/hathor/images/header/icon-48-article-add.png',
			'/administrator/templates/hathor/images/header/icon-48-article-edit.png',
			'/administrator/templates/hathor/images/header/icon-48-article.png',
			'/administrator/templates/hathor/images/header/icon-48-assoc.png',
			'/administrator/templates/hathor/images/header/icon-48-banner-categories.png',
			'/administrator/templates/hathor/images/header/icon-48-banner-client.png',
			'/administrator/templates/hathor/images/header/icon-48-banner-tracks.png',
			'/administrator/templates/hathor/images/header/icon-48-banner.png',
			'/administrator/templates/hathor/images/header/icon-48-calendar.png',
			'/administrator/templates/hathor/images/header/icon-48-category-add.png',
			'/administrator/templates/hathor/images/header/icon-48-category.png',
			'/administrator/templates/hathor/images/header/icon-48-checkin.png',
			'/administrator/templates/hathor/images/header/icon-48-clear.png',
			'/administrator/templates/hathor/images/header/icon-48-component.png',
			'/administrator/templates/hathor/images/header/icon-48-config.png',
			'/administrator/templates/hathor/images/header/icon-48-contacts-categories.png',
			'/administrator/templates/hathor/images/header/icon-48-contacts.png',
			'/administrator/templates/hathor/images/header/icon-48-content.png',
			'/administrator/templates/hathor/images/header/icon-48-cpanel.png',
			'/administrator/templates/hathor/images/header/icon-48-default.png',
			'/administrator/templates/hathor/images/header/icon-48-deny.png',
			'/administrator/templates/hathor/images/header/icon-48-download.png',
			'/administrator/templates/hathor/images/header/icon-48-edit.png',
			'/administrator/templates/hathor/images/header/icon-48-extension.png',
			'/administrator/templates/hathor/images/header/icon-48-featured.png',
			'/administrator/templates/hathor/images/header/icon-48-frontpage.png',
			'/administrator/templates/hathor/images/header/icon-48-generic.png',
			'/administrator/templates/hathor/images/header/icon-48-groups-add.png',
			'/administrator/templates/hathor/images/header/icon-48-groups.png',
			'/administrator/templates/hathor/images/header/icon-48-help-forum.png',
			'/administrator/templates/hathor/images/header/icon-48-help-this.png',
			'/administrator/templates/hathor/images/header/icon-48-help_header.png',
			'/administrator/templates/hathor/images/header/icon-48-inbox.png',
			'/administrator/templates/hathor/images/header/icon-48-info.png',
			'/administrator/templates/hathor/images/header/icon-48-install.png',
			'/administrator/templates/hathor/images/header/icon-48-jupdate-updatefound.png',
			'/administrator/templates/hathor/images/header/icon-48-jupdate-uptodate.png',
			'/administrator/templates/hathor/images/header/icon-48-language.png',
			'/administrator/templates/hathor/images/header/icon-48-levels-add.png',
			'/administrator/templates/hathor/images/header/icon-48-levels.png',
			'/administrator/templates/hathor/images/header/icon-48-links-cat.png',
			'/administrator/templates/hathor/images/header/icon-48-links.png',
			'/administrator/templates/hathor/images/header/icon-48-massmail.png',
			'/administrator/templates/hathor/images/header/icon-48-media.png',
			'/administrator/templates/hathor/images/header/icon-48-menu-add.png',
			'/administrator/templates/hathor/images/header/icon-48-menu.png',
			'/administrator/templates/hathor/images/header/icon-48-menumgr.png',
			'/administrator/templates/hathor/images/header/icon-48-module.png',
			'/administrator/templates/hathor/images/header/icon-48-move.png',
			'/administrator/templates/hathor/images/header/icon-48-new-privatemessage.png',
			'/administrator/templates/hathor/images/header/icon-48-newcategory.png',
			'/administrator/templates/hathor/images/header/icon-48-newsfeeds-cat.png',
			'/administrator/templates/hathor/images/header/icon-48-newsfeeds.png',
			'/administrator/templates/hathor/images/header/icon-48-notice.png',
			'/administrator/templates/hathor/images/header/icon-48-plugin.png',
			'/administrator/templates/hathor/images/header/icon-48-preview.png',
			'/administrator/templates/hathor/images/header/icon-48-print.png',
			'/administrator/templates/hathor/images/header/icon-48-purge.png',
			'/administrator/templates/hathor/images/header/icon-48-puzzle.png',
			'/administrator/templates/hathor/images/header/icon-48-read-privatemessage.png',
			'/administrator/templates/hathor/images/header/icon-48-readmess.png',
			'/administrator/templates/hathor/images/header/icon-48-redirect.png',
			'/administrator/templates/hathor/images/header/icon-48-revert.png',
			'/administrator/templates/hathor/images/header/icon-48-search.png',
			'/administrator/templates/hathor/images/header/icon-48-section.png',
			'/administrator/templates/hathor/images/header/icon-48-send.png',
			'/administrator/templates/hathor/images/header/icon-48-static.png',
			'/administrator/templates/hathor/images/header/icon-48-stats.png',
			'/administrator/templates/hathor/images/header/icon-48-tags.png',
			'/administrator/templates/hathor/images/header/icon-48-themes.png',
			'/administrator/templates/hathor/images/header/icon-48-trash.png',
			'/administrator/templates/hathor/images/header/icon-48-unarchive.png',
			'/administrator/templates/hathor/images/header/icon-48-upload.png',
			'/administrator/templates/hathor/images/header/icon-48-user-add.png',
			'/administrator/templates/hathor/images/header/icon-48-user-edit.png',
			'/administrator/templates/hathor/images/header/icon-48-user-profile.png',
			'/administrator/templates/hathor/images/header/icon-48-user.png',
			'/administrator/templates/hathor/images/header/icon-48-writemess.png',
			'/administrator/templates/hathor/images/header/icon-messaging.png',
			'/administrator/templates/hathor/images/j_arrow.png',
			'/administrator/templates/hathor/images/j_arrow_down.png',
			'/administrator/templates/hathor/images/j_arrow_left.png',
			'/administrator/templates/hathor/images/j_arrow_right.png',
			'/administrator/templates/hathor/images/j_login_lock.png',
			'/administrator/templates/hathor/images/j_logo.png',
			'/administrator/templates/hathor/images/logo.png',
			'/administrator/templates/hathor/images/menu/icon-16-alert.png',
			'/administrator/templates/hathor/images/menu/icon-16-apply.png',
			'/administrator/templates/hathor/images/menu/icon-16-archive.png',
			'/administrator/templates/hathor/images/menu/icon-16-article.png',
			'/administrator/templates/hathor/images/menu/icon-16-assoc.png',
			'/administrator/templates/hathor/images/menu/icon-16-back-user.png',
			'/administrator/templates/hathor/images/menu/icon-16-banner-categories.png',
			'/administrator/templates/hathor/images/menu/icon-16-banner-client.png',
			'/administrator/templates/hathor/images/menu/icon-16-banner-tracks.png',
			'/administrator/templates/hathor/images/menu/icon-16-banner.png',
			'/administrator/templates/hathor/images/menu/icon-16-calendar.png',
			'/administrator/templates/hathor/images/menu/icon-16-category.png',
			'/administrator/templates/hathor/images/menu/icon-16-checkin.png',
			'/administrator/templates/hathor/images/menu/icon-16-clear.png',
			'/administrator/templates/hathor/images/menu/icon-16-component.png',
			'/administrator/templates/hathor/images/menu/icon-16-config.png',
			'/administrator/templates/hathor/images/menu/icon-16-contacts-categories.png',
			'/administrator/templates/hathor/images/menu/icon-16-contacts.png',
			'/administrator/templates/hathor/images/menu/icon-16-content.png',
			'/administrator/templates/hathor/images/menu/icon-16-cpanel.png',
			'/administrator/templates/hathor/images/menu/icon-16-default.png',
			'/administrator/templates/hathor/images/menu/icon-16-delete.png',
			'/administrator/templates/hathor/images/menu/icon-16-deny.png',
			'/administrator/templates/hathor/images/menu/icon-16-download.png',
			'/administrator/templates/hathor/images/menu/icon-16-edit.png',
			'/administrator/templates/hathor/images/menu/icon-16-featured.png',
			'/administrator/templates/hathor/images/menu/icon-16-frontpage.png',
			'/administrator/templates/hathor/images/menu/icon-16-generic.png',
			'/administrator/templates/hathor/images/menu/icon-16-groups.png',
			'/administrator/templates/hathor/images/menu/icon-16-help-community.png',
			'/administrator/templates/hathor/images/menu/icon-16-help-dev.png',
			'/administrator/templates/hathor/images/menu/icon-16-help-docs.png',
			'/administrator/templates/hathor/images/menu/icon-16-help-forum.png',
			'/administrator/templates/hathor/images/menu/icon-16-help-jed.png',
			'/administrator/templates/hathor/images/menu/icon-16-help-jrd.png',
			'/administrator/templates/hathor/images/menu/icon-16-help-security.png',
			'/administrator/templates/hathor/images/menu/icon-16-help-shop.png',
			'/administrator/templates/hathor/images/menu/icon-16-help-this.png',
			'/administrator/templates/hathor/images/menu/icon-16-help-trans.png',
			'/administrator/templates/hathor/images/menu/icon-16-help.png',
			'/administrator/templates/hathor/images/menu/icon-16-inbox.png',
			'/administrator/templates/hathor/images/menu/icon-16-info.png',
			'/administrator/templates/hathor/images/menu/icon-16-install.png',
			'/administrator/templates/hathor/images/menu/icon-16-language.png',
			'/administrator/templates/hathor/images/menu/icon-16-levels.png',
			'/administrator/templates/hathor/images/menu/icon-16-links-cat.png',
			'/administrator/templates/hathor/images/menu/icon-16-links.png',
			'/administrator/templates/hathor/images/menu/icon-16-logout.png',
			'/administrator/templates/hathor/images/menu/icon-16-maintenance.png',
			'/administrator/templates/hathor/images/menu/icon-16-massmail.png',
			'/administrator/templates/hathor/images/menu/icon-16-media.png',
			'/administrator/templates/hathor/images/menu/icon-16-menu.png',
			'/administrator/templates/hathor/images/menu/icon-16-menumgr.png',
			'/administrator/templates/hathor/images/menu/icon-16-messages.png',
			'/administrator/templates/hathor/images/menu/icon-16-messaging.png',
			'/administrator/templates/hathor/images/menu/icon-16-module.png',
			'/administrator/templates/hathor/images/menu/icon-16-move.png',
			'/administrator/templates/hathor/images/menu/icon-16-new-privatemessage.png',
			'/administrator/templates/hathor/images/menu/icon-16-new.png',
			'/administrator/templates/hathor/images/menu/icon-16-newarticle.png',
			'/administrator/templates/hathor/images/menu/icon-16-newcategory.png',
			'/administrator/templates/hathor/images/menu/icon-16-newgroup.png',
			'/administrator/templates/hathor/images/menu/icon-16-newlevel.png',
			'/administrator/templates/hathor/images/menu/icon-16-newsfeeds-cat.png',
			'/administrator/templates/hathor/images/menu/icon-16-newsfeeds.png',
			'/administrator/templates/hathor/images/menu/icon-16-newuser.png',
			'/administrator/templates/hathor/images/menu/icon-16-nopreview.png',
			'/administrator/templates/hathor/images/menu/icon-16-notdefault.png',
			'/administrator/templates/hathor/images/menu/icon-16-notice.png',
			'/administrator/templates/hathor/images/menu/icon-16-plugin.png',
			'/administrator/templates/hathor/images/menu/icon-16-preview.png',
			'/administrator/templates/hathor/images/menu/icon-16-print.png',
			'/administrator/templates/hathor/images/menu/icon-16-purge.png',
			'/administrator/templates/hathor/images/menu/icon-16-puzzle.png',
			'/administrator/templates/hathor/images/menu/icon-16-read-privatemessage.png',
			'/administrator/templates/hathor/images/menu/icon-16-readmess.png',
			'/administrator/templates/hathor/images/menu/icon-16-redirect.png',
			'/administrator/templates/hathor/images/menu/icon-16-revert.png',
			'/administrator/templates/hathor/images/menu/icon-16-search.png',
			'/administrator/templates/hathor/images/menu/icon-16-send.png',
			'/administrator/templates/hathor/images/menu/icon-16-stats.png',
			'/administrator/templates/hathor/images/menu/icon-16-tags.png',
			'/administrator/templates/hathor/images/menu/icon-16-themes.png',
			'/administrator/templates/hathor/images/menu/icon-16-trash.png',
			'/administrator/templates/hathor/images/menu/icon-16-unarticle.png',
			'/administrator/templates/hathor/images/menu/icon-16-upload.png',
			'/administrator/templates/hathor/images/menu/icon-16-user-dd.png',
			'/administrator/templates/hathor/images/menu/icon-16-user-note.png',
			'/administrator/templates/hathor/images/menu/icon-16-user.png',
			'/administrator/templates/hathor/images/menu/icon-16-viewsite.png',
			'/administrator/templates/hathor/images/menu/icon-16-writemess.png',
			'/administrator/templates/hathor/images/mini_icon.png',
			'/administrator/templates/hathor/images/notice-alert.png',
			'/administrator/templates/hathor/images/notice-info.png',
			'/administrator/templates/hathor/images/notice-note.png',
			'/administrator/templates/hathor/images/required.png',
			'/administrator/templates/hathor/images/selector-arrow-hc.png',
			'/administrator/templates/hathor/images/selector-arrow-rtl.png',
			'/administrator/templates/hathor/images/selector-arrow-std.png',
			'/administrator/templates/hathor/images/selector-arrow.png',
			'/administrator/templates/hathor/images/system/calendar.png',
			'/administrator/templates/hathor/images/system/selector-arrow.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-adduser.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-alert.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-apply.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-archive.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-article-add.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-article.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-back.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-banner-categories.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-banner-client.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-banner-tracks.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-banner.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-batch.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-calendar.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-cancel.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-checkin.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-cog.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-component.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-config.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-contacts-categories.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-contacts.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-copy.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-css.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-default.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-delete-style.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-delete.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-deny.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-download.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-edit.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-error.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-export.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-extension.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-featured.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-forward.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-help.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-html.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-inbox.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-info.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-links.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-lock.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-menu.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-messaging.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-messanging.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-module.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-move.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-new-privatemessage.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-new-style.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-new.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-notice.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-preview.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-print.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-publish.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-purge.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-read-privatemessage.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-refresh.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-remove.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-revert.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-save-copy.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-save-new.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-save.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-search.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-send.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-stats.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-trash.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-unarchive.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-unblock.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-unpublish.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-upload.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-user-add.png',
			'/administrator/templates/hathor/images/toolbar/icon-32-xml.png',
			'/administrator/templates/hathor/index.php',
			'/administrator/templates/hathor/js/template.js',
			'/administrator/templates/hathor/language/en-GB/en-GB.tpl_hathor.ini',
			'/administrator/templates/hathor/language/en-GB/en-GB.tpl_hathor.sys.ini',
			'/administrator/templates/hathor/less/buttons.less',
			'/administrator/templates/hathor/less/colour_baseline.less',
			'/administrator/templates/hathor/less/colour_blue.less',
			'/administrator/templates/hathor/less/colour_brown.less',
			'/administrator/templates/hathor/less/colour_standard.less',
			'/administrator/templates/hathor/less/forms.less',
			'/administrator/templates/hathor/less/hathor_variables.less',
			'/administrator/templates/hathor/less/icomoon.less',
			'/administrator/templates/hathor/less/modals.less',
			'/administrator/templates/hathor/less/template.less',
			'/administrator/templates/hathor/less/variables.less',
			'/administrator/templates/hathor/login.php',
			'/administrator/templates/hathor/postinstall/hathormessage.php',
			'/administrator/templates/hathor/templateDetails.xml',
			'/administrator/templates/hathor/template_preview.png',
			'/administrator/templates/hathor/template_thumbnail.png',
			'/administrator/templates/isis/component.php',
			'/administrator/templates/isis/cpanel.php',
			'/administrator/templates/isis/css/template-rtl.css',
			'/administrator/templates/isis/css/template.css',
			'/administrator/templates/isis/error.php',
			'/administrator/templates/isis/favicon.ico',
			'/administrator/templates/isis/html/com_media/imageslist/default_folder.php',
			'/administrator/templates/isis/html/com_media/imageslist/default_image.php',
			'/administrator/templates/isis/html/com_media/medialist/thumbs_folders.php',
			'/administrator/templates/isis/html/com_media/medialist/thumbs_imgs.php',
			'/administrator/templates/isis/html/editor_content.css',
			'/administrator/templates/isis/html/layouts/joomla/form/field/media.php',
			'/administrator/templates/isis/html/layouts/joomla/form/field/user.php',
			'/administrator/templates/isis/html/layouts/joomla/pagination/link.php',
			'/administrator/templates/isis/html/layouts/joomla/pagination/links.php',
			'/administrator/templates/isis/html/layouts/joomla/system/message.php',
			'/administrator/templates/isis/html/layouts/joomla/toolbar/versions.php',
			'/administrator/templates/isis/html/mod_version/default.php',
			'/administrator/templates/isis/html/modules.php',
			'/administrator/templates/isis/html/pagination.php',
			'/administrator/templates/isis/images/admin/blank.png',
			'/administrator/templates/isis/images/admin/checked_out.png',
			'/administrator/templates/isis/images/admin/collapseall.png',
			'/administrator/templates/isis/images/admin/disabled.png',
			'/administrator/templates/isis/images/admin/downarrow-1.png',
			'/administrator/templates/isis/images/admin/downarrow.png',
			'/administrator/templates/isis/images/admin/downarrow0.png',
			'/administrator/templates/isis/images/admin/expandall.png',
			'/administrator/templates/isis/images/admin/featured.png',
			'/administrator/templates/isis/images/admin/filesave.png',
			'/administrator/templates/isis/images/admin/filter_16.png',
			'/administrator/templates/isis/images/admin/icon-16-add.png',
			'/administrator/templates/isis/images/admin/icon-16-allow.png',
			'/administrator/templates/isis/images/admin/icon-16-allowinactive.png',
			'/administrator/templates/isis/images/admin/icon-16-deny.png',
			'/administrator/templates/isis/images/admin/icon-16-denyinactive.png',
			'/administrator/templates/isis/images/admin/icon-16-links.png',
			'/administrator/templates/isis/images/admin/icon-16-notice-note.png',
			'/administrator/templates/isis/images/admin/icon-16-protected.png',
			'/administrator/templates/isis/images/admin/menu_divider.png',
			'/administrator/templates/isis/images/admin/note_add_16.png',
			'/administrator/templates/isis/images/admin/publish_g.png',
			'/administrator/templates/isis/images/admin/publish_r.png',
			'/administrator/templates/isis/images/admin/publish_x.png',
			'/administrator/templates/isis/images/admin/publish_y.png',
			'/administrator/templates/isis/images/admin/sort_asc.png',
			'/administrator/templates/isis/images/admin/sort_desc.png',
			'/administrator/templates/isis/images/admin/tick.png',
			'/administrator/templates/isis/images/admin/trash.png',
			'/administrator/templates/isis/images/admin/uparrow-1.png',
			'/administrator/templates/isis/images/admin/uparrow.png',
			'/administrator/templates/isis/images/admin/uparrow0.png',
			'/administrator/templates/isis/images/emailButton.png',
			'/administrator/templates/isis/images/joomla.png',
			'/administrator/templates/isis/images/login-joomla-inverse.png',
			'/administrator/templates/isis/images/login-joomla.png',
			'/administrator/templates/isis/images/logo-inverse.png',
			'/administrator/templates/isis/images/logo.png',
			'/administrator/templates/isis/images/pdf_button.png',
			'/administrator/templates/isis/images/printButton.png',
			'/administrator/templates/isis/images/system/sort_asc.png',
			'/administrator/templates/isis/images/system/sort_desc.png',
			'/administrator/templates/isis/img/glyphicons-halflings-white.png',
			'/administrator/templates/isis/img/glyphicons-halflings.png',
			'/administrator/templates/isis/index.php',
			'/administrator/templates/isis/js/application.js',
			'/administrator/templates/isis/js/classes.js',
			'/administrator/templates/isis/js/template.js',
			'/administrator/templates/isis/language/en-GB/en-GB.tpl_isis.ini',
			'/administrator/templates/isis/language/en-GB/en-GB.tpl_isis.sys.ini',
			'/administrator/templates/isis/less/blocks/_chzn-override.less',
			'/administrator/templates/isis/less/blocks/_custom.less',
			'/administrator/templates/isis/less/blocks/_editors.less',
			'/administrator/templates/isis/less/blocks/_forms.less',
			'/administrator/templates/isis/less/blocks/_global.less',
			'/administrator/templates/isis/less/blocks/_header.less',
			'/administrator/templates/isis/less/blocks/_login.less',
			'/administrator/templates/isis/less/blocks/_media.less',
			'/administrator/templates/isis/less/blocks/_modals.less',
			'/administrator/templates/isis/less/blocks/_navbar.less',
			'/administrator/templates/isis/less/blocks/_quickicons.less',
			'/administrator/templates/isis/less/blocks/_sidebar.less',
			'/administrator/templates/isis/less/blocks/_status.less',
			'/administrator/templates/isis/less/blocks/_tables.less',
			'/administrator/templates/isis/less/blocks/_toolbar.less',
			'/administrator/templates/isis/less/blocks/_treeselect.less',
			'/administrator/templates/isis/less/blocks/_utility-classes.less',
			'/administrator/templates/isis/less/bootstrap/button-groups.less',
			'/administrator/templates/isis/less/bootstrap/buttons.less',
			'/administrator/templates/isis/less/bootstrap/mixins.less',
			'/administrator/templates/isis/less/bootstrap/responsive-1200px-min.less',
			'/administrator/templates/isis/less/bootstrap/responsive-768px-979px.less',
			'/administrator/templates/isis/less/bootstrap/wells.less',
			'/administrator/templates/isis/less/icomoon.less',
			'/administrator/templates/isis/less/pages/_com_cpanel.less',
			'/administrator/templates/isis/less/pages/_com_postinstall.less',
			'/administrator/templates/isis/less/pages/_com_privacy.less',
			'/administrator/templates/isis/less/pages/_com_templates.less',
			'/administrator/templates/isis/less/template-rtl.less',
			'/administrator/templates/isis/less/template.less',
			'/administrator/templates/isis/less/variables.less',
			'/administrator/templates/isis/login.php',
			'/administrator/templates/isis/templateDetails.xml',
			'/administrator/templates/isis/template_preview.png',
			'/administrator/templates/isis/template_thumbnail.png',
			'/administrator/templates/system/html/modules.php',
			'/bin/index.html',
			'/bin/keychain.php',
			'/cli/deletefiles.php',
			'/cli/garbagecron.php',
			'/cli/sessionGc.php',
			'/cli/sessionMetadataGc.php',
			'/cli/update_cron.php',
			'/components/com_banners/banners.php',
			'/components/com_banners/controller.php',
			'/components/com_banners/helpers/banner.php',
			'/components/com_banners/helpers/category.php',
			'/components/com_banners/models/banner.php',
			'/components/com_banners/models/banners.php',
			'/components/com_banners/router.php',
			'/components/com_config/config.php',
			'/components/com_config/controller/cancel.php',
			'/components/com_config/controller/canceladmin.php',
			'/components/com_config/controller/cmsbase.php',
			'/components/com_config/controller/config/display.php',
			'/components/com_config/controller/config/save.php',
			'/components/com_config/controller/display.php',
			'/components/com_config/controller/helper.php',
			'/components/com_config/controller/modules/cancel.php',
			'/components/com_config/controller/modules/display.php',
			'/components/com_config/controller/modules/save.php',
			'/components/com_config/controller/templates/display.php',
			'/components/com_config/controller/templates/save.php',
			'/components/com_config/model/cms.php',
			'/components/com_config/model/config.php',
			'/components/com_config/model/form.php',
			'/components/com_config/model/form/config.xml',
			'/components/com_config/model/form/modules.xml',
			'/components/com_config/model/form/modules_advanced.xml',
			'/components/com_config/model/form/templates.xml',
			'/components/com_config/model/modules.php',
			'/components/com_config/model/templates.php',
			'/components/com_config/view/cms/html.php',
			'/components/com_config/view/cms/json.php',
			'/components/com_config/view/config/html.php',
			'/components/com_config/view/config/tmpl/default.php',
			'/components/com_config/view/config/tmpl/default.xml',
			'/components/com_config/view/config/tmpl/default_metadata.php',
			'/components/com_config/view/config/tmpl/default_seo.php',
			'/components/com_config/view/config/tmpl/default_site.php',
			'/components/com_config/view/modules/html.php',
			'/components/com_config/view/modules/tmpl/default.php',
			'/components/com_config/view/modules/tmpl/default_options.php',
			'/components/com_config/view/modules/tmpl/default_positions.php',
			'/components/com_config/view/templates/html.php',
			'/components/com_config/view/templates/tmpl/default.php',
			'/components/com_config/view/templates/tmpl/default.xml',
			'/components/com_config/view/templates/tmpl/default_options.php',
			'/components/com_contact/contact.php',
			'/components/com_contact/controller.php',
			'/components/com_contact/controllers/contact.php',
			'/components/com_contact/helpers/association.php',
			'/components/com_contact/helpers/category.php',
			'/components/com_contact/helpers/legacyrouter.php',
			'/components/com_contact/models/categories.php',
			'/components/com_contact/models/category.php',
			'/components/com_contact/models/contact.php',
			'/components/com_contact/models/featured.php',
			'/components/com_contact/models/forms/contact.xml',
			'/components/com_contact/models/forms/filter_contacts.xml',
			'/components/com_contact/models/forms/form.xml',
			'/components/com_contact/models/rules/contactemail.php',
			'/components/com_contact/models/rules/contactemailmessage.php',
			'/components/com_contact/models/rules/contactemailsubject.php',
			'/components/com_contact/router.php',
			'/components/com_contact/views/categories/tmpl/default.php',
			'/components/com_contact/views/categories/tmpl/default.xml',
			'/components/com_contact/views/categories/tmpl/default_items.php',
			'/components/com_contact/views/categories/view.html.php',
			'/components/com_contact/views/category/tmpl/default.php',
			'/components/com_contact/views/category/tmpl/default.xml',
			'/components/com_contact/views/category/tmpl/default_children.php',
			'/components/com_contact/views/category/tmpl/default_items.php',
			'/components/com_contact/views/category/view.feed.php',
			'/components/com_contact/views/category/view.html.php',
			'/components/com_contact/views/contact/tmpl/default.php',
			'/components/com_contact/views/contact/tmpl/default.xml',
			'/components/com_contact/views/contact/tmpl/default_address.php',
			'/components/com_contact/views/contact/tmpl/default_articles.php',
			'/components/com_contact/views/contact/tmpl/default_form.php',
			'/components/com_contact/views/contact/tmpl/default_links.php',
			'/components/com_contact/views/contact/tmpl/default_profile.php',
			'/components/com_contact/views/contact/tmpl/default_user_custom_fields.php',
			'/components/com_contact/views/contact/view.html.php',
			'/components/com_contact/views/contact/view.vcf.php',
			'/components/com_contact/views/featured/tmpl/default.php',
			'/components/com_contact/views/featured/tmpl/default.xml',
			'/components/com_contact/views/featured/tmpl/default_items.php',
			'/components/com_contact/views/featured/view.html.php',
			'/components/com_content/content.php',
			'/components/com_content/controller.php',
			'/components/com_content/controllers/article.php',
			'/components/com_content/helpers/association.php',
			'/components/com_content/helpers/category.php',
			'/components/com_content/helpers/legacyrouter.php',
			'/components/com_content/helpers/query.php',
			'/components/com_content/helpers/route.php',
			'/components/com_content/models/archive.php',
			'/components/com_content/models/article.php',
			'/components/com_content/models/articles.php',
			'/components/com_content/models/categories.php',
			'/components/com_content/models/category.php',
			'/components/com_content/models/featured.php',
			'/components/com_content/models/form.php',
			'/components/com_content/models/forms/article.xml',
			'/components/com_content/models/forms/filter_articles.xml',
			'/components/com_content/router.php',
			'/components/com_content/views/archive/tmpl/default.php',
			'/components/com_content/views/archive/tmpl/default.xml',
			'/components/com_content/views/archive/tmpl/default_items.php',
			'/components/com_content/views/archive/view.html.php',
			'/components/com_content/views/article/tmpl/default.php',
			'/components/com_content/views/article/tmpl/default.xml',
			'/components/com_content/views/article/tmpl/default_links.php',
			'/components/com_content/views/article/view.html.php',
			'/components/com_content/views/categories/tmpl/default.php',
			'/components/com_content/views/categories/tmpl/default.xml',
			'/components/com_content/views/categories/tmpl/default_items.php',
			'/components/com_content/views/categories/view.html.php',
			'/components/com_content/views/category/tmpl/blog.php',
			'/components/com_content/views/category/tmpl/blog.xml',
			'/components/com_content/views/category/tmpl/blog_children.php',
			'/components/com_content/views/category/tmpl/blog_item.php',
			'/components/com_content/views/category/tmpl/blog_links.php',
			'/components/com_content/views/category/tmpl/default.php',
			'/components/com_content/views/category/tmpl/default.xml',
			'/components/com_content/views/category/tmpl/default_articles.php',
			'/components/com_content/views/category/tmpl/default_children.php',
			'/components/com_content/views/category/view.feed.php',
			'/components/com_content/views/category/view.html.php',
			'/components/com_content/views/featured/tmpl/default.php',
			'/components/com_content/views/featured/tmpl/default.xml',
			'/components/com_content/views/featured/tmpl/default_item.php',
			'/components/com_content/views/featured/tmpl/default_links.php',
			'/components/com_content/views/featured/view.feed.php',
			'/components/com_content/views/featured/view.html.php',
			'/components/com_content/views/form/tmpl/edit.php',
			'/components/com_content/views/form/tmpl/edit.xml',
			'/components/com_content/views/form/view.html.php',
			'/components/com_contenthistory/contenthistory.php',
			'/components/com_fields/controller.php',
			'/components/com_fields/fields.php',
			'/components/com_fields/models/forms/filter_fields.xml',
			'/components/com_finder/controller.php',
			'/components/com_finder/controllers/suggestions.json.php',
			'/components/com_finder/finder.php',
			'/components/com_finder/helpers/html/filter.php',
			'/components/com_finder/helpers/html/query.php',
			'/components/com_finder/models/search.php',
			'/components/com_finder/models/suggestions.php',
			'/components/com_finder/router.php',
			'/components/com_finder/views/search/tmpl/default.php',
			'/components/com_finder/views/search/tmpl/default.xml',
			'/components/com_finder/views/search/tmpl/default_form.php',
			'/components/com_finder/views/search/tmpl/default_result.php',
			'/components/com_finder/views/search/tmpl/default_results.php',
			'/components/com_finder/views/search/view.feed.php',
			'/components/com_finder/views/search/view.html.php',
			'/components/com_finder/views/search/view.opensearch.php',
			'/components/com_mailto/controller.php',
			'/components/com_mailto/helpers/mailto.php',
			'/components/com_mailto/mailto.php',
			'/components/com_mailto/mailto.xml',
			'/components/com_mailto/models/forms/mailto.xml',
			'/components/com_mailto/models/mailto.php',
			'/components/com_mailto/views/mailto/tmpl/default.php',
			'/components/com_mailto/views/mailto/view.html.php',
			'/components/com_mailto/views/sent/tmpl/default.php',
			'/components/com_mailto/views/sent/view.html.php',
			'/components/com_media/media.php',
			'/components/com_menus/controller.php',
			'/components/com_menus/menus.php',
			'/components/com_menus/models/forms/filter_items.xml',
			'/components/com_modules/controller.php',
			'/components/com_modules/models/forms/filter_modules.xml',
			'/components/com_modules/modules.php',
			'/components/com_newsfeeds/controller.php',
			'/components/com_newsfeeds/helpers/association.php',
			'/components/com_newsfeeds/helpers/category.php',
			'/components/com_newsfeeds/helpers/legacyrouter.php',
			'/components/com_newsfeeds/models/categories.php',
			'/components/com_newsfeeds/models/category.php',
			'/components/com_newsfeeds/models/newsfeed.php',
			'/components/com_newsfeeds/newsfeeds.php',
			'/components/com_newsfeeds/router.php',
			'/components/com_newsfeeds/views/categories/tmpl/default.php',
			'/components/com_newsfeeds/views/categories/tmpl/default.xml',
			'/components/com_newsfeeds/views/categories/tmpl/default_items.php',
			'/components/com_newsfeeds/views/categories/view.html.php',
			'/components/com_newsfeeds/views/category/tmpl/default.php',
			'/components/com_newsfeeds/views/category/tmpl/default.xml',
			'/components/com_newsfeeds/views/category/tmpl/default_children.php',
			'/components/com_newsfeeds/views/category/tmpl/default_items.php',
			'/components/com_newsfeeds/views/category/view.html.php',
			'/components/com_newsfeeds/views/newsfeed/tmpl/default.php',
			'/components/com_newsfeeds/views/newsfeed/tmpl/default.xml',
			'/components/com_newsfeeds/views/newsfeed/view.html.php',
			'/components/com_privacy/controller.php',
			'/components/com_privacy/controllers/request.php',
			'/components/com_privacy/models/confirm.php',
			'/components/com_privacy/models/forms/confirm.xml',
			'/components/com_privacy/models/forms/remind.xml',
			'/components/com_privacy/models/forms/request.xml',
			'/components/com_privacy/models/remind.php',
			'/components/com_privacy/models/request.php',
			'/components/com_privacy/privacy.php',
			'/components/com_privacy/router.php',
			'/components/com_privacy/views/confirm/tmpl/default.php',
			'/components/com_privacy/views/confirm/tmpl/default.xml',
			'/components/com_privacy/views/confirm/view.html.php',
			'/components/com_privacy/views/remind/tmpl/default.php',
			'/components/com_privacy/views/remind/tmpl/default.xml',
			'/components/com_privacy/views/remind/view.html.php',
			'/components/com_privacy/views/request/tmpl/default.php',
			'/components/com_privacy/views/request/tmpl/default.xml',
			'/components/com_privacy/views/request/view.html.php',
			'/components/com_tags/controller.php',
			'/components/com_tags/controllers/tags.php',
			'/components/com_tags/models/tag.php',
			'/components/com_tags/models/tags.php',
			'/components/com_tags/router.php',
			'/components/com_tags/tags.php',
			'/components/com_tags/views/tag/tmpl/default.php',
			'/components/com_tags/views/tag/tmpl/default.xml',
			'/components/com_tags/views/tag/tmpl/default_items.php',
			'/components/com_tags/views/tag/tmpl/list.php',
			'/components/com_tags/views/tag/tmpl/list.xml',
			'/components/com_tags/views/tag/tmpl/list_items.php',
			'/components/com_tags/views/tag/view.feed.php',
			'/components/com_tags/views/tag/view.html.php',
			'/components/com_tags/views/tags/tmpl/default.php',
			'/components/com_tags/views/tags/tmpl/default.xml',
			'/components/com_tags/views/tags/tmpl/default_items.php',
			'/components/com_tags/views/tags/view.feed.php',
			'/components/com_tags/views/tags/view.html.php',
			'/components/com_users/controller.php',
			'/components/com_users/controllers/profile.php',
			'/components/com_users/controllers/registration.php',
			'/components/com_users/controllers/remind.php',
			'/components/com_users/controllers/reset.php',
			'/components/com_users/controllers/user.php',
			'/components/com_users/helpers/html/users.php',
			'/components/com_users/helpers/legacyrouter.php',
			'/components/com_users/helpers/route.php',
			'/components/com_users/models/forms/frontend.xml',
			'/components/com_users/models/forms/frontend_admin.xml',
			'/components/com_users/models/forms/login.xml',
			'/components/com_users/models/forms/profile.xml',
			'/components/com_users/models/forms/registration.xml',
			'/components/com_users/models/forms/remind.xml',
			'/components/com_users/models/forms/reset_complete.xml',
			'/components/com_users/models/forms/reset_confirm.xml',
			'/components/com_users/models/forms/reset_request.xml',
			'/components/com_users/models/forms/sitelang.xml',
			'/components/com_users/models/login.php',
			'/components/com_users/models/profile.php',
			'/components/com_users/models/registration.php',
			'/components/com_users/models/remind.php',
			'/components/com_users/models/reset.php',
			'/components/com_users/models/rules/loginuniquefield.php',
			'/components/com_users/models/rules/logoutuniquefield.php',
			'/components/com_users/router.php',
			'/components/com_users/users.php',
			'/components/com_users/views/login/tmpl/default.php',
			'/components/com_users/views/login/tmpl/default.xml',
			'/components/com_users/views/login/tmpl/default_login.php',
			'/components/com_users/views/login/tmpl/default_logout.php',
			'/components/com_users/views/login/tmpl/logout.xml',
			'/components/com_users/views/login/view.html.php',
			'/components/com_users/views/profile/tmpl/default.php',
			'/components/com_users/views/profile/tmpl/default.xml',
			'/components/com_users/views/profile/tmpl/default_core.php',
			'/components/com_users/views/profile/tmpl/default_custom.php',
			'/components/com_users/views/profile/tmpl/default_params.php',
			'/components/com_users/views/profile/tmpl/edit.php',
			'/components/com_users/views/profile/tmpl/edit.xml',
			'/components/com_users/views/profile/view.html.php',
			'/components/com_users/views/registration/tmpl/complete.php',
			'/components/com_users/views/registration/tmpl/default.php',
			'/components/com_users/views/registration/tmpl/default.xml',
			'/components/com_users/views/registration/view.html.php',
			'/components/com_users/views/remind/tmpl/default.php',
			'/components/com_users/views/remind/tmpl/default.xml',
			'/components/com_users/views/remind/view.html.php',
			'/components/com_users/views/reset/tmpl/complete.php',
			'/components/com_users/views/reset/tmpl/confirm.php',
			'/components/com_users/views/reset/tmpl/default.php',
			'/components/com_users/views/reset/tmpl/default.xml',
			'/components/com_users/views/reset/view.html.php',
			'/components/com_wrapper/controller.php',
			'/components/com_wrapper/router.php',
			'/components/com_wrapper/views/wrapper/tmpl/default.php',
			'/components/com_wrapper/views/wrapper/tmpl/default.xml',
			'/components/com_wrapper/views/wrapper/view.html.php',
			'/components/com_wrapper/wrapper.php',
			'/components/com_wrapper/wrapper.xml',
			'/language/en-GB/en-GB.com_ajax.ini',
			'/language/en-GB/en-GB.com_config.ini',
			'/language/en-GB/en-GB.com_contact.ini',
			'/language/en-GB/en-GB.com_content.ini',
			'/language/en-GB/en-GB.com_finder.ini',
			'/language/en-GB/en-GB.com_mailto.ini',
			'/language/en-GB/en-GB.com_media.ini',
			'/language/en-GB/en-GB.com_messages.ini',
			'/language/en-GB/en-GB.com_newsfeeds.ini',
			'/language/en-GB/en-GB.com_privacy.ini',
			'/language/en-GB/en-GB.com_tags.ini',
			'/language/en-GB/en-GB.com_users.ini',
			'/language/en-GB/en-GB.com_weblinks.ini',
			'/language/en-GB/en-GB.com_wrapper.ini',
			'/language/en-GB/en-GB.files_joomla.sys.ini',
			'/language/en-GB/en-GB.finder_cli.ini',
			'/language/en-GB/en-GB.ini',
			'/language/en-GB/en-GB.lib_fof.ini',
			'/language/en-GB/en-GB.lib_fof.sys.ini',
			'/language/en-GB/en-GB.lib_idna_convert.sys.ini',
			'/language/en-GB/en-GB.lib_joomla.ini',
			'/language/en-GB/en-GB.lib_joomla.sys.ini',
			'/language/en-GB/en-GB.lib_phpass.sys.ini',
			'/language/en-GB/en-GB.lib_phputf8.sys.ini',
			'/language/en-GB/en-GB.lib_simplepie.sys.ini',
			'/language/en-GB/en-GB.localise.php',
			'/language/en-GB/en-GB.mod_articles_archive.ini',
			'/language/en-GB/en-GB.mod_articles_archive.sys.ini',
			'/language/en-GB/en-GB.mod_articles_categories.ini',
			'/language/en-GB/en-GB.mod_articles_categories.sys.ini',
			'/language/en-GB/en-GB.mod_articles_category.ini',
			'/language/en-GB/en-GB.mod_articles_category.sys.ini',
			'/language/en-GB/en-GB.mod_articles_latest.ini',
			'/language/en-GB/en-GB.mod_articles_latest.sys.ini',
			'/language/en-GB/en-GB.mod_articles_news.ini',
			'/language/en-GB/en-GB.mod_articles_news.sys.ini',
			'/language/en-GB/en-GB.mod_articles_popular.ini',
			'/language/en-GB/en-GB.mod_articles_popular.sys.ini',
			'/language/en-GB/en-GB.mod_banners.ini',
			'/language/en-GB/en-GB.mod_banners.sys.ini',
			'/language/en-GB/en-GB.mod_breadcrumbs.ini',
			'/language/en-GB/en-GB.mod_breadcrumbs.sys.ini',
			'/language/en-GB/en-GB.mod_custom.ini',
			'/language/en-GB/en-GB.mod_custom.sys.ini',
			'/language/en-GB/en-GB.mod_feed.ini',
			'/language/en-GB/en-GB.mod_feed.sys.ini',
			'/language/en-GB/en-GB.mod_finder.ini',
			'/language/en-GB/en-GB.mod_finder.sys.ini',
			'/language/en-GB/en-GB.mod_footer.ini',
			'/language/en-GB/en-GB.mod_footer.sys.ini',
			'/language/en-GB/en-GB.mod_languages.ini',
			'/language/en-GB/en-GB.mod_languages.sys.ini',
			'/language/en-GB/en-GB.mod_login.ini',
			'/language/en-GB/en-GB.mod_login.sys.ini',
			'/language/en-GB/en-GB.mod_menu.ini',
			'/language/en-GB/en-GB.mod_menu.sys.ini',
			'/language/en-GB/en-GB.mod_random_image.ini',
			'/language/en-GB/en-GB.mod_random_image.sys.ini',
			'/language/en-GB/en-GB.mod_related_items.ini',
			'/language/en-GB/en-GB.mod_related_items.sys.ini',
			'/language/en-GB/en-GB.mod_search.ini',
			'/language/en-GB/en-GB.mod_search.sys.ini',
			'/language/en-GB/en-GB.mod_stats.ini',
			'/language/en-GB/en-GB.mod_stats.sys.ini',
			'/language/en-GB/en-GB.mod_syndicate.ini',
			'/language/en-GB/en-GB.mod_syndicate.sys.ini',
			'/language/en-GB/en-GB.mod_tags_popular.ini',
			'/language/en-GB/en-GB.mod_tags_popular.sys.ini',
			'/language/en-GB/en-GB.mod_tags_similar.ini',
			'/language/en-GB/en-GB.mod_tags_similar.sys.ini',
			'/language/en-GB/en-GB.mod_users_latest.ini',
			'/language/en-GB/en-GB.mod_users_latest.sys.ini',
			'/language/en-GB/en-GB.mod_weblinks.ini',
			'/language/en-GB/en-GB.mod_weblinks.sys.ini',
			'/language/en-GB/en-GB.mod_whosonline.ini',
			'/language/en-GB/en-GB.mod_whosonline.sys.ini',
			'/language/en-GB/en-GB.mod_wrapper.ini',
			'/language/en-GB/en-GB.mod_wrapper.sys.ini',
			'/language/en-GB/en-GB.tpl_beez3.ini',
			'/language/en-GB/en-GB.tpl_beez3.sys.ini',
			'/language/en-GB/en-GB.tpl_protostar.ini',
			'/language/en-GB/en-GB.tpl_protostar.sys.ini',
			'/language/en-GB/en-GB.xml',
			'/layouts/joomla/content/icons/email.php',
			'/layouts/joomla/content/icons/print_popup.php',
			'/layouts/joomla/content/icons/print_screen.php',
			'/layouts/joomla/content/info_block/block.php',
			'/layouts/joomla/edit/details.php',
			'/layouts/joomla/edit/item_title.php',
			'/layouts/joomla/form/field/radio.php',
			'/layouts/joomla/html/formbehavior/ajaxchosen.php',
			'/layouts/joomla/html/formbehavior/chosen.php',
			'/layouts/joomla/html/sortablelist.php',
			'/layouts/joomla/html/tag.php',
			'/layouts/joomla/sidebars/toggle.php',
			'/layouts/joomla/tinymce/buttons.php',
			'/layouts/joomla/tinymce/buttons/button.php',
			'/layouts/joomla/toolbar/confirm.php',
			'/layouts/joomla/toolbar/help.php',
			'/layouts/joomla/toolbar/slider.php',
			'/layouts/libraries/cms/html/bootstrap/addtabscript.php',
			'/layouts/libraries/cms/html/bootstrap/starttabsetscript.php',
			'/libraries/cms/class/loader.php',
			'/libraries/cms/html/access.php',
			'/libraries/cms/html/actionsdropdown.php',
			'/libraries/cms/html/adminlanguage.php',
			'/libraries/cms/html/batch.php',
			'/libraries/cms/html/behavior.php',
			'/libraries/cms/html/bootstrap.php',
			'/libraries/cms/html/category.php',
			'/libraries/cms/html/content.php',
			'/libraries/cms/html/contentlanguage.php',
			'/libraries/cms/html/date.php',
			'/libraries/cms/html/debug.php',
			'/libraries/cms/html/dropdown.php',
			'/libraries/cms/html/email.php',
			'/libraries/cms/html/form.php',
			'/libraries/cms/html/formbehavior.php',
			'/libraries/cms/html/grid.php',
			'/libraries/cms/html/icons.php',
			'/libraries/cms/html/jgrid.php',
			'/libraries/cms/html/jquery.php',
			'/libraries/cms/html/language/en-GB/en-GB.jhtmldate.ini',
			'/libraries/cms/html/links.php',
			'/libraries/cms/html/list.php',
			'/libraries/cms/html/menu.php',
			'/libraries/cms/html/number.php',
			'/libraries/cms/html/rules.php',
			'/libraries/cms/html/searchtools.php',
			'/libraries/cms/html/select.php',
			'/libraries/cms/html/sidebar.php',
			'/libraries/cms/html/sliders.php',
			'/libraries/cms/html/sortablelist.php',
			'/libraries/cms/html/string.php',
			'/libraries/cms/html/tabs.php',
			'/libraries/cms/html/tag.php',
			'/libraries/cms/html/tel.php',
			'/libraries/cms/html/user.php',
			'/libraries/cms/less/formatter/joomla.php',
			'/libraries/cms/less/less.php',
			'/libraries/fof/LICENSE.txt',
			'/libraries/fof/autoloader/component.php',
			'/libraries/fof/autoloader/fof.php',
			'/libraries/fof/config/domain/dispatcher.php',
			'/libraries/fof/config/domain/interface.php',
			'/libraries/fof/config/domain/tables.php',
			'/libraries/fof/config/domain/views.php',
			'/libraries/fof/config/provider.php',
			'/libraries/fof/controller/controller.php',
			'/libraries/fof/database/database.php',
			'/libraries/fof/database/driver.php',
			'/libraries/fof/database/driver/joomla.php',
			'/libraries/fof/database/driver/mysql.php',
			'/libraries/fof/database/driver/mysqli.php',
			'/libraries/fof/database/driver/oracle.php',
			'/libraries/fof/database/driver/pdo.php',
			'/libraries/fof/database/driver/pdomysql.php',
			'/libraries/fof/database/driver/postgresql.php',
			'/libraries/fof/database/driver/sqlazure.php',
			'/libraries/fof/database/driver/sqlite.php',
			'/libraries/fof/database/driver/sqlsrv.php',
			'/libraries/fof/database/factory.php',
			'/libraries/fof/database/installer.php',
			'/libraries/fof/database/interface.php',
			'/libraries/fof/database/iterator.php',
			'/libraries/fof/database/iterator/azure.php',
			'/libraries/fof/database/iterator/mysql.php',
			'/libraries/fof/database/iterator/mysqli.php',
			'/libraries/fof/database/iterator/oracle.php',
			'/libraries/fof/database/iterator/pdo.php',
			'/libraries/fof/database/iterator/pdomysql.php',
			'/libraries/fof/database/iterator/postgresql.php',
			'/libraries/fof/database/iterator/sqlite.php',
			'/libraries/fof/database/iterator/sqlsrv.php',
			'/libraries/fof/database/query.php',
			'/libraries/fof/database/query/element.php',
			'/libraries/fof/database/query/limitable.php',
			'/libraries/fof/database/query/mysql.php',
			'/libraries/fof/database/query/mysqli.php',
			'/libraries/fof/database/query/oracle.php',
			'/libraries/fof/database/query/pdo.php',
			'/libraries/fof/database/query/pdomysql.php',
			'/libraries/fof/database/query/postgresql.php',
			'/libraries/fof/database/query/preparable.php',
			'/libraries/fof/database/query/sqlazure.php',
			'/libraries/fof/database/query/sqlite.php',
			'/libraries/fof/database/query/sqlsrv.php',
			'/libraries/fof/dispatcher/dispatcher.php',
			'/libraries/fof/download/adapter/abstract.php',
			'/libraries/fof/download/adapter/cacert.pem',
			'/libraries/fof/download/adapter/curl.php',
			'/libraries/fof/download/adapter/fopen.php',
			'/libraries/fof/download/download.php',
			'/libraries/fof/download/interface.php',
			'/libraries/fof/encrypt/aes.php',
			'/libraries/fof/encrypt/aes/abstract.php',
			'/libraries/fof/encrypt/aes/interface.php',
			'/libraries/fof/encrypt/aes/mcrypt.php',
			'/libraries/fof/encrypt/aes/openssl.php',
			'/libraries/fof/encrypt/base32.php',
			'/libraries/fof/encrypt/randval.php',
			'/libraries/fof/encrypt/randvalinterface.php',
			'/libraries/fof/encrypt/totp.php',
			'/libraries/fof/form/field.php',
			'/libraries/fof/form/field/accesslevel.php',
			'/libraries/fof/form/field/actions.php',
			'/libraries/fof/form/field/button.php',
			'/libraries/fof/form/field/cachehandler.php',
			'/libraries/fof/form/field/calendar.php',
			'/libraries/fof/form/field/captcha.php',
			'/libraries/fof/form/field/checkbox.php',
			'/libraries/fof/form/field/checkboxes.php',
			'/libraries/fof/form/field/components.php',
			'/libraries/fof/form/field/editor.php',
			'/libraries/fof/form/field/email.php',
			'/libraries/fof/form/field/groupedbutton.php',
			'/libraries/fof/form/field/groupedlist.php',
			'/libraries/fof/form/field/hidden.php',
			'/libraries/fof/form/field/image.php',
			'/libraries/fof/form/field/imagelist.php',
			'/libraries/fof/form/field/integer.php',
			'/libraries/fof/form/field/language.php',
			'/libraries/fof/form/field/list.php',
			'/libraries/fof/form/field/media.php',
			'/libraries/fof/form/field/model.php',
			'/libraries/fof/form/field/ordering.php',
			'/libraries/fof/form/field/password.php',
			'/libraries/fof/form/field/plugins.php',
			'/libraries/fof/form/field/published.php',
			'/libraries/fof/form/field/radio.php',
			'/libraries/fof/form/field/relation.php',
			'/libraries/fof/form/field/rules.php',
			'/libraries/fof/form/field/selectrow.php',
			'/libraries/fof/form/field/sessionhandler.php',
			'/libraries/fof/form/field/spacer.php',
			'/libraries/fof/form/field/sql.php',
			'/libraries/fof/form/field/tag.php',
			'/libraries/fof/form/field/tel.php',
			'/libraries/fof/form/field/text.php',
			'/libraries/fof/form/field/textarea.php',
			'/libraries/fof/form/field/timezone.php',
			'/libraries/fof/form/field/title.php',
			'/libraries/fof/form/field/url.php',
			'/libraries/fof/form/field/user.php',
			'/libraries/fof/form/field/usergroup.php',
			'/libraries/fof/form/form.php',
			'/libraries/fof/form/header.php',
			'/libraries/fof/form/header/accesslevel.php',
			'/libraries/fof/form/header/field.php',
			'/libraries/fof/form/header/fielddate.php',
			'/libraries/fof/form/header/fieldfilterable.php',
			'/libraries/fof/form/header/fieldsearchable.php',
			'/libraries/fof/form/header/fieldselectable.php',
			'/libraries/fof/form/header/fieldsql.php',
			'/libraries/fof/form/header/filterdate.php',
			'/libraries/fof/form/header/filterfilterable.php',
			'/libraries/fof/form/header/filtersearchable.php',
			'/libraries/fof/form/header/filterselectable.php',
			'/libraries/fof/form/header/filtersql.php',
			'/libraries/fof/form/header/language.php',
			'/libraries/fof/form/header/model.php',
			'/libraries/fof/form/header/ordering.php',
			'/libraries/fof/form/header/published.php',
			'/libraries/fof/form/header/rowselect.php',
			'/libraries/fof/form/helper.php',
			'/libraries/fof/hal/document.php',
			'/libraries/fof/hal/link.php',
			'/libraries/fof/hal/links.php',
			'/libraries/fof/hal/render/interface.php',
			'/libraries/fof/hal/render/json.php',
			'/libraries/fof/include.php',
			'/libraries/fof/inflector/inflector.php',
			'/libraries/fof/input/input.php',
			'/libraries/fof/input/jinput/cli.php',
			'/libraries/fof/input/jinput/cookie.php',
			'/libraries/fof/input/jinput/files.php',
			'/libraries/fof/input/jinput/input.php',
			'/libraries/fof/input/jinput/json.php',
			'/libraries/fof/integration/joomla/filesystem/filesystem.php',
			'/libraries/fof/integration/joomla/platform.php',
			'/libraries/fof/layout/file.php',
			'/libraries/fof/layout/helper.php',
			'/libraries/fof/less/formatter/classic.php',
			'/libraries/fof/less/formatter/compressed.php',
			'/libraries/fof/less/formatter/joomla.php',
			'/libraries/fof/less/formatter/lessjs.php',
			'/libraries/fof/less/less.php',
			'/libraries/fof/less/parser/parser.php',
			'/libraries/fof/model/behavior.php',
			'/libraries/fof/model/behavior/access.php',
			'/libraries/fof/model/behavior/emptynonzero.php',
			'/libraries/fof/model/behavior/enabled.php',
			'/libraries/fof/model/behavior/filters.php',
			'/libraries/fof/model/behavior/language.php',
			'/libraries/fof/model/behavior/private.php',
			'/libraries/fof/model/dispatcher/behavior.php',
			'/libraries/fof/model/field.php',
			'/libraries/fof/model/field/boolean.php',
			'/libraries/fof/model/field/date.php',
			'/libraries/fof/model/field/number.php',
			'/libraries/fof/model/field/text.php',
			'/libraries/fof/model/model.php',
			'/libraries/fof/platform/filesystem/filesystem.php',
			'/libraries/fof/platform/filesystem/interface.php',
			'/libraries/fof/platform/interface.php',
			'/libraries/fof/platform/platform.php',
			'/libraries/fof/query/abstract.php',
			'/libraries/fof/render/abstract.php',
			'/libraries/fof/render/joomla.php',
			'/libraries/fof/render/joomla3.php',
			'/libraries/fof/render/strapper.php',
			'/libraries/fof/string/utils.php',
			'/libraries/fof/table/behavior.php',
			'/libraries/fof/table/behavior/assets.php',
			'/libraries/fof/table/behavior/contenthistory.php',
			'/libraries/fof/table/behavior/tags.php',
			'/libraries/fof/table/dispatcher/behavior.php',
			'/libraries/fof/table/nested.php',
			'/libraries/fof/table/relations.php',
			'/libraries/fof/table/table.php',
			'/libraries/fof/template/utils.php',
			'/libraries/fof/toolbar/toolbar.php',
			'/libraries/fof/utils/array/array.php',
			'/libraries/fof/utils/cache/cleaner.php',
			'/libraries/fof/utils/config/helper.php',
			'/libraries/fof/utils/filescheck/filescheck.php',
			'/libraries/fof/utils/ini/parser.php',
			'/libraries/fof/utils/installscript/installscript.php',
			'/libraries/fof/utils/ip/ip.php',
			'/libraries/fof/utils/object/object.php',
			'/libraries/fof/utils/observable/dispatcher.php',
			'/libraries/fof/utils/observable/event.php',
			'/libraries/fof/utils/phpfunc/phpfunc.php',
			'/libraries/fof/utils/timer/timer.php',
			'/libraries/fof/utils/update/collection.php',
			'/libraries/fof/utils/update/extension.php',
			'/libraries/fof/utils/update/joomla.php',
			'/libraries/fof/utils/update/update.php',
			'/libraries/fof/version.txt',
			'/libraries/fof/view/csv.php',
			'/libraries/fof/view/form.php',
			'/libraries/fof/view/html.php',
			'/libraries/fof/view/json.php',
			'/libraries/fof/view/raw.php',
			'/libraries/fof/view/view.php',
			'/libraries/idna_convert/LICENCE',
			'/libraries/idna_convert/ReadMe.txt',
			'/libraries/idna_convert/idna_convert.class.php',
			'/libraries/idna_convert/transcode_wrapper.php',
			'/libraries/idna_convert/uctc.php',
			'/libraries/joomla/application/web/router.php',
			'/libraries/joomla/application/web/router/base.php',
			'/libraries/joomla/application/web/router/rest.php',
			'/libraries/joomla/archive/archive.php',
			'/libraries/joomla/archive/bzip2.php',
			'/libraries/joomla/archive/extractable.php',
			'/libraries/joomla/archive/gzip.php',
			'/libraries/joomla/archive/tar.php',
			'/libraries/joomla/archive/wrapper/archive.php',
			'/libraries/joomla/archive/zip.php',
			'/libraries/joomla/controller/base.php',
			'/libraries/joomla/controller/controller.php',
			'/libraries/joomla/database/database.php',
			'/libraries/joomla/database/driver.php',
			'/libraries/joomla/database/driver/mysql.php',
			'/libraries/joomla/database/driver/mysqli.php',
			'/libraries/joomla/database/driver/oracle.php',
			'/libraries/joomla/database/driver/pdo.php',
			'/libraries/joomla/database/driver/pdomysql.php',
			'/libraries/joomla/database/driver/pgsql.php',
			'/libraries/joomla/database/driver/postgresql.php',
			'/libraries/joomla/database/driver/sqlazure.php',
			'/libraries/joomla/database/driver/sqlite.php',
			'/libraries/joomla/database/driver/sqlsrv.php',
			'/libraries/joomla/database/exception/connecting.php',
			'/libraries/joomla/database/exception/executing.php',
			'/libraries/joomla/database/exception/unsupported.php',
			'/libraries/joomla/database/exporter.php',
			'/libraries/joomla/database/exporter/mysql.php',
			'/libraries/joomla/database/exporter/mysqli.php',
			'/libraries/joomla/database/exporter/pdomysql.php',
			'/libraries/joomla/database/exporter/pgsql.php',
			'/libraries/joomla/database/exporter/postgresql.php',
			'/libraries/joomla/database/factory.php',
			'/libraries/joomla/database/importer.php',
			'/libraries/joomla/database/importer/mysql.php',
			'/libraries/joomla/database/importer/mysqli.php',
			'/libraries/joomla/database/importer/pdomysql.php',
			'/libraries/joomla/database/importer/pgsql.php',
			'/libraries/joomla/database/importer/postgresql.php',
			'/libraries/joomla/database/interface.php',
			'/libraries/joomla/database/iterator.php',
			'/libraries/joomla/database/iterator/mysql.php',
			'/libraries/joomla/database/iterator/mysqli.php',
			'/libraries/joomla/database/iterator/oracle.php',
			'/libraries/joomla/database/iterator/pdo.php',
			'/libraries/joomla/database/iterator/pdomysql.php',
			'/libraries/joomla/database/iterator/pgsql.php',
			'/libraries/joomla/database/iterator/postgresql.php',
			'/libraries/joomla/database/iterator/sqlazure.php',
			'/libraries/joomla/database/iterator/sqlite.php',
			'/libraries/joomla/database/iterator/sqlsrv.php',
			'/libraries/joomla/database/query.php',
			'/libraries/joomla/database/query/element.php',
			'/libraries/joomla/database/query/limitable.php',
			'/libraries/joomla/database/query/mysql.php',
			'/libraries/joomla/database/query/mysqli.php',
			'/libraries/joomla/database/query/oracle.php',
			'/libraries/joomla/database/query/pdo.php',
			'/libraries/joomla/database/query/pdomysql.php',
			'/libraries/joomla/database/query/pgsql.php',
			'/libraries/joomla/database/query/postgresql.php',
			'/libraries/joomla/database/query/preparable.php',
			'/libraries/joomla/database/query/sqlazure.php',
			'/libraries/joomla/database/query/sqlite.php',
			'/libraries/joomla/database/query/sqlsrv.php',
			'/libraries/joomla/event/dispatcher.php',
			'/libraries/joomla/event/event.php',
			'/libraries/joomla/facebook/album.php',
			'/libraries/joomla/facebook/checkin.php',
			'/libraries/joomla/facebook/comment.php',
			'/libraries/joomla/facebook/event.php',
			'/libraries/joomla/facebook/facebook.php',
			'/libraries/joomla/facebook/group.php',
			'/libraries/joomla/facebook/link.php',
			'/libraries/joomla/facebook/note.php',
			'/libraries/joomla/facebook/oauth.php',
			'/libraries/joomla/facebook/object.php',
			'/libraries/joomla/facebook/photo.php',
			'/libraries/joomla/facebook/post.php',
			'/libraries/joomla/facebook/status.php',
			'/libraries/joomla/facebook/user.php',
			'/libraries/joomla/facebook/video.php',
			'/libraries/joomla/form/fields/accesslevel.php',
			'/libraries/joomla/form/fields/aliastag.php',
			'/libraries/joomla/form/fields/cachehandler.php',
			'/libraries/joomla/form/fields/calendar.php',
			'/libraries/joomla/form/fields/checkbox.php',
			'/libraries/joomla/form/fields/checkboxes.php',
			'/libraries/joomla/form/fields/color.php',
			'/libraries/joomla/form/fields/combo.php',
			'/libraries/joomla/form/fields/components.php',
			'/libraries/joomla/form/fields/databaseconnection.php',
			'/libraries/joomla/form/fields/email.php',
			'/libraries/joomla/form/fields/file.php',
			'/libraries/joomla/form/fields/filelist.php',
			'/libraries/joomla/form/fields/folderlist.php',
			'/libraries/joomla/form/fields/groupedlist.php',
			'/libraries/joomla/form/fields/hidden.php',
			'/libraries/joomla/form/fields/imagelist.php',
			'/libraries/joomla/form/fields/integer.php',
			'/libraries/joomla/form/fields/language.php',
			'/libraries/joomla/form/fields/list.php',
			'/libraries/joomla/form/fields/meter.php',
			'/libraries/joomla/form/fields/note.php',
			'/libraries/joomla/form/fields/number.php',
			'/libraries/joomla/form/fields/password.php',
			'/libraries/joomla/form/fields/plugins.php',
			'/libraries/joomla/form/fields/predefinedlist.php',
			'/libraries/joomla/form/fields/radio.php',
			'/libraries/joomla/form/fields/range.php',
			'/libraries/joomla/form/fields/repeatable.php',
			'/libraries/joomla/form/fields/rules.php',
			'/libraries/joomla/form/fields/sessionhandler.php',
			'/libraries/joomla/form/fields/spacer.php',
			'/libraries/joomla/form/fields/sql.php',
			'/libraries/joomla/form/fields/subform.php',
			'/libraries/joomla/form/fields/tel.php',
			'/libraries/joomla/form/fields/text.php',
			'/libraries/joomla/form/fields/textarea.php',
			'/libraries/joomla/form/fields/timezone.php',
			'/libraries/joomla/form/fields/url.php',
			'/libraries/joomla/form/fields/usergroup.php',
			'/libraries/joomla/github/account.php',
			'/libraries/joomla/github/commits.php',
			'/libraries/joomla/github/forks.php',
			'/libraries/joomla/github/github.php',
			'/libraries/joomla/github/hooks.php',
			'/libraries/joomla/github/http.php',
			'/libraries/joomla/github/meta.php',
			'/libraries/joomla/github/milestones.php',
			'/libraries/joomla/github/object.php',
			'/libraries/joomla/github/package.php',
			'/libraries/joomla/github/package/activity.php',
			'/libraries/joomla/github/package/activity/events.php',
			'/libraries/joomla/github/package/activity/notifications.php',
			'/libraries/joomla/github/package/activity/starring.php',
			'/libraries/joomla/github/package/activity/watching.php',
			'/libraries/joomla/github/package/authorization.php',
			'/libraries/joomla/github/package/data.php',
			'/libraries/joomla/github/package/data/blobs.php',
			'/libraries/joomla/github/package/data/commits.php',
			'/libraries/joomla/github/package/data/refs.php',
			'/libraries/joomla/github/package/data/tags.php',
			'/libraries/joomla/github/package/data/trees.php',
			'/libraries/joomla/github/package/gists.php',
			'/libraries/joomla/github/package/gists/comments.php',
			'/libraries/joomla/github/package/gitignore.php',
			'/libraries/joomla/github/package/issues.php',
			'/libraries/joomla/github/package/issues/assignees.php',
			'/libraries/joomla/github/package/issues/comments.php',
			'/libraries/joomla/github/package/issues/events.php',
			'/libraries/joomla/github/package/issues/labels.php',
			'/libraries/joomla/github/package/issues/milestones.php',
			'/libraries/joomla/github/package/markdown.php',
			'/libraries/joomla/github/package/orgs.php',
			'/libraries/joomla/github/package/orgs/members.php',
			'/libraries/joomla/github/package/orgs/teams.php',
			'/libraries/joomla/github/package/pulls.php',
			'/libraries/joomla/github/package/pulls/comments.php',
			'/libraries/joomla/github/package/repositories.php',
			'/libraries/joomla/github/package/repositories/collaborators.php',
			'/libraries/joomla/github/package/repositories/comments.php',
			'/libraries/joomla/github/package/repositories/commits.php',
			'/libraries/joomla/github/package/repositories/contents.php',
			'/libraries/joomla/github/package/repositories/downloads.php',
			'/libraries/joomla/github/package/repositories/forks.php',
			'/libraries/joomla/github/package/repositories/hooks.php',
			'/libraries/joomla/github/package/repositories/keys.php',
			'/libraries/joomla/github/package/repositories/merging.php',
			'/libraries/joomla/github/package/repositories/statistics.php',
			'/libraries/joomla/github/package/repositories/statuses.php',
			'/libraries/joomla/github/package/search.php',
			'/libraries/joomla/github/package/users.php',
			'/libraries/joomla/github/package/users/emails.php',
			'/libraries/joomla/github/package/users/followers.php',
			'/libraries/joomla/github/package/users/keys.php',
			'/libraries/joomla/github/refs.php',
			'/libraries/joomla/github/statuses.php',
			'/libraries/joomla/google/auth.php',
			'/libraries/joomla/google/auth/oauth2.php',
			'/libraries/joomla/google/data.php',
			'/libraries/joomla/google/data/adsense.php',
			'/libraries/joomla/google/data/calendar.php',
			'/libraries/joomla/google/data/picasa.php',
			'/libraries/joomla/google/data/picasa/album.php',
			'/libraries/joomla/google/data/picasa/photo.php',
			'/libraries/joomla/google/data/plus.php',
			'/libraries/joomla/google/data/plus/activities.php',
			'/libraries/joomla/google/data/plus/comments.php',
			'/libraries/joomla/google/data/plus/people.php',
			'/libraries/joomla/google/embed.php',
			'/libraries/joomla/google/embed/analytics.php',
			'/libraries/joomla/google/embed/maps.php',
			'/libraries/joomla/google/google.php',
			'/libraries/joomla/grid/grid.php',
			'/libraries/joomla/keychain/keychain.php',
			'/libraries/joomla/linkedin/communications.php',
			'/libraries/joomla/linkedin/companies.php',
			'/libraries/joomla/linkedin/groups.php',
			'/libraries/joomla/linkedin/jobs.php',
			'/libraries/joomla/linkedin/linkedin.php',
			'/libraries/joomla/linkedin/oauth.php',
			'/libraries/joomla/linkedin/object.php',
			'/libraries/joomla/linkedin/people.php',
			'/libraries/joomla/linkedin/stream.php',
			'/libraries/joomla/mediawiki/categories.php',
			'/libraries/joomla/mediawiki/http.php',
			'/libraries/joomla/mediawiki/images.php',
			'/libraries/joomla/mediawiki/links.php',
			'/libraries/joomla/mediawiki/mediawiki.php',
			'/libraries/joomla/mediawiki/object.php',
			'/libraries/joomla/mediawiki/pages.php',
			'/libraries/joomla/mediawiki/search.php',
			'/libraries/joomla/mediawiki/sites.php',
			'/libraries/joomla/mediawiki/users.php',
			'/libraries/joomla/model/base.php',
			'/libraries/joomla/model/database.php',
			'/libraries/joomla/model/model.php',
			'/libraries/joomla/oauth1/client.php',
			'/libraries/joomla/oauth2/client.php',
			'/libraries/joomla/observable/interface.php',
			'/libraries/joomla/observer/interface.php',
			'/libraries/joomla/observer/mapper.php',
			'/libraries/joomla/observer/updater.php',
			'/libraries/joomla/observer/updater/interface.php',
			'/libraries/joomla/observer/wrapper/mapper.php',
			'/libraries/joomla/openstreetmap/changesets.php',
			'/libraries/joomla/openstreetmap/elements.php',
			'/libraries/joomla/openstreetmap/gps.php',
			'/libraries/joomla/openstreetmap/info.php',
			'/libraries/joomla/openstreetmap/oauth.php',
			'/libraries/joomla/openstreetmap/object.php',
			'/libraries/joomla/openstreetmap/openstreetmap.php',
			'/libraries/joomla/openstreetmap/user.php',
			'/libraries/joomla/platform.php',
			'/libraries/joomla/route/wrapper/route.php',
			'/libraries/joomla/session/handler/interface.php',
			'/libraries/joomla/session/handler/joomla.php',
			'/libraries/joomla/session/handler/native.php',
			'/libraries/joomla/session/storage.php',
			'/libraries/joomla/session/storage/apc.php',
			'/libraries/joomla/session/storage/apcu.php',
			'/libraries/joomla/session/storage/database.php',
			'/libraries/joomla/session/storage/memcache.php',
			'/libraries/joomla/session/storage/memcached.php',
			'/libraries/joomla/session/storage/none.php',
			'/libraries/joomla/session/storage/redis.php',
			'/libraries/joomla/session/storage/wincache.php',
			'/libraries/joomla/session/storage/xcache.php',
			'/libraries/joomla/string/string.php',
			'/libraries/joomla/string/wrapper/normalise.php',
			'/libraries/joomla/string/wrapper/punycode.php',
			'/libraries/joomla/twitter/block.php',
			'/libraries/joomla/twitter/directmessages.php',
			'/libraries/joomla/twitter/favorites.php',
			'/libraries/joomla/twitter/friends.php',
			'/libraries/joomla/twitter/help.php',
			'/libraries/joomla/twitter/lists.php',
			'/libraries/joomla/twitter/oauth.php',
			'/libraries/joomla/twitter/object.php',
			'/libraries/joomla/twitter/places.php',
			'/libraries/joomla/twitter/profile.php',
			'/libraries/joomla/twitter/search.php',
			'/libraries/joomla/twitter/statuses.php',
			'/libraries/joomla/twitter/trends.php',
			'/libraries/joomla/twitter/twitter.php',
			'/libraries/joomla/twitter/users.php',
			'/libraries/joomla/utilities/arrayhelper.php',
			'/libraries/joomla/view/base.php',
			'/libraries/joomla/view/html.php',
			'/libraries/joomla/view/view.php',
			'/libraries/legacy/application/application.php',
			'/libraries/legacy/base/node.php',
			'/libraries/legacy/base/observable.php',
			'/libraries/legacy/base/observer.php',
			'/libraries/legacy/base/tree.php',
			'/libraries/legacy/database/exception.php',
			'/libraries/legacy/database/mysql.php',
			'/libraries/legacy/database/mysqli.php',
			'/libraries/legacy/database/sqlazure.php',
			'/libraries/legacy/database/sqlsrv.php',
			'/libraries/legacy/dispatcher/dispatcher.php',
			'/libraries/legacy/error/error.php',
			'/libraries/legacy/exception/exception.php',
			'/libraries/legacy/form/field/category.php',
			'/libraries/legacy/form/field/componentlayout.php',
			'/libraries/legacy/form/field/modulelayout.php',
			'/libraries/legacy/log/logexception.php',
			'/libraries/legacy/request/request.php',
			'/libraries/legacy/response/response.php',
			'/libraries/legacy/simplecrypt/simplecrypt.php',
			'/libraries/legacy/simplepie/factory.php',
			'/libraries/legacy/table/session.php',
			'/libraries/legacy/utilities/xmlelement.php',
			'/libraries/phputf8/LICENSE',
			'/libraries/phputf8/README',
			'/libraries/phputf8/mbstring/core.php',
			'/libraries/phputf8/native/core.php',
			'/libraries/phputf8/ord.php',
			'/libraries/phputf8/str_ireplace.php',
			'/libraries/phputf8/str_pad.php',
			'/libraries/phputf8/str_split.php',
			'/libraries/phputf8/strcasecmp.php',
			'/libraries/phputf8/strcspn.php',
			'/libraries/phputf8/stristr.php',
			'/libraries/phputf8/strrev.php',
			'/libraries/phputf8/strspn.php',
			'/libraries/phputf8/substr_replace.php',
			'/libraries/phputf8/trim.php',
			'/libraries/phputf8/ucfirst.php',
			'/libraries/phputf8/ucwords.php',
			'/libraries/phputf8/utf8.php',
			'/libraries/phputf8/utils/ascii.php',
			'/libraries/phputf8/utils/bad.php',
			'/libraries/phputf8/utils/patterns.php',
			'/libraries/phputf8/utils/position.php',
			'/libraries/phputf8/utils/specials.php',
			'/libraries/phputf8/utils/unicode.php',
			'/libraries/phputf8/utils/validation.php',
			'/libraries/src/Access/Wrapper/Access.php',
			'/libraries/src/Cache/Storage/ApcStorage.php',
			'/libraries/src/Cache/Storage/CacheliteStorage.php',
			'/libraries/src/Cache/Storage/MemcacheStorage.php',
			'/libraries/src/Cache/Storage/XcacheStorage.php',
			'/libraries/src/Client/ClientWrapper.php',
			'/libraries/src/Crypt/Cipher/BlowfishCipher.php',
			'/libraries/src/Crypt/Cipher/McryptCipher.php',
			'/libraries/src/Crypt/Cipher/Rijndael256Cipher.php',
			'/libraries/src/Crypt/Cipher/SimpleCipher.php',
			'/libraries/src/Crypt/Cipher/TripleDesCipher.php',
			'/libraries/src/Crypt/CipherInterface.php',
			'/libraries/src/Crypt/CryptPassword.php',
			'/libraries/src/Crypt/Key.php',
			'/libraries/src/Crypt/Password/SimpleCryptPassword.php',
			'/libraries/src/Crypt/README.md',
			'/libraries/src/Filesystem/Wrapper/FileWrapper.php',
			'/libraries/src/Filesystem/Wrapper/FolderWrapper.php',
			'/libraries/src/Filesystem/Wrapper/PathWrapper.php',
			'/libraries/src/Filter/Wrapper/OutputFilterWrapper.php',
			'/libraries/src/Form/Field/HelpsiteField.php',
			'/libraries/src/Form/FormWrapper.php',
			'/libraries/src/Helper/ContentHistoryHelper.php',
			'/libraries/src/Helper/SearchHelper.php',
			'/libraries/src/Http/Wrapper/FactoryWrapper.php',
			'/libraries/src/Language/LanguageStemmer.php',
			'/libraries/src/Language/Stemmer/Porteren.php',
			'/libraries/src/Language/Wrapper/JTextWrapper.php',
			'/libraries/src/Language/Wrapper/LanguageHelperWrapper.php',
			'/libraries/src/Language/Wrapper/TransliterateWrapper.php',
			'/libraries/src/Mail/MailWrapper.php',
			'/libraries/src/Menu/MenuHelper.php',
			'/libraries/src/Menu/Node.php',
			'/libraries/src/Menu/Node/Component.php',
			'/libraries/src/Menu/Node/Container.php',
			'/libraries/src/Menu/Node/Heading.php',
			'/libraries/src/Menu/Node/Separator.php',
			'/libraries/src/Menu/Node/Url.php',
			'/libraries/src/Menu/Tree.php',
			'/libraries/src/Table/Observer/AbstractObserver.php',
			'/libraries/src/Table/Observer/ContentHistory.php',
			'/libraries/src/Table/Observer/Tags.php',
			'/libraries/src/Toolbar/Button/SliderButton.php',
			'/libraries/src/User/UserWrapper.php',
			'/libraries/vendor/.htaccess',
			'/libraries/vendor/brumann/polyfill-unserialize/LICENSE',
			'/libraries/vendor/brumann/polyfill-unserialize/composer.json',
			'/libraries/vendor/brumann/polyfill-unserialize/src/Unserialize.php',
			'/libraries/vendor/ircmaxell/password-compat/LICENSE.md',
			'/libraries/vendor/ircmaxell/password-compat/lib/password.php',
			'/libraries/vendor/joomla/application/src/AbstractCliApplication.php',
			'/libraries/vendor/joomla/application/src/AbstractDaemonApplication.php',
			'/libraries/vendor/joomla/application/src/Cli/CliInput.php',
			'/libraries/vendor/joomla/application/src/Cli/CliOutput.php',
			'/libraries/vendor/joomla/application/src/Cli/ColorProcessor.php',
			'/libraries/vendor/joomla/application/src/Cli/ColorStyle.php',
			'/libraries/vendor/joomla/application/src/Cli/Output/Processor/ColorProcessor.php',
			'/libraries/vendor/joomla/application/src/Cli/Output/Processor/ProcessorInterface.php',
			'/libraries/vendor/joomla/application/src/Cli/Output/Stdout.php',
			'/libraries/vendor/joomla/application/src/Cli/Output/Xml.php',
			'/libraries/vendor/joomla/compat/LICENSE',
			'/libraries/vendor/joomla/compat/src/CallbackFilterIterator.php',
			'/libraries/vendor/joomla/compat/src/JsonSerializable.php',
			'/libraries/vendor/joomla/event/src/DelegatingDispatcher.php',
			'/libraries/vendor/joomla/image/LICENSE',
			'/libraries/vendor/joomla/image/src/Filter/Backgroundfill.php',
			'/libraries/vendor/joomla/image/src/Filter/Brightness.php',
			'/libraries/vendor/joomla/image/src/Filter/Contrast.php',
			'/libraries/vendor/joomla/image/src/Filter/Edgedetect.php',
			'/libraries/vendor/joomla/image/src/Filter/Emboss.php',
			'/libraries/vendor/joomla/image/src/Filter/Grayscale.php',
			'/libraries/vendor/joomla/image/src/Filter/Negate.php',
			'/libraries/vendor/joomla/image/src/Filter/Sketchy.php',
			'/libraries/vendor/joomla/image/src/Filter/Smooth.php',
			'/libraries/vendor/joomla/image/src/Image.php',
			'/libraries/vendor/joomla/image/src/ImageFilter.php',
			'/libraries/vendor/joomla/input/src/Cli.php',
			'/libraries/vendor/joomla/registry/src/AbstractRegistryFormat.php',
			'/libraries/vendor/joomla/session/Joomla/Session/LICENSE',
			'/libraries/vendor/joomla/session/Joomla/Session/Session.php',
			'/libraries/vendor/joomla/session/Joomla/Session/Storage.php',
			'/libraries/vendor/joomla/session/Joomla/Session/Storage/Apc.php',
			'/libraries/vendor/joomla/session/Joomla/Session/Storage/Apcu.php',
			'/libraries/vendor/joomla/session/Joomla/Session/Storage/Database.php',
			'/libraries/vendor/joomla/session/Joomla/Session/Storage/Memcache.php',
			'/libraries/vendor/joomla/session/Joomla/Session/Storage/Memcached.php',
			'/libraries/vendor/joomla/session/Joomla/Session/Storage/None.php',
			'/libraries/vendor/joomla/session/Joomla/Session/Storage/Wincache.php',
			'/libraries/vendor/joomla/session/Joomla/Session/Storage/Xcache.php',
			'/libraries/vendor/joomla/string/src/String.php',
			'/libraries/vendor/leafo/lessphp/LICENSE',
			'/libraries/vendor/leafo/lessphp/lessc.inc.php',
			'/libraries/vendor/leafo/lessphp/lessify',
			'/libraries/vendor/leafo/lessphp/lessify.inc.php',
			'/libraries/vendor/leafo/lessphp/plessc',
			'/libraries/vendor/paragonie/random_compat/lib/byte_safe_strings.php',
			'/libraries/vendor/paragonie/random_compat/lib/cast_to_int.php',
			'/libraries/vendor/paragonie/random_compat/lib/error_polyfill.php',
			'/libraries/vendor/paragonie/random_compat/lib/random_bytes_com_dotnet.php',
			'/libraries/vendor/paragonie/random_compat/lib/random_bytes_dev_urandom.php',
			'/libraries/vendor/paragonie/random_compat/lib/random_bytes_libsodium.php',
			'/libraries/vendor/paragonie/random_compat/lib/random_bytes_libsodium_legacy.php',
			'/libraries/vendor/paragonie/random_compat/lib/random_bytes_mcrypt.php',
			'/libraries/vendor/paragonie/random_compat/lib/random_bytes_openssl.php',
			'/libraries/vendor/paragonie/random_compat/lib/random_int.php',
			'/libraries/vendor/paragonie/sodium_compat/src/Core/Xsalsa20.php',
			'/libraries/vendor/paragonie/sodium_compat/src/Core32/Curve25519/README.md',
			'/libraries/vendor/phpmailer/phpmailer/PHPMailerAutoload.php',
			'/libraries/vendor/phpmailer/phpmailer/class.phpmailer.php',
			'/libraries/vendor/phpmailer/phpmailer/class.phpmaileroauth.php',
			'/libraries/vendor/phpmailer/phpmailer/class.phpmaileroauthgoogle.php',
			'/libraries/vendor/phpmailer/phpmailer/class.pop3.php',
			'/libraries/vendor/phpmailer/phpmailer/class.smtp.php',
			'/libraries/vendor/phpmailer/phpmailer/extras/EasyPeasyICS.php',
			'/libraries/vendor/phpmailer/phpmailer/extras/htmlfilter.php',
			'/libraries/vendor/phpmailer/phpmailer/extras/ntlm_sasl_client.php',
			'/libraries/vendor/simplepie/simplepie/LICENSE.txt',
			'/libraries/vendor/simplepie/simplepie/autoloader.php',
			'/libraries/vendor/simplepie/simplepie/db.sql',
			'/libraries/vendor/simplepie/simplepie/idn/LICENCE',
			'/libraries/vendor/simplepie/simplepie/idn/idna_convert.class.php',
			'/libraries/vendor/simplepie/simplepie/idn/npdata.ser',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Author.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Cache.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Cache/Base.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Cache/DB.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Cache/File.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Cache/Memcache.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Cache/MySQL.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Caption.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Category.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Content/Type/Sniffer.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Copyright.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Core.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Credit.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Decode/HTML/Entities.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Enclosure.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Exception.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/File.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/HTTP/Parser.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/IRI.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Item.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Locator.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Misc.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Net/IPv6.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Parse/Date.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Parser.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Rating.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Registry.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Restriction.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Sanitize.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Source.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/XML/Declaration/Parser.php',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/gzdecode.php',
			'/libraries/vendor/symfony/polyfill-php55/LICENSE',
			'/libraries/vendor/symfony/polyfill-php55/Php55.php',
			'/libraries/vendor/symfony/polyfill-php55/Php55ArrayColumn.php',
			'/libraries/vendor/symfony/polyfill-php55/bootstrap.php',
			'/libraries/vendor/symfony/polyfill-php56/LICENSE',
			'/libraries/vendor/symfony/polyfill-php56/Php56.php',
			'/libraries/vendor/symfony/polyfill-php56/bootstrap.php',
			'/libraries/vendor/symfony/polyfill-php71/LICENSE',
			'/libraries/vendor/symfony/polyfill-php71/Php71.php',
			'/libraries/vendor/symfony/polyfill-php71/bootstrap.php',
			'/libraries/vendor/symfony/polyfill-util/Binary.php',
			'/libraries/vendor/symfony/polyfill-util/BinaryNoFuncOverload.php',
			'/libraries/vendor/symfony/polyfill-util/BinaryOnFuncOverload.php',
			'/libraries/vendor/symfony/polyfill-util/LICENSE',
			'/libraries/vendor/typo3/phar-stream-wrapper/composer.json',
			'/libraries/vendor/web.config',
			'/media/cms/css/debug.css',
			'/media/com_associations/js/sidebyside-uncompressed.js',
			'/media/com_contenthistory/css/jquery.pretty-text-diff.css',
			'/media/com_contenthistory/js/diff_match_patch.js',
			'/media/com_contenthistory/js/jquery.pretty-text-diff.js',
			'/media/com_contenthistory/js/jquery.pretty-text-diff.min.js',
			'/media/com_finder/js/autocompleter.js',
			'/media/com_joomlaupdate/js/json2.js',
			'/media/com_joomlaupdate/js/json2.min.js',
			'/media/editors/codemirror/LICENSE',
			'/media/editors/codemirror/addon/comment/comment.js',
			'/media/editors/codemirror/addon/comment/comment.min.js',
			'/media/editors/codemirror/addon/comment/continuecomment.js',
			'/media/editors/codemirror/addon/comment/continuecomment.min.js',
			'/media/editors/codemirror/addon/dialog/dialog.css',
			'/media/editors/codemirror/addon/dialog/dialog.js',
			'/media/editors/codemirror/addon/dialog/dialog.min.css',
			'/media/editors/codemirror/addon/dialog/dialog.min.js',
			'/media/editors/codemirror/addon/display/autorefresh.js',
			'/media/editors/codemirror/addon/display/autorefresh.min.js',
			'/media/editors/codemirror/addon/display/fullscreen.css',
			'/media/editors/codemirror/addon/display/fullscreen.js',
			'/media/editors/codemirror/addon/display/fullscreen.min.css',
			'/media/editors/codemirror/addon/display/fullscreen.min.js',
			'/media/editors/codemirror/addon/display/panel.js',
			'/media/editors/codemirror/addon/display/panel.min.js',
			'/media/editors/codemirror/addon/display/placeholder.js',
			'/media/editors/codemirror/addon/display/placeholder.min.js',
			'/media/editors/codemirror/addon/display/rulers.js',
			'/media/editors/codemirror/addon/display/rulers.min.js',
			'/media/editors/codemirror/addon/edit/closebrackets.js',
			'/media/editors/codemirror/addon/edit/closebrackets.min.js',
			'/media/editors/codemirror/addon/edit/closetag.js',
			'/media/editors/codemirror/addon/edit/closetag.min.js',
			'/media/editors/codemirror/addon/edit/continuelist.js',
			'/media/editors/codemirror/addon/edit/continuelist.min.js',
			'/media/editors/codemirror/addon/edit/matchbrackets.js',
			'/media/editors/codemirror/addon/edit/matchbrackets.min.js',
			'/media/editors/codemirror/addon/edit/matchtags.js',
			'/media/editors/codemirror/addon/edit/matchtags.min.js',
			'/media/editors/codemirror/addon/edit/trailingspace.js',
			'/media/editors/codemirror/addon/edit/trailingspace.min.js',
			'/media/editors/codemirror/addon/fold/brace-fold.js',
			'/media/editors/codemirror/addon/fold/brace-fold.min.js',
			'/media/editors/codemirror/addon/fold/comment-fold.js',
			'/media/editors/codemirror/addon/fold/comment-fold.min.js',
			'/media/editors/codemirror/addon/fold/foldcode.js',
			'/media/editors/codemirror/addon/fold/foldcode.min.js',
			'/media/editors/codemirror/addon/fold/foldgutter.css',
			'/media/editors/codemirror/addon/fold/foldgutter.js',
			'/media/editors/codemirror/addon/fold/foldgutter.min.css',
			'/media/editors/codemirror/addon/fold/foldgutter.min.js',
			'/media/editors/codemirror/addon/fold/indent-fold.js',
			'/media/editors/codemirror/addon/fold/indent-fold.min.js',
			'/media/editors/codemirror/addon/fold/markdown-fold.js',
			'/media/editors/codemirror/addon/fold/markdown-fold.min.js',
			'/media/editors/codemirror/addon/fold/xml-fold.js',
			'/media/editors/codemirror/addon/fold/xml-fold.min.js',
			'/media/editors/codemirror/addon/hint/anyword-hint.js',
			'/media/editors/codemirror/addon/hint/anyword-hint.min.js',
			'/media/editors/codemirror/addon/hint/css-hint.js',
			'/media/editors/codemirror/addon/hint/css-hint.min.js',
			'/media/editors/codemirror/addon/hint/html-hint.js',
			'/media/editors/codemirror/addon/hint/html-hint.min.js',
			'/media/editors/codemirror/addon/hint/javascript-hint.js',
			'/media/editors/codemirror/addon/hint/javascript-hint.min.js',
			'/media/editors/codemirror/addon/hint/show-hint.css',
			'/media/editors/codemirror/addon/hint/show-hint.js',
			'/media/editors/codemirror/addon/hint/show-hint.min.css',
			'/media/editors/codemirror/addon/hint/show-hint.min.js',
			'/media/editors/codemirror/addon/hint/sql-hint.js',
			'/media/editors/codemirror/addon/hint/sql-hint.min.js',
			'/media/editors/codemirror/addon/hint/xml-hint.js',
			'/media/editors/codemirror/addon/hint/xml-hint.min.js',
			'/media/editors/codemirror/addon/lint/coffeescript-lint.js',
			'/media/editors/codemirror/addon/lint/coffeescript-lint.min.js',
			'/media/editors/codemirror/addon/lint/css-lint.js',
			'/media/editors/codemirror/addon/lint/css-lint.min.js',
			'/media/editors/codemirror/addon/lint/html-lint.js',
			'/media/editors/codemirror/addon/lint/html-lint.min.js',
			'/media/editors/codemirror/addon/lint/javascript-lint.js',
			'/media/editors/codemirror/addon/lint/javascript-lint.min.js',
			'/media/editors/codemirror/addon/lint/json-lint.js',
			'/media/editors/codemirror/addon/lint/json-lint.min.js',
			'/media/editors/codemirror/addon/lint/lint.css',
			'/media/editors/codemirror/addon/lint/lint.js',
			'/media/editors/codemirror/addon/lint/lint.min.css',
			'/media/editors/codemirror/addon/lint/lint.min.js',
			'/media/editors/codemirror/addon/lint/yaml-lint.js',
			'/media/editors/codemirror/addon/lint/yaml-lint.min.js',
			'/media/editors/codemirror/addon/merge/merge.css',
			'/media/editors/codemirror/addon/merge/merge.js',
			'/media/editors/codemirror/addon/merge/merge.min.css',
			'/media/editors/codemirror/addon/merge/merge.min.js',
			'/media/editors/codemirror/addon/mode/loadmode.js',
			'/media/editors/codemirror/addon/mode/loadmode.min.js',
			'/media/editors/codemirror/addon/mode/multiplex.js',
			'/media/editors/codemirror/addon/mode/multiplex.min.js',
			'/media/editors/codemirror/addon/mode/multiplex_test.js',
			'/media/editors/codemirror/addon/mode/multiplex_test.min.js',
			'/media/editors/codemirror/addon/mode/overlay.js',
			'/media/editors/codemirror/addon/mode/overlay.min.js',
			'/media/editors/codemirror/addon/mode/simple.js',
			'/media/editors/codemirror/addon/mode/simple.min.js',
			'/media/editors/codemirror/addon/runmode/colorize.js',
			'/media/editors/codemirror/addon/runmode/colorize.min.js',
			'/media/editors/codemirror/addon/runmode/runmode-standalone.js',
			'/media/editors/codemirror/addon/runmode/runmode-standalone.min.js',
			'/media/editors/codemirror/addon/runmode/runmode.js',
			'/media/editors/codemirror/addon/runmode/runmode.min.js',
			'/media/editors/codemirror/addon/runmode/runmode.node.js',
			'/media/editors/codemirror/addon/scroll/annotatescrollbar.js',
			'/media/editors/codemirror/addon/scroll/annotatescrollbar.min.js',
			'/media/editors/codemirror/addon/scroll/scrollpastend.js',
			'/media/editors/codemirror/addon/scroll/scrollpastend.min.js',
			'/media/editors/codemirror/addon/scroll/simplescrollbars.css',
			'/media/editors/codemirror/addon/scroll/simplescrollbars.js',
			'/media/editors/codemirror/addon/scroll/simplescrollbars.min.css',
			'/media/editors/codemirror/addon/scroll/simplescrollbars.min.js',
			'/media/editors/codemirror/addon/search/jump-to-line.js',
			'/media/editors/codemirror/addon/search/jump-to-line.min.js',
			'/media/editors/codemirror/addon/search/match-highlighter.js',
			'/media/editors/codemirror/addon/search/match-highlighter.min.js',
			'/media/editors/codemirror/addon/search/matchesonscrollbar.css',
			'/media/editors/codemirror/addon/search/matchesonscrollbar.js',
			'/media/editors/codemirror/addon/search/matchesonscrollbar.min.css',
			'/media/editors/codemirror/addon/search/matchesonscrollbar.min.js',
			'/media/editors/codemirror/addon/search/search.js',
			'/media/editors/codemirror/addon/search/search.min.js',
			'/media/editors/codemirror/addon/search/searchcursor.js',
			'/media/editors/codemirror/addon/search/searchcursor.min.js',
			'/media/editors/codemirror/addon/selection/active-line.js',
			'/media/editors/codemirror/addon/selection/active-line.min.js',
			'/media/editors/codemirror/addon/selection/mark-selection.js',
			'/media/editors/codemirror/addon/selection/mark-selection.min.js',
			'/media/editors/codemirror/addon/selection/selection-pointer.js',
			'/media/editors/codemirror/addon/selection/selection-pointer.min.js',
			'/media/editors/codemirror/addon/tern/tern.css',
			'/media/editors/codemirror/addon/tern/tern.js',
			'/media/editors/codemirror/addon/tern/tern.min.css',
			'/media/editors/codemirror/addon/tern/tern.min.js',
			'/media/editors/codemirror/addon/tern/worker.js',
			'/media/editors/codemirror/addon/tern/worker.min.js',
			'/media/editors/codemirror/addon/wrap/hardwrap.js',
			'/media/editors/codemirror/addon/wrap/hardwrap.min.js',
			'/media/editors/codemirror/keymap/emacs.js',
			'/media/editors/codemirror/keymap/emacs.min.js',
			'/media/editors/codemirror/keymap/sublime.js',
			'/media/editors/codemirror/keymap/sublime.min.js',
			'/media/editors/codemirror/keymap/vim.js',
			'/media/editors/codemirror/keymap/vim.min.js',
			'/media/editors/codemirror/lib/addons.css',
			'/media/editors/codemirror/lib/addons.js',
			'/media/editors/codemirror/lib/addons.min.css',
			'/media/editors/codemirror/lib/addons.min.js',
			'/media/editors/codemirror/lib/codemirror.css',
			'/media/editors/codemirror/lib/codemirror.js',
			'/media/editors/codemirror/lib/codemirror.min.css',
			'/media/editors/codemirror/lib/codemirror.min.js',
			'/media/editors/codemirror/mode/apl/apl.js',
			'/media/editors/codemirror/mode/apl/apl.min.js',
			'/media/editors/codemirror/mode/asciiarmor/asciiarmor.js',
			'/media/editors/codemirror/mode/asciiarmor/asciiarmor.min.js',
			'/media/editors/codemirror/mode/asn.1/asn.1.js',
			'/media/editors/codemirror/mode/asn.1/asn.min.js',
			'/media/editors/codemirror/mode/asterisk/asterisk.js',
			'/media/editors/codemirror/mode/asterisk/asterisk.min.js',
			'/media/editors/codemirror/mode/brainfuck/brainfuck.js',
			'/media/editors/codemirror/mode/brainfuck/brainfuck.min.js',
			'/media/editors/codemirror/mode/clike/clike.js',
			'/media/editors/codemirror/mode/clike/clike.min.js',
			'/media/editors/codemirror/mode/clojure/clojure.js',
			'/media/editors/codemirror/mode/clojure/clojure.min.js',
			'/media/editors/codemirror/mode/cmake/cmake.js',
			'/media/editors/codemirror/mode/cmake/cmake.min.js',
			'/media/editors/codemirror/mode/cobol/cobol.js',
			'/media/editors/codemirror/mode/cobol/cobol.min.js',
			'/media/editors/codemirror/mode/coffeescript/coffeescript.js',
			'/media/editors/codemirror/mode/coffeescript/coffeescript.min.js',
			'/media/editors/codemirror/mode/commonlisp/commonlisp.js',
			'/media/editors/codemirror/mode/commonlisp/commonlisp.min.js',
			'/media/editors/codemirror/mode/crystal/crystal.js',
			'/media/editors/codemirror/mode/crystal/crystal.min.js',
			'/media/editors/codemirror/mode/css/css.js',
			'/media/editors/codemirror/mode/css/css.min.js',
			'/media/editors/codemirror/mode/cypher/cypher.js',
			'/media/editors/codemirror/mode/cypher/cypher.min.js',
			'/media/editors/codemirror/mode/d/d.js',
			'/media/editors/codemirror/mode/d/d.min.js',
			'/media/editors/codemirror/mode/dart/dart.js',
			'/media/editors/codemirror/mode/dart/dart.min.js',
			'/media/editors/codemirror/mode/diff/diff.js',
			'/media/editors/codemirror/mode/diff/diff.min.js',
			'/media/editors/codemirror/mode/django/django.js',
			'/media/editors/codemirror/mode/django/django.min.js',
			'/media/editors/codemirror/mode/dockerfile/dockerfile.js',
			'/media/editors/codemirror/mode/dockerfile/dockerfile.min.js',
			'/media/editors/codemirror/mode/dtd/dtd.js',
			'/media/editors/codemirror/mode/dtd/dtd.min.js',
			'/media/editors/codemirror/mode/dylan/dylan.js',
			'/media/editors/codemirror/mode/dylan/dylan.min.js',
			'/media/editors/codemirror/mode/ebnf/ebnf.js',
			'/media/editors/codemirror/mode/ebnf/ebnf.min.js',
			'/media/editors/codemirror/mode/ecl/ecl.js',
			'/media/editors/codemirror/mode/ecl/ecl.min.js',
			'/media/editors/codemirror/mode/eiffel/eiffel.js',
			'/media/editors/codemirror/mode/eiffel/eiffel.min.js',
			'/media/editors/codemirror/mode/elm/elm.js',
			'/media/editors/codemirror/mode/elm/elm.min.js',
			'/media/editors/codemirror/mode/erlang/erlang.js',
			'/media/editors/codemirror/mode/erlang/erlang.min.js',
			'/media/editors/codemirror/mode/factor/factor.js',
			'/media/editors/codemirror/mode/factor/factor.min.js',
			'/media/editors/codemirror/mode/fcl/fcl.js',
			'/media/editors/codemirror/mode/fcl/fcl.min.js',
			'/media/editors/codemirror/mode/forth/forth.js',
			'/media/editors/codemirror/mode/forth/forth.min.js',
			'/media/editors/codemirror/mode/fortran/fortran.js',
			'/media/editors/codemirror/mode/fortran/fortran.min.js',
			'/media/editors/codemirror/mode/gas/gas.js',
			'/media/editors/codemirror/mode/gas/gas.min.js',
			'/media/editors/codemirror/mode/gfm/gfm.js',
			'/media/editors/codemirror/mode/gfm/gfm.min.js',
			'/media/editors/codemirror/mode/gherkin/gherkin.js',
			'/media/editors/codemirror/mode/gherkin/gherkin.min.js',
			'/media/editors/codemirror/mode/go/go.js',
			'/media/editors/codemirror/mode/go/go.min.js',
			'/media/editors/codemirror/mode/groovy/groovy.js',
			'/media/editors/codemirror/mode/groovy/groovy.min.js',
			'/media/editors/codemirror/mode/haml/haml.js',
			'/media/editors/codemirror/mode/haml/haml.min.js',
			'/media/editors/codemirror/mode/handlebars/handlebars.js',
			'/media/editors/codemirror/mode/handlebars/handlebars.min.js',
			'/media/editors/codemirror/mode/haskell-literate/haskell-literate.js',
			'/media/editors/codemirror/mode/haskell-literate/haskell-literate.min.js',
			'/media/editors/codemirror/mode/haskell/haskell.js',
			'/media/editors/codemirror/mode/haskell/haskell.min.js',
			'/media/editors/codemirror/mode/haxe/haxe.js',
			'/media/editors/codemirror/mode/haxe/haxe.min.js',
			'/media/editors/codemirror/mode/htmlembedded/htmlembedded.js',
			'/media/editors/codemirror/mode/htmlembedded/htmlembedded.min.js',
			'/media/editors/codemirror/mode/htmlmixed/htmlmixed.js',
			'/media/editors/codemirror/mode/htmlmixed/htmlmixed.min.js',
			'/media/editors/codemirror/mode/http/http.js',
			'/media/editors/codemirror/mode/http/http.min.js',
			'/media/editors/codemirror/mode/idl/idl.js',
			'/media/editors/codemirror/mode/idl/idl.min.js',
			'/media/editors/codemirror/mode/javascript/javascript.js',
			'/media/editors/codemirror/mode/javascript/javascript.min.js',
			'/media/editors/codemirror/mode/jinja2/jinja2.js',
			'/media/editors/codemirror/mode/jinja2/jinja2.min.js',
			'/media/editors/codemirror/mode/jsx/jsx.js',
			'/media/editors/codemirror/mode/jsx/jsx.min.js',
			'/media/editors/codemirror/mode/julia/julia.js',
			'/media/editors/codemirror/mode/julia/julia.min.js',
			'/media/editors/codemirror/mode/livescript/livescript.js',
			'/media/editors/codemirror/mode/livescript/livescript.min.js',
			'/media/editors/codemirror/mode/lua/lua.js',
			'/media/editors/codemirror/mode/lua/lua.min.js',
			'/media/editors/codemirror/mode/markdown/markdown.js',
			'/media/editors/codemirror/mode/markdown/markdown.min.js',
			'/media/editors/codemirror/mode/mathematica/mathematica.js',
			'/media/editors/codemirror/mode/mathematica/mathematica.min.js',
			'/media/editors/codemirror/mode/mbox/mbox.js',
			'/media/editors/codemirror/mode/mbox/mbox.min.js',
			'/media/editors/codemirror/mode/meta.js',
			'/media/editors/codemirror/mode/meta.min.js',
			'/media/editors/codemirror/mode/mirc/mirc.js',
			'/media/editors/codemirror/mode/mirc/mirc.min.js',
			'/media/editors/codemirror/mode/mllike/mllike.js',
			'/media/editors/codemirror/mode/mllike/mllike.min.js',
			'/media/editors/codemirror/mode/modelica/modelica.js',
			'/media/editors/codemirror/mode/modelica/modelica.min.js',
			'/media/editors/codemirror/mode/mscgen/mscgen.js',
			'/media/editors/codemirror/mode/mscgen/mscgen.min.js',
			'/media/editors/codemirror/mode/mumps/mumps.js',
			'/media/editors/codemirror/mode/mumps/mumps.min.js',
			'/media/editors/codemirror/mode/nginx/nginx.js',
			'/media/editors/codemirror/mode/nginx/nginx.min.js',
			'/media/editors/codemirror/mode/nsis/nsis.js',
			'/media/editors/codemirror/mode/nsis/nsis.min.js',
			'/media/editors/codemirror/mode/ntriples/ntriples.js',
			'/media/editors/codemirror/mode/ntriples/ntriples.min.js',
			'/media/editors/codemirror/mode/octave/octave.js',
			'/media/editors/codemirror/mode/octave/octave.min.js',
			'/media/editors/codemirror/mode/oz/oz.js',
			'/media/editors/codemirror/mode/oz/oz.min.js',
			'/media/editors/codemirror/mode/pascal/pascal.js',
			'/media/editors/codemirror/mode/pascal/pascal.min.js',
			'/media/editors/codemirror/mode/pegjs/pegjs.js',
			'/media/editors/codemirror/mode/pegjs/pegjs.min.js',
			'/media/editors/codemirror/mode/perl/perl.js',
			'/media/editors/codemirror/mode/perl/perl.min.js',
			'/media/editors/codemirror/mode/php/php.js',
			'/media/editors/codemirror/mode/php/php.min.js',
			'/media/editors/codemirror/mode/pig/pig.js',
			'/media/editors/codemirror/mode/pig/pig.min.js',
			'/media/editors/codemirror/mode/powershell/powershell.js',
			'/media/editors/codemirror/mode/powershell/powershell.min.js',
			'/media/editors/codemirror/mode/properties/properties.js',
			'/media/editors/codemirror/mode/properties/properties.min.js',
			'/media/editors/codemirror/mode/protobuf/protobuf.js',
			'/media/editors/codemirror/mode/protobuf/protobuf.min.js',
			'/media/editors/codemirror/mode/pug/pug.js',
			'/media/editors/codemirror/mode/pug/pug.min.js',
			'/media/editors/codemirror/mode/puppet/puppet.js',
			'/media/editors/codemirror/mode/puppet/puppet.min.js',
			'/media/editors/codemirror/mode/python/python.js',
			'/media/editors/codemirror/mode/python/python.min.js',
			'/media/editors/codemirror/mode/q/q.js',
			'/media/editors/codemirror/mode/q/q.min.js',
			'/media/editors/codemirror/mode/r/r.js',
			'/media/editors/codemirror/mode/r/r.min.js',
			'/media/editors/codemirror/mode/rpm/changes/index.html',
			'/media/editors/codemirror/mode/rpm/rpm.js',
			'/media/editors/codemirror/mode/rpm/rpm.min.js',
			'/media/editors/codemirror/mode/rst/rst.js',
			'/media/editors/codemirror/mode/rst/rst.min.js',
			'/media/editors/codemirror/mode/ruby/ruby.js',
			'/media/editors/codemirror/mode/ruby/ruby.min.js',
			'/media/editors/codemirror/mode/rust/rust.js',
			'/media/editors/codemirror/mode/rust/rust.min.js',
			'/media/editors/codemirror/mode/sas/sas.js',
			'/media/editors/codemirror/mode/sas/sas.min.js',
			'/media/editors/codemirror/mode/sass/sass.js',
			'/media/editors/codemirror/mode/sass/sass.min.js',
			'/media/editors/codemirror/mode/scheme/scheme.js',
			'/media/editors/codemirror/mode/scheme/scheme.min.js',
			'/media/editors/codemirror/mode/shell/shell.js',
			'/media/editors/codemirror/mode/shell/shell.min.js',
			'/media/editors/codemirror/mode/sieve/sieve.js',
			'/media/editors/codemirror/mode/sieve/sieve.min.js',
			'/media/editors/codemirror/mode/slim/slim.js',
			'/media/editors/codemirror/mode/slim/slim.min.js',
			'/media/editors/codemirror/mode/smalltalk/smalltalk.js',
			'/media/editors/codemirror/mode/smalltalk/smalltalk.min.js',
			'/media/editors/codemirror/mode/smarty/smarty.js',
			'/media/editors/codemirror/mode/smarty/smarty.min.js',
			'/media/editors/codemirror/mode/solr/solr.js',
			'/media/editors/codemirror/mode/solr/solr.min.js',
			'/media/editors/codemirror/mode/soy/soy.js',
			'/media/editors/codemirror/mode/soy/soy.min.js',
			'/media/editors/codemirror/mode/sparql/sparql.js',
			'/media/editors/codemirror/mode/sparql/sparql.min.js',
			'/media/editors/codemirror/mode/spreadsheet/spreadsheet.js',
			'/media/editors/codemirror/mode/spreadsheet/spreadsheet.min.js',
			'/media/editors/codemirror/mode/sql/sql.js',
			'/media/editors/codemirror/mode/sql/sql.min.js',
			'/media/editors/codemirror/mode/stex/stex.js',
			'/media/editors/codemirror/mode/stex/stex.min.js',
			'/media/editors/codemirror/mode/stylus/stylus.js',
			'/media/editors/codemirror/mode/stylus/stylus.min.js',
			'/media/editors/codemirror/mode/swift/swift.js',
			'/media/editors/codemirror/mode/swift/swift.min.js',
			'/media/editors/codemirror/mode/tcl/tcl.js',
			'/media/editors/codemirror/mode/tcl/tcl.min.js',
			'/media/editors/codemirror/mode/textile/textile.js',
			'/media/editors/codemirror/mode/textile/textile.min.js',
			'/media/editors/codemirror/mode/tiddlywiki/tiddlywiki.css',
			'/media/editors/codemirror/mode/tiddlywiki/tiddlywiki.js',
			'/media/editors/codemirror/mode/tiddlywiki/tiddlywiki.min.css',
			'/media/editors/codemirror/mode/tiddlywiki/tiddlywiki.min.js',
			'/media/editors/codemirror/mode/tiki/tiki.css',
			'/media/editors/codemirror/mode/tiki/tiki.js',
			'/media/editors/codemirror/mode/tiki/tiki.min.css',
			'/media/editors/codemirror/mode/tiki/tiki.min.js',
			'/media/editors/codemirror/mode/toml/toml.js',
			'/media/editors/codemirror/mode/toml/toml.min.js',
			'/media/editors/codemirror/mode/tornado/tornado.js',
			'/media/editors/codemirror/mode/tornado/tornado.min.js',
			'/media/editors/codemirror/mode/troff/troff.js',
			'/media/editors/codemirror/mode/troff/troff.min.js',
			'/media/editors/codemirror/mode/ttcn-cfg/ttcn-cfg.js',
			'/media/editors/codemirror/mode/ttcn-cfg/ttcn-cfg.min.js',
			'/media/editors/codemirror/mode/ttcn/ttcn.js',
			'/media/editors/codemirror/mode/ttcn/ttcn.min.js',
			'/media/editors/codemirror/mode/turtle/turtle.js',
			'/media/editors/codemirror/mode/turtle/turtle.min.js',
			'/media/editors/codemirror/mode/twig/twig.js',
			'/media/editors/codemirror/mode/twig/twig.min.js',
			'/media/editors/codemirror/mode/vb/vb.js',
			'/media/editors/codemirror/mode/vb/vb.min.js',
			'/media/editors/codemirror/mode/vbscript/vbscript.js',
			'/media/editors/codemirror/mode/vbscript/vbscript.min.js',
			'/media/editors/codemirror/mode/velocity/velocity.js',
			'/media/editors/codemirror/mode/velocity/velocity.min.js',
			'/media/editors/codemirror/mode/verilog/verilog.js',
			'/media/editors/codemirror/mode/verilog/verilog.min.js',
			'/media/editors/codemirror/mode/vhdl/vhdl.js',
			'/media/editors/codemirror/mode/vhdl/vhdl.min.js',
			'/media/editors/codemirror/mode/vue/vue.js',
			'/media/editors/codemirror/mode/vue/vue.min.js',
			'/media/editors/codemirror/mode/webidl/webidl.js',
			'/media/editors/codemirror/mode/webidl/webidl.min.js',
			'/media/editors/codemirror/mode/xml/xml.js',
			'/media/editors/codemirror/mode/xml/xml.min.js',
			'/media/editors/codemirror/mode/xquery/xquery.js',
			'/media/editors/codemirror/mode/xquery/xquery.min.js',
			'/media/editors/codemirror/mode/yacas/yacas.js',
			'/media/editors/codemirror/mode/yacas/yacas.min.js',
			'/media/editors/codemirror/mode/yaml-frontmatter/yaml-frontmatter.js',
			'/media/editors/codemirror/mode/yaml-frontmatter/yaml-frontmatter.min.js',
			'/media/editors/codemirror/mode/yaml/yaml.js',
			'/media/editors/codemirror/mode/yaml/yaml.min.js',
			'/media/editors/codemirror/mode/z80/z80.js',
			'/media/editors/codemirror/mode/z80/z80.min.js',
			'/media/editors/codemirror/theme/3024-day.css',
			'/media/editors/codemirror/theme/3024-night.css',
			'/media/editors/codemirror/theme/abcdef.css',
			'/media/editors/codemirror/theme/ambiance-mobile.css',
			'/media/editors/codemirror/theme/ambiance.css',
			'/media/editors/codemirror/theme/base16-dark.css',
			'/media/editors/codemirror/theme/base16-light.css',
			'/media/editors/codemirror/theme/bespin.css',
			'/media/editors/codemirror/theme/blackboard.css',
			'/media/editors/codemirror/theme/cobalt.css',
			'/media/editors/codemirror/theme/colorforth.css',
			'/media/editors/codemirror/theme/darcula.css',
			'/media/editors/codemirror/theme/dracula.css',
			'/media/editors/codemirror/theme/duotone-dark.css',
			'/media/editors/codemirror/theme/duotone-light.css',
			'/media/editors/codemirror/theme/eclipse.css',
			'/media/editors/codemirror/theme/elegant.css',
			'/media/editors/codemirror/theme/erlang-dark.css',
			'/media/editors/codemirror/theme/gruvbox-dark.css',
			'/media/editors/codemirror/theme/hopscotch.css',
			'/media/editors/codemirror/theme/icecoder.css',
			'/media/editors/codemirror/theme/idea.css',
			'/media/editors/codemirror/theme/isotope.css',
			'/media/editors/codemirror/theme/lesser-dark.css',
			'/media/editors/codemirror/theme/liquibyte.css',
			'/media/editors/codemirror/theme/lucario.css',
			'/media/editors/codemirror/theme/material.css',
			'/media/editors/codemirror/theme/mbo.css',
			'/media/editors/codemirror/theme/mdn-like.css',
			'/media/editors/codemirror/theme/midnight.css',
			'/media/editors/codemirror/theme/monokai.css',
			'/media/editors/codemirror/theme/neat.css',
			'/media/editors/codemirror/theme/neo.css',
			'/media/editors/codemirror/theme/night.css',
			'/media/editors/codemirror/theme/oceanic-next.css',
			'/media/editors/codemirror/theme/panda-syntax.css',
			'/media/editors/codemirror/theme/paraiso-dark.css',
			'/media/editors/codemirror/theme/paraiso-light.css',
			'/media/editors/codemirror/theme/pastel-on-dark.css',
			'/media/editors/codemirror/theme/railscasts.css',
			'/media/editors/codemirror/theme/rubyblue.css',
			'/media/editors/codemirror/theme/seti.css',
			'/media/editors/codemirror/theme/shadowfox.css',
			'/media/editors/codemirror/theme/solarized.css',
			'/media/editors/codemirror/theme/ssms.css',
			'/media/editors/codemirror/theme/the-matrix.css',
			'/media/editors/codemirror/theme/tomorrow-night-bright.css',
			'/media/editors/codemirror/theme/tomorrow-night-eighties.css',
			'/media/editors/codemirror/theme/ttcn.css',
			'/media/editors/codemirror/theme/twilight.css',
			'/media/editors/codemirror/theme/vibrant-ink.css',
			'/media/editors/codemirror/theme/xq-dark.css',
			'/media/editors/codemirror/theme/xq-light.css',
			'/media/editors/codemirror/theme/yeti.css',
			'/media/editors/codemirror/theme/zenburn.css',
			'/media/editors/none/js/none.js',
			'/media/editors/none/js/none.min.js',
			'/media/editors/tinymce/changelog.txt',
			'/media/editors/tinymce/js/plugins/dragdrop/plugin.js',
			'/media/editors/tinymce/js/plugins/dragdrop/plugin.min.js',
			'/media/editors/tinymce/js/tiny-close.js',
			'/media/editors/tinymce/js/tiny-close.min.js',
			'/media/editors/tinymce/js/tinymce-builder.js',
			'/media/editors/tinymce/js/tinymce.js',
			'/media/editors/tinymce/js/tinymce.min.js',
			'/media/editors/tinymce/langs/af.js',
			'/media/editors/tinymce/langs/ar.js',
			'/media/editors/tinymce/langs/be.js',
			'/media/editors/tinymce/langs/bg.js',
			'/media/editors/tinymce/langs/bs.js',
			'/media/editors/tinymce/langs/ca.js',
			'/media/editors/tinymce/langs/cs.js',
			'/media/editors/tinymce/langs/cy.js',
			'/media/editors/tinymce/langs/da.js',
			'/media/editors/tinymce/langs/de.js',
			'/media/editors/tinymce/langs/el.js',
			'/media/editors/tinymce/langs/es.js',
			'/media/editors/tinymce/langs/et.js',
			'/media/editors/tinymce/langs/eu.js',
			'/media/editors/tinymce/langs/fa.js',
			'/media/editors/tinymce/langs/fi.js',
			'/media/editors/tinymce/langs/fo.js',
			'/media/editors/tinymce/langs/fr.js',
			'/media/editors/tinymce/langs/ga.js',
			'/media/editors/tinymce/langs/gl.js',
			'/media/editors/tinymce/langs/he.js',
			'/media/editors/tinymce/langs/hr.js',
			'/media/editors/tinymce/langs/hu.js',
			'/media/editors/tinymce/langs/id.js',
			'/media/editors/tinymce/langs/it.js',
			'/media/editors/tinymce/langs/ja.js',
			'/media/editors/tinymce/langs/ka.js',
			'/media/editors/tinymce/langs/kk.js',
			'/media/editors/tinymce/langs/km.js',
			'/media/editors/tinymce/langs/ko.js',
			'/media/editors/tinymce/langs/lb.js',
			'/media/editors/tinymce/langs/lt.js',
			'/media/editors/tinymce/langs/lv.js',
			'/media/editors/tinymce/langs/mk.js',
			'/media/editors/tinymce/langs/ms.js',
			'/media/editors/tinymce/langs/nb.js',
			'/media/editors/tinymce/langs/nl.js',
			'/media/editors/tinymce/langs/pl.js',
			'/media/editors/tinymce/langs/pt-BR.js',
			'/media/editors/tinymce/langs/pt-PT.js',
			'/media/editors/tinymce/langs/readme.md',
			'/media/editors/tinymce/langs/ro.js',
			'/media/editors/tinymce/langs/ru.js',
			'/media/editors/tinymce/langs/si-LK.js',
			'/media/editors/tinymce/langs/sk.js',
			'/media/editors/tinymce/langs/sl.js',
			'/media/editors/tinymce/langs/sr.js',
			'/media/editors/tinymce/langs/sv.js',
			'/media/editors/tinymce/langs/sw.js',
			'/media/editors/tinymce/langs/sy.js',
			'/media/editors/tinymce/langs/ta.js',
			'/media/editors/tinymce/langs/th.js',
			'/media/editors/tinymce/langs/tr.js',
			'/media/editors/tinymce/langs/ug.js',
			'/media/editors/tinymce/langs/uk.js',
			'/media/editors/tinymce/langs/vi.js',
			'/media/editors/tinymce/langs/zh-CN.js',
			'/media/editors/tinymce/langs/zh-TW.js',
			'/media/editors/tinymce/license.txt',
			'/media/editors/tinymce/plugins/advlist/plugin.min.js',
			'/media/editors/tinymce/plugins/anchor/plugin.min.js',
			'/media/editors/tinymce/plugins/autolink/plugin.min.js',
			'/media/editors/tinymce/plugins/autoresize/plugin.min.js',
			'/media/editors/tinymce/plugins/autosave/plugin.min.js',
			'/media/editors/tinymce/plugins/bbcode/plugin.min.js',
			'/media/editors/tinymce/plugins/charmap/plugin.min.js',
			'/media/editors/tinymce/plugins/code/plugin.min.js',
			'/media/editors/tinymce/plugins/codesample/css/prism.css',
			'/media/editors/tinymce/plugins/codesample/plugin.min.js',
			'/media/editors/tinymce/plugins/colorpicker/plugin.min.js',
			'/media/editors/tinymce/plugins/contextmenu/plugin.min.js',
			'/media/editors/tinymce/plugins/directionality/plugin.min.js',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-cool.gif',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-cry.gif',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-embarassed.gif',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-foot-in-mouth.gif',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-frown.gif',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-innocent.gif',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-kiss.gif',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-laughing.gif',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-money-mouth.gif',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-sealed.gif',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-smile.gif',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-surprised.gif',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-tongue-out.gif',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-undecided.gif',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-wink.gif',
			'/media/editors/tinymce/plugins/emoticons/img/smiley-yell.gif',
			'/media/editors/tinymce/plugins/emoticons/plugin.min.js',
			'/media/editors/tinymce/plugins/example/dialog.html',
			'/media/editors/tinymce/plugins/example/plugin.min.js',
			'/media/editors/tinymce/plugins/example_dependency/plugin.min.js',
			'/media/editors/tinymce/plugins/fullpage/plugin.min.js',
			'/media/editors/tinymce/plugins/fullscreen/plugin.min.js',
			'/media/editors/tinymce/plugins/hr/plugin.min.js',
			'/media/editors/tinymce/plugins/image/plugin.min.js',
			'/media/editors/tinymce/plugins/imagetools/plugin.min.js',
			'/media/editors/tinymce/plugins/importcss/plugin.min.js',
			'/media/editors/tinymce/plugins/insertdatetime/plugin.min.js',
			'/media/editors/tinymce/plugins/layer/plugin.min.js',
			'/media/editors/tinymce/plugins/legacyoutput/plugin.min.js',
			'/media/editors/tinymce/plugins/link/plugin.min.js',
			'/media/editors/tinymce/plugins/lists/plugin.min.js',
			'/media/editors/tinymce/plugins/media/plugin.min.js',
			'/media/editors/tinymce/plugins/nonbreaking/plugin.min.js',
			'/media/editors/tinymce/plugins/noneditable/plugin.min.js',
			'/media/editors/tinymce/plugins/pagebreak/plugin.min.js',
			'/media/editors/tinymce/plugins/paste/plugin.min.js',
			'/media/editors/tinymce/plugins/preview/plugin.min.js',
			'/media/editors/tinymce/plugins/print/plugin.min.js',
			'/media/editors/tinymce/plugins/save/plugin.min.js',
			'/media/editors/tinymce/plugins/searchreplace/plugin.min.js',
			'/media/editors/tinymce/plugins/spellchecker/plugin.min.js',
			'/media/editors/tinymce/plugins/tabfocus/plugin.min.js',
			'/media/editors/tinymce/plugins/table/plugin.min.js',
			'/media/editors/tinymce/plugins/template/plugin.min.js',
			'/media/editors/tinymce/plugins/textcolor/plugin.min.js',
			'/media/editors/tinymce/plugins/textpattern/plugin.min.js',
			'/media/editors/tinymce/plugins/toc/plugin.min.js',
			'/media/editors/tinymce/plugins/visualblocks/css/visualblocks.css',
			'/media/editors/tinymce/plugins/visualblocks/plugin.min.js',
			'/media/editors/tinymce/plugins/visualchars/plugin.min.js',
			'/media/editors/tinymce/plugins/wordcount/plugin.min.js',
			'/media/editors/tinymce/skins/lightgray/content.inline.min.css',
			'/media/editors/tinymce/skins/lightgray/content.min.css',
			'/media/editors/tinymce/skins/lightgray/fonts/tinymce-small.eot',
			'/media/editors/tinymce/skins/lightgray/fonts/tinymce-small.svg',
			'/media/editors/tinymce/skins/lightgray/fonts/tinymce-small.ttf',
			'/media/editors/tinymce/skins/lightgray/fonts/tinymce-small.woff',
			'/media/editors/tinymce/skins/lightgray/fonts/tinymce.eot',
			'/media/editors/tinymce/skins/lightgray/fonts/tinymce.svg',
			'/media/editors/tinymce/skins/lightgray/fonts/tinymce.ttf',
			'/media/editors/tinymce/skins/lightgray/fonts/tinymce.woff',
			'/media/editors/tinymce/skins/lightgray/img/anchor.gif',
			'/media/editors/tinymce/skins/lightgray/img/loader.gif',
			'/media/editors/tinymce/skins/lightgray/img/object.gif',
			'/media/editors/tinymce/skins/lightgray/img/trans.gif',
			'/media/editors/tinymce/skins/lightgray/skin.ie7.min.css',
			'/media/editors/tinymce/skins/lightgray/skin.min.css',
			'/media/editors/tinymce/templates/layout1.html',
			'/media/editors/tinymce/templates/snippet1.html',
			'/media/editors/tinymce/themes/modern/theme.min.js',
			'/media/editors/tinymce/tinymce.min.js',
			'/media/jui/css/bootstrap-extended.css',
			'/media/jui/css/bootstrap-responsive.css',
			'/media/jui/css/bootstrap-responsive.min.css',
			'/media/jui/css/bootstrap-rtl.css',
			'/media/jui/css/bootstrap-tooltip-extended.css',
			'/media/jui/css/bootstrap.css',
			'/media/jui/css/bootstrap.min.css',
			'/media/jui/css/chosen-sprite.png',
			'/media/jui/css/chosen-sprite@2x.png',
			'/media/jui/css/chosen.css',
			'/media/jui/css/icomoon.css',
			'/media/jui/css/jquery.minicolors.css',
			'/media/jui/css/jquery.searchtools.css',
			'/media/jui/css/jquery.simplecolors.css',
			'/media/jui/css/sortablelist.css',
			'/media/jui/fonts/IcoMoon.dev.commented.svg',
			'/media/jui/fonts/IcoMoon.dev.svg',
			'/media/jui/fonts/IcoMoon.eot',
			'/media/jui/fonts/IcoMoon.svg',
			'/media/jui/fonts/IcoMoon.ttf',
			'/media/jui/fonts/IcoMoon.woff',
			'/media/jui/fonts/icomoon-license.txt',
			'/media/jui/images/ajax-loader.gif',
			'/media/jui/img/ajax-loader.gif',
			'/media/jui/img/alpha.png',
			'/media/jui/img/bg-overlay.png',
			'/media/jui/img/glyphicons-halflings-white.png',
			'/media/jui/img/glyphicons-halflings.png',
			'/media/jui/img/hue.png',
			'/media/jui/img/joomla.png',
			'/media/jui/img/jquery.minicolors.png',
			'/media/jui/img/saturation.png',
			'/media/jui/js/ajax-chosen.js',
			'/media/jui/js/ajax-chosen.min.js',
			'/media/jui/js/bootstrap-tooltip-extended.js',
			'/media/jui/js/bootstrap-tooltip-extended.min.js',
			'/media/jui/js/bootstrap.js',
			'/media/jui/js/bootstrap.min.js',
			'/media/jui/js/chosen.jquery.js',
			'/media/jui/js/chosen.jquery.min.js',
			'/media/jui/js/cms-uncompressed.js',
			'/media/jui/js/cms.js',
			'/media/jui/js/fielduser.js',
			'/media/jui/js/fielduser.min.js',
			'/media/jui/js/html5-uncompressed.js',
			'/media/jui/js/html5.js',
			'/media/jui/js/icomoon-lte-ie7.js',
			'/media/jui/js/jquery-migrate.js',
			'/media/jui/js/jquery-migrate.min.js',
			'/media/jui/js/jquery-noconflict.js',
			'/media/jui/js/jquery.autocomplete.js',
			'/media/jui/js/jquery.autocomplete.min.js',
			'/media/jui/js/jquery.js',
			'/media/jui/js/jquery.min.js',
			'/media/jui/js/jquery.minicolors.js',
			'/media/jui/js/jquery.minicolors.min.js',
			'/media/jui/js/jquery.searchtools.js',
			'/media/jui/js/jquery.searchtools.min.js',
			'/media/jui/js/jquery.simplecolors.js',
			'/media/jui/js/jquery.simplecolors.min.js',
			'/media/jui/js/jquery.ui.core.js',
			'/media/jui/js/jquery.ui.core.min.js',
			'/media/jui/js/jquery.ui.sortable.js',
			'/media/jui/js/jquery.ui.sortable.min.js',
			'/media/jui/js/sortablelist.js',
			'/media/jui/js/treeselectmenu.jquery.js',
			'/media/jui/js/treeselectmenu.jquery.min.js',
			'/media/jui/less/accordion.less',
			'/media/jui/less/alerts.less',
			'/media/jui/less/bootstrap-extended.less',
			'/media/jui/less/bootstrap-rtl.less',
			'/media/jui/less/bootstrap.less',
			'/media/jui/less/breadcrumbs.less',
			'/media/jui/less/button-groups.less',
			'/media/jui/less/buttons.less',
			'/media/jui/less/carousel.less',
			'/media/jui/less/close.less',
			'/media/jui/less/code.less',
			'/media/jui/less/component-animations.less',
			'/media/jui/less/dropdowns.less',
			'/media/jui/less/forms.less',
			'/media/jui/less/grid.less',
			'/media/jui/less/hero-unit.less',
			'/media/jui/less/icomoon.less',
			'/media/jui/less/labels-badges.less',
			'/media/jui/less/layouts.less',
			'/media/jui/less/media.less',
			'/media/jui/less/mixins.less',
			'/media/jui/less/modals.joomla.less',
			'/media/jui/less/modals.less',
			'/media/jui/less/navbar.less',
			'/media/jui/less/navs.less',
			'/media/jui/less/pager.less',
			'/media/jui/less/pagination.less',
			'/media/jui/less/popovers.less',
			'/media/jui/less/progress-bars.less',
			'/media/jui/less/reset.less',
			'/media/jui/less/responsive-1200px-min.less',
			'/media/jui/less/responsive-767px-max.joomla.less',
			'/media/jui/less/responsive-767px-max.less',
			'/media/jui/less/responsive-768px-979px.less',
			'/media/jui/less/responsive-navbar.less',
			'/media/jui/less/responsive-utilities.less',
			'/media/jui/less/responsive.less',
			'/media/jui/less/scaffolding.less',
			'/media/jui/less/sprites.less',
			'/media/jui/less/tables.less',
			'/media/jui/less/thumbnails.less',
			'/media/jui/less/tooltip.less',
			'/media/jui/less/type.less',
			'/media/jui/less/utilities.less',
			'/media/jui/less/variables.less',
			'/media/jui/less/wells.less',
			'/media/media/css/background.png',
			'/media/media/css/bigplay.fw.png',
			'/media/media/css/bigplay.png',
			'/media/media/css/bigplay.svg',
			'/media/media/css/controls-ted.png',
			'/media/media/css/controls-wmp-bg.png',
			'/media/media/css/controls-wmp.png',
			'/media/media/css/controls.fw.png',
			'/media/media/css/controls.png',
			'/media/media/css/controls.svg',
			'/media/media/css/jumpforward.png',
			'/media/media/css/loading.gif',
			'/media/media/css/mediaelementplayer.css',
			'/media/media/css/mediaelementplayer.min.css',
			'/media/media/css/medialist-details.css',
			'/media/media/css/medialist-details_rtl.css',
			'/media/media/css/medialist-thumbs.css',
			'/media/media/css/medialist-thumbs_rtl.css',
			'/media/media/css/mediamanager.css',
			'/media/media/css/mediamanager_rtl.css',
			'/media/media/css/mejs-skins.css',
			'/media/media/css/popup-imagelist.css',
			'/media/media/css/popup-imagelist_rtl.css',
			'/media/media/css/popup-imagemanager.css',
			'/media/media/css/popup-imagemanager_rtl.css',
			'/media/media/css/skipback.png',
			'/media/media/images/bar.gif',
			'/media/media/images/con_info.png',
			'/media/media/images/delete.png',
			'/media/media/images/dots.gif',
			'/media/media/images/failed.png',
			'/media/media/images/folder.gif',
			'/media/media/images/folder.png',
			'/media/media/images/folder_sm.png',
			'/media/media/images/folderup_16.png',
			'/media/media/images/folderup_32.png',
			'/media/media/images/mime-icon-16/avi.png',
			'/media/media/images/mime-icon-16/doc.png',
			'/media/media/images/mime-icon-16/mov.png',
			'/media/media/images/mime-icon-16/mp3.png',
			'/media/media/images/mime-icon-16/mp4.png',
			'/media/media/images/mime-icon-16/odc.png',
			'/media/media/images/mime-icon-16/odd.png',
			'/media/media/images/mime-icon-16/odt.png',
			'/media/media/images/mime-icon-16/ogg.png',
			'/media/media/images/mime-icon-16/pdf.png',
			'/media/media/images/mime-icon-16/ppt.png',
			'/media/media/images/mime-icon-16/rar.png',
			'/media/media/images/mime-icon-16/rtf.png',
			'/media/media/images/mime-icon-16/svg.png',
			'/media/media/images/mime-icon-16/sxd.png',
			'/media/media/images/mime-icon-16/tar.png',
			'/media/media/images/mime-icon-16/tgz.png',
			'/media/media/images/mime-icon-16/wma.png',
			'/media/media/images/mime-icon-16/wmv.png',
			'/media/media/images/mime-icon-16/xls.png',
			'/media/media/images/mime-icon-16/zip.png',
			'/media/media/images/mime-icon-32/avi.png',
			'/media/media/images/mime-icon-32/doc.png',
			'/media/media/images/mime-icon-32/mov.png',
			'/media/media/images/mime-icon-32/mp3.png',
			'/media/media/images/mime-icon-32/mp4.png',
			'/media/media/images/mime-icon-32/odc.png',
			'/media/media/images/mime-icon-32/odd.png',
			'/media/media/images/mime-icon-32/odt.png',
			'/media/media/images/mime-icon-32/ogg.png',
			'/media/media/images/mime-icon-32/pdf.png',
			'/media/media/images/mime-icon-32/ppt.png',
			'/media/media/images/mime-icon-32/rar.png',
			'/media/media/images/mime-icon-32/rtf.png',
			'/media/media/images/mime-icon-32/svg.png',
			'/media/media/images/mime-icon-32/sxd.png',
			'/media/media/images/mime-icon-32/tar.png',
			'/media/media/images/mime-icon-32/tgz.png',
			'/media/media/images/mime-icon-32/wma.png',
			'/media/media/images/mime-icon-32/wmv.png',
			'/media/media/images/mime-icon-32/xls.png',
			'/media/media/images/mime-icon-32/zip.png',
			'/media/media/images/progress.gif',
			'/media/media/images/remove.png',
			'/media/media/images/success.png',
			'/media/media/images/upload.png',
			'/media/media/images/uploading.png',
			'/media/media/js/flashmediaelement-cdn.swf',
			'/media/media/js/flashmediaelement.swf',
			'/media/media/js/mediaelement-and-player.js',
			'/media/media/js/mediaelement-and-player.min.js',
			'/media/media/js/mediafield-mootools.js',
			'/media/media/js/mediafield-mootools.min.js',
			'/media/media/js/mediafield.js',
			'/media/media/js/mediafield.min.js',
			'/media/media/js/mediamanager.js',
			'/media/media/js/mediamanager.min.js',
			'/media/media/js/popup-imagemanager.js',
			'/media/media/js/popup-imagemanager.min.js',
			'/media/media/js/silverlightmediaelement.xap',
			'/media/overrider/css/overrider.css',
			'/media/overrider/js/overrider.js',
			'/media/overrider/js/overrider.min.js',
			'/media/plg_twofactorauth_totp/js/qrcode.js',
			'/media/plg_twofactorauth_totp/js/qrcode.min.js',
			'/media/plg_twofactorauth_totp/js/qrcode_SJIS.js',
			'/media/plg_twofactorauth_totp/js/qrcode_UTF8.js',
			'/media/system/css/adminlist.css',
			'/media/system/css/jquery.Jcrop.min.css',
			'/media/system/css/modal.css',
			'/media/system/images/modal/bg_e.png',
			'/media/system/images/modal/bg_n.png',
			'/media/system/images/modal/bg_ne.png',
			'/media/system/images/modal/bg_nw.png',
			'/media/system/images/modal/bg_s.png',
			'/media/system/images/modal/bg_se.png',
			'/media/system/images/modal/bg_sw.png',
			'/media/system/images/modal/bg_w.png',
			'/media/system/images/modal/closebox.png',
			'/media/system/images/modal/spinner.gif',
			'/media/system/js/associations-edit-uncompressed.js',
			'/media/system/js/associations-edit.js',
			'/media/system/js/calendar-setup-uncompressed.js',
			'/media/system/js/calendar-setup.js',
			'/media/system/js/calendar-uncompressed.js',
			'/media/system/js/calendar.js',
			'/media/system/js/caption-uncompressed.js',
			'/media/system/js/caption.js',
			'/media/system/js/color-field-adv-init.js',
			'/media/system/js/color-field-adv-init.min.js',
			'/media/system/js/color-field-init.js',
			'/media/system/js/color-field-init.min.js',
			'/media/system/js/combobox-uncompressed.js',
			'/media/system/js/combobox.js',
			'/media/system/js/core-uncompressed.js',
			'/media/system/js/frontediting-uncompressed.js',
			'/media/system/js/frontediting.js',
			'/media/system/js/helpsite.js',
			'/media/system/js/highlighter-uncompressed.js',
			'/media/system/js/highlighter.js',
			'/media/system/js/html5fallback-uncompressed.js',
			'/media/system/js/html5fallback.js',
			'/media/system/js/jquery.Jcrop.js',
			'/media/system/js/jquery.Jcrop.min.js',
			'/media/system/js/keepalive-uncompressed.js',
			'/media/system/js/modal-fields-uncompressed.js',
			'/media/system/js/modal-fields.js',
			'/media/system/js/modal-uncompressed.js',
			'/media/system/js/modal.js',
			'/media/system/js/moduleorder.js',
			'/media/system/js/mootools-core-uncompressed.js',
			'/media/system/js/mootools-core.js',
			'/media/system/js/mootools-more-uncompressed.js',
			'/media/system/js/mootools-more.js',
			'/media/system/js/mootree-uncompressed.js',
			'/media/system/js/mootree.js',
			'/media/system/js/multiselect-uncompressed.js',
			'/media/system/js/passwordstrength.js',
			'/media/system/js/permissions-uncompressed.js',
			'/media/system/js/permissions.js',
			'/media/system/js/polyfill.classlist-uncompressed.js',
			'/media/system/js/polyfill.classlist.js',
			'/media/system/js/polyfill.event-uncompressed.js',
			'/media/system/js/polyfill.event.js',
			'/media/system/js/polyfill.filter-uncompressed.js',
			'/media/system/js/polyfill.filter.js',
			'/media/system/js/polyfill.map-uncompressed.js',
			'/media/system/js/polyfill.map.js',
			'/media/system/js/polyfill.xpath-uncompressed.js',
			'/media/system/js/polyfill.xpath.js',
			'/media/system/js/progressbar-uncompressed.js',
			'/media/system/js/progressbar.js',
			'/media/system/js/punycode-uncompressed.js',
			'/media/system/js/punycode.js',
			'/media/system/js/repeatable-uncompressed.js',
			'/media/system/js/repeatable.js',
			'/media/system/js/sendtestmail-uncompressed.js',
			'/media/system/js/sendtestmail.js',
			'/media/system/js/subform-repeatable-uncompressed.js',
			'/media/system/js/subform-repeatable.js',
			'/media/system/js/switcher-uncompressed.js',
			'/media/system/js/switcher.js',
			'/media/system/js/tabs-state-uncompressed.js',
			'/media/system/js/tabs-state.js',
			'/media/system/js/tabs.js',
			'/media/system/js/validate-uncompressed.js',
			'/media/system/js/validate.js',
			'/modules/mod_articles_archive/helper.php',
			'/modules/mod_articles_categories/helper.php',
			'/modules/mod_articles_category/helper.php',
			'/modules/mod_articles_latest/helper.php',
			'/modules/mod_articles_news/helper.php',
			'/modules/mod_articles_popular/helper.php',
			'/modules/mod_banners/helper.php',
			'/modules/mod_breadcrumbs/helper.php',
			'/modules/mod_feed/helper.php',
			'/modules/mod_finder/helper.php',
			'/modules/mod_languages/helper.php',
			'/modules/mod_login/helper.php',
			'/modules/mod_menu/helper.php',
			'/modules/mod_random_image/helper.php',
			'/modules/mod_related_items/helper.php',
			'/modules/mod_search/helper.php',
			'/modules/mod_search/mod_search.php',
			'/modules/mod_search/mod_search.xml',
			'/modules/mod_search/tmpl/default.php',
			'/modules/mod_stats/helper.php',
			'/modules/mod_syndicate/helper.php',
			'/modules/mod_tags_popular/helper.php',
			'/modules/mod_tags_similar/helper.php',
			'/modules/mod_users_latest/helper.php',
			'/modules/mod_whosonline/helper.php',
			'/modules/mod_wrapper/helper.php',
			'/plugins/authentication/gmail/gmail.php',
			'/plugins/authentication/gmail/gmail.xml',
			'/plugins/captcha/recaptcha/postinstall/actions.php',
			'/plugins/content/confirmconsent/fields/consentbox.php',
			'/plugins/editors/codemirror/fonts.php',
			'/plugins/editors/codemirror/layouts/editors/codemirror/init.php',
			'/plugins/editors/tinymce/field/skins.php',
			'/plugins/editors/tinymce/field/tinymcebuilder.php',
			'/plugins/editors/tinymce/field/uploaddirs.php',
			'/plugins/editors/tinymce/form/setoptions.xml',
			'/plugins/quickicon/joomlaupdate/joomlaupdate.php',
			'/plugins/system/languagecode/language/en-GB/en-GB.plg_system_languagecode.ini',
			'/plugins/system/languagecode/language/en-GB/en-GB.plg_system_languagecode.sys.ini',
			'/plugins/system/p3p/p3p.php',
			'/plugins/system/p3p/p3p.xml',
			'/plugins/system/privacyconsent/field/privacy.php',
			'/plugins/system/privacyconsent/privacyconsent/privacyconsent.xml',
			'/plugins/system/stats/field/base.php',
			'/plugins/system/stats/field/data.php',
			'/plugins/system/stats/field/uniqueid.php',
			'/plugins/user/profile/field/dob.php',
			'/plugins/user/profile/field/tos.php',
			'/plugins/user/profile/profiles/profile.xml',
			'/plugins/user/terms/field/terms.php',
			'/plugins/user/terms/terms/terms.xml',
			'/templates/beez3/component.php',
			'/templates/beez3/css/general.css',
			'/templates/beez3/css/ie7only.css',
			'/templates/beez3/css/ieonly.css',
			'/templates/beez3/css/layout.css',
			'/templates/beez3/css/nature.css',
			'/templates/beez3/css/nature_rtl.css',
			'/templates/beez3/css/personal.css',
			'/templates/beez3/css/personal_rtl.css',
			'/templates/beez3/css/position.css',
			'/templates/beez3/css/print.css',
			'/templates/beez3/css/red.css',
			'/templates/beez3/css/template.css',
			'/templates/beez3/css/template_rtl.css',
			'/templates/beez3/css/turq.css',
			'/templates/beez3/css/turq.less',
			'/templates/beez3/error.php',
			'/templates/beez3/favicon.ico',
			'/templates/beez3/html/com_contact/categories/default.php',
			'/templates/beez3/html/com_contact/categories/default_items.php',
			'/templates/beez3/html/com_contact/category/default.php',
			'/templates/beez3/html/com_contact/category/default_children.php',
			'/templates/beez3/html/com_contact/category/default_items.php',
			'/templates/beez3/html/com_contact/contact/default.php',
			'/templates/beez3/html/com_contact/contact/default_address.php',
			'/templates/beez3/html/com_contact/contact/default_articles.php',
			'/templates/beez3/html/com_contact/contact/default_form.php',
			'/templates/beez3/html/com_contact/contact/default_links.php',
			'/templates/beez3/html/com_contact/contact/default_profile.php',
			'/templates/beez3/html/com_contact/contact/default_user_custom_fields.php',
			'/templates/beez3/html/com_contact/contact/encyclopedia.php',
			'/templates/beez3/html/com_content/archive/default.php',
			'/templates/beez3/html/com_content/archive/default_items.php',
			'/templates/beez3/html/com_content/article/default.php',
			'/templates/beez3/html/com_content/article/default_links.php',
			'/templates/beez3/html/com_content/categories/default.php',
			'/templates/beez3/html/com_content/categories/default_items.php',
			'/templates/beez3/html/com_content/category/blog.php',
			'/templates/beez3/html/com_content/category/blog_children.php',
			'/templates/beez3/html/com_content/category/blog_item.php',
			'/templates/beez3/html/com_content/category/blog_links.php',
			'/templates/beez3/html/com_content/category/default.php',
			'/templates/beez3/html/com_content/category/default_articles.php',
			'/templates/beez3/html/com_content/category/default_children.php',
			'/templates/beez3/html/com_content/featured/default.php',
			'/templates/beez3/html/com_content/featured/default_item.php',
			'/templates/beez3/html/com_content/featured/default_links.php',
			'/templates/beez3/html/com_content/form/edit.php',
			'/templates/beez3/html/com_newsfeeds/categories/default.php',
			'/templates/beez3/html/com_newsfeeds/categories/default_items.php',
			'/templates/beez3/html/com_newsfeeds/category/default.php',
			'/templates/beez3/html/com_newsfeeds/category/default_children.php',
			'/templates/beez3/html/com_newsfeeds/category/default_items.php',
			'/templates/beez3/html/com_weblinks/categories/default.php',
			'/templates/beez3/html/com_weblinks/categories/default_items.php',
			'/templates/beez3/html/com_weblinks/category/default.php',
			'/templates/beez3/html/com_weblinks/category/default_children.php',
			'/templates/beez3/html/com_weblinks/category/default_items.php',
			'/templates/beez3/html/com_weblinks/form/edit.php',
			'/templates/beez3/html/layouts/joomla/system/message.php',
			'/templates/beez3/html/mod_breadcrumbs/default.php',
			'/templates/beez3/html/mod_languages/default.php',
			'/templates/beez3/html/mod_login/default.php',
			'/templates/beez3/html/mod_login/default_logout.php',
			'/templates/beez3/html/modules.php',
			'/templates/beez3/images/all_bg.gif',
			'/templates/beez3/images/arrow.png',
			'/templates/beez3/images/arrow2_grey.png',
			'/templates/beez3/images/arrow_white_grey.png',
			'/templates/beez3/images/blog_more.gif',
			'/templates/beez3/images/blog_more_hover.gif',
			'/templates/beez3/images/close.png',
			'/templates/beez3/images/content_bg.gif',
			'/templates/beez3/images/footer_bg.gif',
			'/templates/beez3/images/footer_bg.png',
			'/templates/beez3/images/header-bg.gif',
			'/templates/beez3/images/minus.png',
			'/templates/beez3/images/nature/arrow1.gif',
			'/templates/beez3/images/nature/arrow1_rtl.gif',
			'/templates/beez3/images/nature/arrow2.gif',
			'/templates/beez3/images/nature/arrow2_grey.png',
			'/templates/beez3/images/nature/arrow2_rtl.gif',
			'/templates/beez3/images/nature/arrow_nav.gif',
			'/templates/beez3/images/nature/arrow_small.png',
			'/templates/beez3/images/nature/arrow_small_rtl.png',
			'/templates/beez3/images/nature/blog_more.gif',
			'/templates/beez3/images/nature/box.png',
			'/templates/beez3/images/nature/box1.png',
			'/templates/beez3/images/nature/grey_bg.png',
			'/templates/beez3/images/nature/headingback.png',
			'/templates/beez3/images/nature/karo.gif',
			'/templates/beez3/images/nature/level4.png',
			'/templates/beez3/images/nature/nav_level1_a.gif',
			'/templates/beez3/images/nature/nav_level_1.gif',
			'/templates/beez3/images/nature/pfeil.gif',
			'/templates/beez3/images/nature/readmore_arrow.png',
			'/templates/beez3/images/nature/searchbutton.png',
			'/templates/beez3/images/nature/tabs.gif',
			'/templates/beez3/images/nav_level_1.gif',
			'/templates/beez3/images/news.gif',
			'/templates/beez3/images/personal/arrow2_grey.jpg',
			'/templates/beez3/images/personal/arrow2_grey.png',
			'/templates/beez3/images/personal/bg2.png',
			'/templates/beez3/images/personal/button.png',
			'/templates/beez3/images/personal/dot.png',
			'/templates/beez3/images/personal/ecke.gif',
			'/templates/beez3/images/personal/footer.jpg',
			'/templates/beez3/images/personal/grey_bg.png',
			'/templates/beez3/images/personal/navi_active.png',
			'/templates/beez3/images/personal/personal2.png',
			'/templates/beez3/images/personal/readmore_arrow.png',
			'/templates/beez3/images/personal/readmore_arrow_hover.png',
			'/templates/beez3/images/personal/tabs_back.png',
			'/templates/beez3/images/plus.png',
			'/templates/beez3/images/req.png',
			'/templates/beez3/images/slider_minus.png',
			'/templates/beez3/images/slider_minus_rtl.png',
			'/templates/beez3/images/slider_plus.png',
			'/templates/beez3/images/slider_plus_rtl.png',
			'/templates/beez3/images/system/arrow.png',
			'/templates/beez3/images/system/arrow_rtl.png',
			'/templates/beez3/images/system/calendar.png',
			'/templates/beez3/images/system/j_button2_blank.png',
			'/templates/beez3/images/system/j_button2_image.png',
			'/templates/beez3/images/system/j_button2_left.png',
			'/templates/beez3/images/system/j_button2_pagebreak.png',
			'/templates/beez3/images/system/j_button2_readmore.png',
			'/templates/beez3/images/system/notice-alert.png',
			'/templates/beez3/images/system/notice-alert_rtl.png',
			'/templates/beez3/images/system/notice-info.png',
			'/templates/beez3/images/system/notice-info_rtl.png',
			'/templates/beez3/images/system/notice-note.png',
			'/templates/beez3/images/system/notice-note_rtl.png',
			'/templates/beez3/images/system/selector-arrow.png',
			'/templates/beez3/images/table_footer.gif',
			'/templates/beez3/images/trans.gif',
			'/templates/beez3/index.php',
			'/templates/beez3/javascript/hide.js',
			'/templates/beez3/javascript/md_stylechanger.js',
			'/templates/beez3/javascript/respond.js',
			'/templates/beez3/javascript/respond.src.js',
			'/templates/beez3/javascript/template.js',
			'/templates/beez3/jsstrings.php',
			'/templates/beez3/language/en-GB/en-GB.tpl_beez3.ini',
			'/templates/beez3/language/en-GB/en-GB.tpl_beez3.sys.ini',
			'/templates/beez3/templateDetails.xml',
			'/templates/beez3/template_preview.png',
			'/templates/beez3/template_thumbnail.png',
			'/templates/protostar/component.php',
			'/templates/protostar/css/offline.css',
			'/templates/protostar/css/template.css',
			'/templates/protostar/error.php',
			'/templates/protostar/favicon.ico',
			'/templates/protostar/html/com_media/imageslist/default_folder.php',
			'/templates/protostar/html/com_media/imageslist/default_image.php',
			'/templates/protostar/html/layouts/joomla/form/field/contenthistory.php',
			'/templates/protostar/html/layouts/joomla/form/field/media.php',
			'/templates/protostar/html/layouts/joomla/form/field/user.php',
			'/templates/protostar/html/layouts/joomla/system/message.php',
			'/templates/protostar/html/modules.php',
			'/templates/protostar/html/pagination.php',
			'/templates/protostar/images/logo.png',
			'/templates/protostar/images/system/rating_star.png',
			'/templates/protostar/images/system/rating_star_blank.png',
			'/templates/protostar/images/system/sort_asc.png',
			'/templates/protostar/images/system/sort_desc.png',
			'/templates/protostar/img/glyphicons-halflings-white.png',
			'/templates/protostar/img/glyphicons-halflings.png',
			'/templates/protostar/index.php',
			'/templates/protostar/js/application.js',
			'/templates/protostar/js/classes.js',
			'/templates/protostar/js/template.js',
			'/templates/protostar/language/en-GB/en-GB.tpl_protostar.ini',
			'/templates/protostar/language/en-GB/en-GB.tpl_protostar.sys.ini',
			'/templates/protostar/less/icomoon.less',
			'/templates/protostar/less/template.less',
			'/templates/protostar/less/template_rtl.less',
			'/templates/protostar/less/variables.less',
			'/templates/protostar/offline.php',
			'/templates/protostar/templateDetails.xml',
			'/templates/protostar/template_preview.png',
			'/templates/protostar/template_thumbnail.png',
			'/templates/system/css/system.css',
			'/templates/system/css/toolbar.css',
			'/templates/system/html/modules.php',
			'/templates/system/images/calendar.png',
			'/templates/system/images/j_button2_blank.png',
			'/templates/system/images/j_button2_image.png',
			'/templates/system/images/j_button2_left.png',
			'/templates/system/images/j_button2_pagebreak.png',
			'/templates/system/images/j_button2_readmore.png',
			'/templates/system/images/j_button2_right.png',
			'/templates/system/images/selector-arrow.png',
		);

		// TODO There is an issue while deleting folders using the ftp mode
		$folders = array(
			// Joomla! 4.0
			'/templates/system/images',
			'/templates/system/html',
			'/templates/protostar/less',
			'/templates/protostar/language/en-GB',
			'/templates/protostar/language',
			'/templates/protostar/js',
			'/templates/protostar/img',
			'/templates/protostar/images/system',
			'/templates/protostar/images',
			'/templates/protostar/html/layouts/joomla/system',
			'/templates/protostar/html/layouts/joomla/form/field',
			'/templates/protostar/html/layouts/joomla/form',
			'/templates/protostar/html/layouts/joomla',
			'/templates/protostar/html/layouts',
			'/templates/protostar/html/com_media/imageslist',
			'/templates/protostar/html/com_media',
			'/templates/protostar/html',
			'/templates/protostar/css',
			'/templates/protostar',
			'/templates/beez3/language/en-GB',
			'/templates/beez3/language',
			'/templates/beez3/javascript',
			'/templates/beez3/images/system',
			'/templates/beez3/images/personal',
			'/templates/beez3/images/nature',
			'/templates/beez3/images',
			'/templates/beez3/html/mod_login',
			'/templates/beez3/html/mod_languages',
			'/templates/beez3/html/mod_breadcrumbs',
			'/templates/beez3/html/layouts/joomla/system',
			'/templates/beez3/html/layouts/joomla',
			'/templates/beez3/html/layouts',
			'/templates/beez3/html/com_weblinks/form',
			'/templates/beez3/html/com_weblinks/category',
			'/templates/beez3/html/com_weblinks/categories',
			'/templates/beez3/html/com_weblinks',
			'/templates/beez3/html/com_newsfeeds/category',
			'/templates/beez3/html/com_newsfeeds/categories',
			'/templates/beez3/html/com_newsfeeds',
			'/templates/beez3/html/com_content/form',
			'/templates/beez3/html/com_content/featured',
			'/templates/beez3/html/com_content/category',
			'/templates/beez3/html/com_content/categories',
			'/templates/beez3/html/com_content/article',
			'/templates/beez3/html/com_content/archive',
			'/templates/beez3/html/com_content',
			'/templates/beez3/html/com_contact/contact',
			'/templates/beez3/html/com_contact/category',
			'/templates/beez3/html/com_contact/categories',
			'/templates/beez3/html/com_contact',
			'/templates/beez3/html',
			'/templates/beez3/css',
			'/templates/beez3',
			'/plugins/user/terms/terms',
			'/plugins/user/terms/field',
			'/plugins/user/profile/profiles',
			'/plugins/user/profile/field',
			'/plugins/system/stats/field',
			'/plugins/system/privacyconsent/privacyconsent',
			'/plugins/system/privacyconsent/field',
			'/plugins/system/p3p',
			'/plugins/system/languagecode/language/en-GB',
			'/plugins/system/languagecode/language',
			'/plugins/editors/tinymce/form',
			'/plugins/editors/tinymce/field',
			'/plugins/content/confirmconsent/fields',
			'/plugins/captcha/recaptcha/postinstall',
			'/plugins/authentication/gmail',
			'/modules/mod_search/tmpl',
			'/modules/mod_search',
			'/media/system/images/modal',
			'/media/plg_twofactorauth_totp/js',
			'/media/plg_twofactorauth_totp',
			'/media/overrider/js',
			'/media/overrider/css',
			'/media/overrider',
			'/media/media/js',
			'/media/media/images/mime-icon-32',
			'/media/media/images/mime-icon-16',
			'/media/media/images',
			'/media/media/css',
			'/media/media',
			'/media/jui/less',
			'/media/jui/js',
			'/media/jui/img',
			'/media/jui/images',
			'/media/jui/fonts',
			'/media/jui/css',
			'/media/jui',
			'/media/editors/tinymce/themes/modern',
			'/media/editors/tinymce/themes',
			'/media/editors/tinymce/templates',
			'/media/editors/tinymce/skins/lightgray/img',
			'/media/editors/tinymce/skins/lightgray/fonts',
			'/media/editors/tinymce/skins/lightgray',
			'/media/editors/tinymce/skins',
			'/media/editors/tinymce/plugins/wordcount',
			'/media/editors/tinymce/plugins/visualchars',
			'/media/editors/tinymce/plugins/visualblocks/css',
			'/media/editors/tinymce/plugins/visualblocks',
			'/media/editors/tinymce/plugins/toc',
			'/media/editors/tinymce/plugins/textpattern',
			'/media/editors/tinymce/plugins/textcolor',
			'/media/editors/tinymce/plugins/template',
			'/media/editors/tinymce/plugins/table',
			'/media/editors/tinymce/plugins/tabfocus',
			'/media/editors/tinymce/plugins/spellchecker',
			'/media/editors/tinymce/plugins/searchreplace',
			'/media/editors/tinymce/plugins/save',
			'/media/editors/tinymce/plugins/print',
			'/media/editors/tinymce/plugins/preview',
			'/media/editors/tinymce/plugins/paste',
			'/media/editors/tinymce/plugins/pagebreak',
			'/media/editors/tinymce/plugins/noneditable',
			'/media/editors/tinymce/plugins/nonbreaking',
			'/media/editors/tinymce/plugins/media',
			'/media/editors/tinymce/plugins/lists',
			'/media/editors/tinymce/plugins/link',
			'/media/editors/tinymce/plugins/legacyoutput',
			'/media/editors/tinymce/plugins/layer',
			'/media/editors/tinymce/plugins/insertdatetime',
			'/media/editors/tinymce/plugins/importcss',
			'/media/editors/tinymce/plugins/imagetools',
			'/media/editors/tinymce/plugins/image',
			'/media/editors/tinymce/plugins/hr',
			'/media/editors/tinymce/plugins/fullscreen',
			'/media/editors/tinymce/plugins/fullpage',
			'/media/editors/tinymce/plugins/example_dependency',
			'/media/editors/tinymce/plugins/example',
			'/media/editors/tinymce/plugins/emoticons/img',
			'/media/editors/tinymce/plugins/emoticons',
			'/media/editors/tinymce/plugins/directionality',
			'/media/editors/tinymce/plugins/contextmenu',
			'/media/editors/tinymce/plugins/colorpicker',
			'/media/editors/tinymce/plugins/codesample/css',
			'/media/editors/tinymce/plugins/codesample',
			'/media/editors/tinymce/plugins/code',
			'/media/editors/tinymce/plugins/charmap',
			'/media/editors/tinymce/plugins/bbcode',
			'/media/editors/tinymce/plugins/autosave',
			'/media/editors/tinymce/plugins/autoresize',
			'/media/editors/tinymce/plugins/autolink',
			'/media/editors/tinymce/plugins/anchor',
			'/media/editors/tinymce/plugins/advlist',
			'/media/editors/tinymce/plugins',
			'/media/editors/tinymce/langs',
			'/media/editors/tinymce/js/plugins/dragdrop',
			'/media/editors/tinymce/js/plugins',
			'/media/editors/tinymce/js',
			'/media/editors/tinymce',
			'/media/editors/none/js',
			'/media/editors/none',
			'/media/editors/codemirror/theme',
			'/media/editors/codemirror/mode/z80',
			'/media/editors/codemirror/mode/yaml-frontmatter',
			'/media/editors/codemirror/mode/yaml',
			'/media/editors/codemirror/mode/yacas',
			'/media/editors/codemirror/mode/xquery',
			'/media/editors/codemirror/mode/xml',
			'/media/editors/codemirror/mode/webidl',
			'/media/editors/codemirror/mode/vue',
			'/media/editors/codemirror/mode/vhdl',
			'/media/editors/codemirror/mode/verilog',
			'/media/editors/codemirror/mode/velocity',
			'/media/editors/codemirror/mode/vbscript',
			'/media/editors/codemirror/mode/vb',
			'/media/editors/codemirror/mode/twig',
			'/media/editors/codemirror/mode/turtle',
			'/media/editors/codemirror/mode/ttcn-cfg',
			'/media/editors/codemirror/mode/ttcn',
			'/media/editors/codemirror/mode/troff',
			'/media/editors/codemirror/mode/tornado',
			'/media/editors/codemirror/mode/toml',
			'/media/editors/codemirror/mode/tiki',
			'/media/editors/codemirror/mode/tiddlywiki',
			'/media/editors/codemirror/mode/textile',
			'/media/editors/codemirror/mode/tcl',
			'/media/editors/codemirror/mode/swift',
			'/media/editors/codemirror/mode/stylus',
			'/media/editors/codemirror/mode/stex',
			'/media/editors/codemirror/mode/sql',
			'/media/editors/codemirror/mode/spreadsheet',
			'/media/editors/codemirror/mode/sparql',
			'/media/editors/codemirror/mode/soy',
			'/media/editors/codemirror/mode/solr',
			'/media/editors/codemirror/mode/smarty',
			'/media/editors/codemirror/mode/smalltalk',
			'/media/editors/codemirror/mode/slim',
			'/media/editors/codemirror/mode/sieve',
			'/media/editors/codemirror/mode/shell',
			'/media/editors/codemirror/mode/scheme',
			'/media/editors/codemirror/mode/sass',
			'/media/editors/codemirror/mode/sas',
			'/media/editors/codemirror/mode/rust',
			'/media/editors/codemirror/mode/ruby',
			'/media/editors/codemirror/mode/rst',
			'/media/editors/codemirror/mode/rpm/changes',
			'/media/editors/codemirror/mode/rpm',
			'/media/editors/codemirror/mode/r',
			'/media/editors/codemirror/mode/q',
			'/media/editors/codemirror/mode/python',
			'/media/editors/codemirror/mode/puppet',
			'/media/editors/codemirror/mode/pug',
			'/media/editors/codemirror/mode/protobuf',
			'/media/editors/codemirror/mode/properties',
			'/media/editors/codemirror/mode/powershell',
			'/media/editors/codemirror/mode/pig',
			'/media/editors/codemirror/mode/php',
			'/media/editors/codemirror/mode/perl',
			'/media/editors/codemirror/mode/pegjs',
			'/media/editors/codemirror/mode/pascal',
			'/media/editors/codemirror/mode/oz',
			'/media/editors/codemirror/mode/octave',
			'/media/editors/codemirror/mode/ntriples',
			'/media/editors/codemirror/mode/nsis',
			'/media/editors/codemirror/mode/nginx',
			'/media/editors/codemirror/mode/mumps',
			'/media/editors/codemirror/mode/mscgen',
			'/media/editors/codemirror/mode/modelica',
			'/media/editors/codemirror/mode/mllike',
			'/media/editors/codemirror/mode/mirc',
			'/media/editors/codemirror/mode/mbox',
			'/media/editors/codemirror/mode/mathematica',
			'/media/editors/codemirror/mode/markdown',
			'/media/editors/codemirror/mode/lua',
			'/media/editors/codemirror/mode/livescript',
			'/media/editors/codemirror/mode/julia',
			'/media/editors/codemirror/mode/jsx',
			'/media/editors/codemirror/mode/jinja2',
			'/media/editors/codemirror/mode/javascript',
			'/media/editors/codemirror/mode/idl',
			'/media/editors/codemirror/mode/http',
			'/media/editors/codemirror/mode/htmlmixed',
			'/media/editors/codemirror/mode/htmlembedded',
			'/media/editors/codemirror/mode/haxe',
			'/media/editors/codemirror/mode/haskell-literate',
			'/media/editors/codemirror/mode/haskell',
			'/media/editors/codemirror/mode/handlebars',
			'/media/editors/codemirror/mode/haml',
			'/media/editors/codemirror/mode/groovy',
			'/media/editors/codemirror/mode/go',
			'/media/editors/codemirror/mode/gherkin',
			'/media/editors/codemirror/mode/gfm',
			'/media/editors/codemirror/mode/gas',
			'/media/editors/codemirror/mode/fortran',
			'/media/editors/codemirror/mode/forth',
			'/media/editors/codemirror/mode/fcl',
			'/media/editors/codemirror/mode/factor',
			'/media/editors/codemirror/mode/erlang',
			'/media/editors/codemirror/mode/elm',
			'/media/editors/codemirror/mode/eiffel',
			'/media/editors/codemirror/mode/ecl',
			'/media/editors/codemirror/mode/ebnf',
			'/media/editors/codemirror/mode/dylan',
			'/media/editors/codemirror/mode/dtd',
			'/media/editors/codemirror/mode/dockerfile',
			'/media/editors/codemirror/mode/django',
			'/media/editors/codemirror/mode/diff',
			'/media/editors/codemirror/mode/dart',
			'/media/editors/codemirror/mode/d',
			'/media/editors/codemirror/mode/cypher',
			'/media/editors/codemirror/mode/css',
			'/media/editors/codemirror/mode/crystal',
			'/media/editors/codemirror/mode/commonlisp',
			'/media/editors/codemirror/mode/coffeescript',
			'/media/editors/codemirror/mode/cobol',
			'/media/editors/codemirror/mode/cmake',
			'/media/editors/codemirror/mode/clojure',
			'/media/editors/codemirror/mode/clike',
			'/media/editors/codemirror/mode/brainfuck',
			'/media/editors/codemirror/mode/asterisk',
			'/media/editors/codemirror/mode/asn.1',
			'/media/editors/codemirror/mode/asciiarmor',
			'/media/editors/codemirror/mode/apl',
			'/media/editors/codemirror/mode',
			'/media/editors/codemirror/lib',
			'/media/editors/codemirror/keymap',
			'/media/editors/codemirror/addon/wrap',
			'/media/editors/codemirror/addon/tern',
			'/media/editors/codemirror/addon/selection',
			'/media/editors/codemirror/addon/search',
			'/media/editors/codemirror/addon/scroll',
			'/media/editors/codemirror/addon/runmode',
			'/media/editors/codemirror/addon/mode',
			'/media/editors/codemirror/addon/merge',
			'/media/editors/codemirror/addon/lint',
			'/media/editors/codemirror/addon/hint',
			'/media/editors/codemirror/addon/fold',
			'/media/editors/codemirror/addon/edit',
			'/media/editors/codemirror/addon/display',
			'/media/editors/codemirror/addon/dialog',
			'/media/editors/codemirror/addon/comment',
			'/media/editors/codemirror/addon',
			'/media/editors/codemirror',
			'/media/editors',
			'/media/com_contenthistory/css',
			'/media/cms/css',
			'/media/cms',
			'/libraries/vendor/symfony/polyfill-util',
			'/libraries/vendor/symfony/polyfill-php71',
			'/libraries/vendor/symfony/polyfill-php56',
			'/libraries/vendor/symfony/polyfill-php55',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/XML/Declaration',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/XML',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Parse',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Net',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/HTTP',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Decode/HTML',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Decode',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Content/Type',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Content',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie/Cache',
			'/libraries/vendor/simplepie/simplepie/library/SimplePie',
			'/libraries/vendor/simplepie/simplepie/library',
			'/libraries/vendor/simplepie/simplepie/idn',
			'/libraries/vendor/simplepie/simplepie',
			'/libraries/vendor/simplepie',
			'/libraries/vendor/phpmailer/phpmailer/extras',
			'/libraries/vendor/leafo/lessphp',
			'/libraries/vendor/leafo',
			'/libraries/vendor/joomla/session/Joomla/Session/Storage',
			'/libraries/vendor/joomla/session/Joomla/Session',
			'/libraries/vendor/joomla/session/Joomla',
			'/libraries/vendor/joomla/image/src/Filter',
			'/libraries/vendor/joomla/image/src',
			'/libraries/vendor/joomla/image',
			'/libraries/vendor/joomla/compat/src',
			'/libraries/vendor/joomla/compat',
			'/libraries/vendor/joomla/application/src/Cli/Output/Processor',
			'/libraries/vendor/joomla/application/src/Cli/Output',
			'/libraries/vendor/joomla/application/src/Cli',
			'/libraries/vendor/ircmaxell/password-compat/lib',
			'/libraries/vendor/ircmaxell/password-compat',
			'/libraries/vendor/ircmaxell',
			'/libraries/vendor/brumann/polyfill-unserialize/src',
			'/libraries/vendor/brumann/polyfill-unserialize',
			'/libraries/vendor/brumann',
			'/libraries/src/Table/Observer',
			'/libraries/src/Menu/Node',
			'/libraries/src/Language/Wrapper',
			'/libraries/src/Language/Stemmer',
			'/libraries/src/Http/Wrapper',
			'/libraries/src/Filter/Wrapper',
			'/libraries/src/Filesystem/Wrapper',
			'/libraries/src/Crypt/Password',
			'/libraries/src/Access/Wrapper',
			'/libraries/phputf8/utils',
			'/libraries/phputf8/native',
			'/libraries/phputf8/mbstring',
			'/libraries/phputf8',
			'/libraries/legacy/utilities',
			'/libraries/legacy/table',
			'/libraries/legacy/simplepie',
			'/libraries/legacy/simplecrypt',
			'/libraries/legacy/response',
			'/libraries/legacy/request',
			'/libraries/legacy/log',
			'/libraries/legacy/form/field',
			'/libraries/legacy/form',
			'/libraries/legacy/exception',
			'/libraries/legacy/error',
			'/libraries/legacy/dispatcher',
			'/libraries/legacy/database',
			'/libraries/legacy/base',
			'/libraries/legacy/application',
			'/libraries/legacy',
			'/libraries/joomla/view',
			'/libraries/joomla/utilities',
			'/libraries/joomla/twitter',
			'/libraries/joomla/string/wrapper',
			'/libraries/joomla/string',
			'/libraries/joomla/session/storage',
			'/libraries/joomla/session/handler',
			'/libraries/joomla/session',
			'/libraries/joomla/route/wrapper',
			'/libraries/joomla/route',
			'/libraries/joomla/openstreetmap',
			'/libraries/joomla/observer/wrapper',
			'/libraries/joomla/observer/updater',
			'/libraries/joomla/observer',
			'/libraries/joomla/observable',
			'/libraries/joomla/oauth2',
			'/libraries/joomla/oauth1',
			'/libraries/joomla/model',
			'/libraries/joomla/mediawiki',
			'/libraries/joomla/linkedin',
			'/libraries/joomla/keychain',
			'/libraries/joomla/grid',
			'/libraries/joomla/google/embed',
			'/libraries/joomla/google/data/plus',
			'/libraries/joomla/google/data/picasa',
			'/libraries/joomla/google/data',
			'/libraries/joomla/google/auth',
			'/libraries/joomla/google',
			'/libraries/joomla/github/package/users',
			'/libraries/joomla/github/package/repositories',
			'/libraries/joomla/github/package/pulls',
			'/libraries/joomla/github/package/orgs',
			'/libraries/joomla/github/package/issues',
			'/libraries/joomla/github/package/gists',
			'/libraries/joomla/github/package/data',
			'/libraries/joomla/github/package/activity',
			'/libraries/joomla/github/package',
			'/libraries/joomla/github',
			'/libraries/joomla/form/fields',
			'/libraries/joomla/form',
			'/libraries/joomla/facebook',
			'/libraries/joomla/event',
			'/libraries/joomla/database/query',
			'/libraries/joomla/database/iterator',
			'/libraries/joomla/database/importer',
			'/libraries/joomla/database/exporter',
			'/libraries/joomla/database/exception',
			'/libraries/joomla/database/driver',
			'/libraries/joomla/database',
			'/libraries/joomla/controller',
			'/libraries/joomla/archive/wrapper',
			'/libraries/joomla/archive',
			'/libraries/joomla/application/web/router',
			'/libraries/joomla/application/web',
			'/libraries/joomla/application',
			'/libraries/joomla',
			'/libraries/idna_convert',
			'/libraries/fof/view',
			'/libraries/fof/utils/update',
			'/libraries/fof/utils/timer',
			'/libraries/fof/utils/phpfunc',
			'/libraries/fof/utils/observable',
			'/libraries/fof/utils/object',
			'/libraries/fof/utils/ip',
			'/libraries/fof/utils/installscript',
			'/libraries/fof/utils/ini',
			'/libraries/fof/utils/filescheck',
			'/libraries/fof/utils/config',
			'/libraries/fof/utils/cache',
			'/libraries/fof/utils/array',
			'/libraries/fof/utils',
			'/libraries/fof/toolbar',
			'/libraries/fof/template',
			'/libraries/fof/table/dispatcher',
			'/libraries/fof/table/behavior',
			'/libraries/fof/table',
			'/libraries/fof/string',
			'/libraries/fof/render',
			'/libraries/fof/query',
			'/libraries/fof/platform/filesystem',
			'/libraries/fof/platform',
			'/libraries/fof/model/field',
			'/libraries/fof/model/dispatcher',
			'/libraries/fof/model/behavior',
			'/libraries/fof/model',
			'/libraries/fof/less/parser',
			'/libraries/fof/less/formatter',
			'/libraries/fof/less',
			'/libraries/fof/layout',
			'/libraries/fof/integration/joomla/filesystem',
			'/libraries/fof/integration/joomla',
			'/libraries/fof/integration',
			'/libraries/fof/input/jinput',
			'/libraries/fof/input',
			'/libraries/fof/inflector',
			'/libraries/fof/hal/render',
			'/libraries/fof/hal',
			'/libraries/fof/form/header',
			'/libraries/fof/form/field',
			'/libraries/fof/form',
			'/libraries/fof/encrypt/aes',
			'/libraries/fof/encrypt',
			'/libraries/fof/download/adapter',
			'/libraries/fof/download',
			'/libraries/fof/dispatcher',
			'/libraries/fof/database/query',
			'/libraries/fof/database/iterator',
			'/libraries/fof/database/driver',
			'/libraries/fof/database',
			'/libraries/fof/controller',
			'/libraries/fof/config/domain',
			'/libraries/fof/config',
			'/libraries/fof/autoloader',
			'/libraries/fof',
			'/libraries/cms/less/formatter',
			'/libraries/cms/less',
			'/libraries/cms/html/language/en-GB',
			'/libraries/cms/html/language',
			'/libraries/cms/html',
			'/libraries/cms/class',
			'/libraries/cms',
			'/layouts/joomla/tinymce/buttons',
			'/layouts/joomla/html/formbehavior',
			'/components/com_wrapper/views/wrapper/tmpl',
			'/components/com_wrapper/views/wrapper',
			'/components/com_wrapper/views',
			'/components/com_users/views/reset/tmpl',
			'/components/com_users/views/reset',
			'/components/com_users/views/remind/tmpl',
			'/components/com_users/views/remind',
			'/components/com_users/views/registration/tmpl',
			'/components/com_users/views/registration',
			'/components/com_users/views/profile/tmpl',
			'/components/com_users/views/profile',
			'/components/com_users/views/login/tmpl',
			'/components/com_users/views/login',
			'/components/com_users/views',
			'/components/com_users/models/rules',
			'/components/com_users/models/forms',
			'/components/com_users/models',
			'/components/com_users/helpers/html',
			'/components/com_users/helpers',
			'/components/com_users/controllers',
			'/components/com_tags/views/tags/tmpl',
			'/components/com_tags/views/tags',
			'/components/com_tags/views/tag/tmpl',
			'/components/com_tags/views/tag',
			'/components/com_tags/views',
			'/components/com_tags/models',
			'/components/com_tags/controllers',
			'/components/com_privacy/views/request/tmpl',
			'/components/com_privacy/views/request',
			'/components/com_privacy/views/remind/tmpl',
			'/components/com_privacy/views/remind',
			'/components/com_privacy/views/confirm/tmpl',
			'/components/com_privacy/views/confirm',
			'/components/com_privacy/views',
			'/components/com_privacy/models/forms',
			'/components/com_privacy/models',
			'/components/com_privacy/controllers',
			'/components/com_newsfeeds/views/newsfeed/tmpl',
			'/components/com_newsfeeds/views/newsfeed',
			'/components/com_newsfeeds/views/category/tmpl',
			'/components/com_newsfeeds/views/category',
			'/components/com_newsfeeds/views/categories/tmpl',
			'/components/com_newsfeeds/views/categories',
			'/components/com_newsfeeds/views',
			'/components/com_newsfeeds/models',
			'/components/com_modules/models/forms',
			'/components/com_modules/models',
			'/components/com_menus/models/forms',
			'/components/com_menus/models',
			'/components/com_mailto/views/sent/tmpl',
			'/components/com_mailto/views/sent',
			'/components/com_mailto/views/mailto/tmpl',
			'/components/com_mailto/views/mailto',
			'/components/com_mailto/views',
			'/components/com_mailto/models/forms',
			'/components/com_mailto/models',
			'/components/com_mailto/helpers',
			'/components/com_mailto',
			'/components/com_finder/views/search/tmpl',
			'/components/com_finder/views/search',
			'/components/com_finder/views',
			'/components/com_finder/models',
			'/components/com_finder/helpers/html',
			'/components/com_finder/controllers',
			'/components/com_fields/models/forms',
			'/components/com_fields/models',
			'/components/com_content/views/form/tmpl',
			'/components/com_content/views/form',
			'/components/com_content/views/featured/tmpl',
			'/components/com_content/views/featured',
			'/components/com_content/views/category/tmpl',
			'/components/com_content/views/category',
			'/components/com_content/views/categories/tmpl',
			'/components/com_content/views/categories',
			'/components/com_content/views/article/tmpl',
			'/components/com_content/views/article',
			'/components/com_content/views/archive/tmpl',
			'/components/com_content/views/archive',
			'/components/com_content/views',
			'/components/com_content/models/forms',
			'/components/com_content/models',
			'/components/com_content/controllers',
			'/components/com_contact/views/featured/tmpl',
			'/components/com_contact/views/featured',
			'/components/com_contact/views/contact/tmpl',
			'/components/com_contact/views/contact',
			'/components/com_contact/views/category/tmpl',
			'/components/com_contact/views/category',
			'/components/com_contact/views/categories/tmpl',
			'/components/com_contact/views/categories',
			'/components/com_contact/views',
			'/components/com_contact/models/rules',
			'/components/com_contact/models/forms',
			'/components/com_contact/models',
			'/components/com_contact/controllers',
			'/components/com_config/view/templates/tmpl',
			'/components/com_config/view/templates',
			'/components/com_config/view/modules/tmpl',
			'/components/com_config/view/modules',
			'/components/com_config/view/config/tmpl',
			'/components/com_config/view/config',
			'/components/com_config/view/cms',
			'/components/com_config/view',
			'/components/com_config/model/form',
			'/components/com_config/model',
			'/components/com_config/controller/templates',
			'/components/com_config/controller/modules',
			'/components/com_config/controller/config',
			'/components/com_config/controller',
			'/components/com_banners/models',
			'/components/com_banners/helpers',
			'/bin',
			'/administrator/templates/system/html',
			'/administrator/templates/isis/less/pages',
			'/administrator/templates/isis/less/bootstrap',
			'/administrator/templates/isis/less/blocks',
			'/administrator/templates/isis/less',
			'/administrator/templates/isis/language/en-GB',
			'/administrator/templates/isis/language',
			'/administrator/templates/isis/js',
			'/administrator/templates/isis/img',
			'/administrator/templates/isis/images/system',
			'/administrator/templates/isis/images/admin',
			'/administrator/templates/isis/images',
			'/administrator/templates/isis/html/mod_version',
			'/administrator/templates/isis/html/layouts/joomla/toolbar',
			'/administrator/templates/isis/html/layouts/joomla/system',
			'/administrator/templates/isis/html/layouts/joomla/pagination',
			'/administrator/templates/isis/html/layouts/joomla/form/field',
			'/administrator/templates/isis/html/layouts/joomla/form',
			'/administrator/templates/isis/html/layouts/joomla',
			'/administrator/templates/isis/html/layouts',
			'/administrator/templates/isis/html/com_media/medialist',
			'/administrator/templates/isis/html/com_media/imageslist',
			'/administrator/templates/isis/html/com_media',
			'/administrator/templates/isis/html',
			'/administrator/templates/isis/css',
			'/administrator/templates/isis',
			'/administrator/templates/hathor/postinstall',
			'/administrator/templates/hathor/less',
			'/administrator/templates/hathor/language/en-GB',
			'/administrator/templates/hathor/language',
			'/administrator/templates/hathor/js',
			'/administrator/templates/hathor/images/toolbar',
			'/administrator/templates/hathor/images/system',
			'/administrator/templates/hathor/images/menu',
			'/administrator/templates/hathor/images/header',
			'/administrator/templates/hathor/images/admin',
			'/administrator/templates/hathor/images',
			'/administrator/templates/hathor/html/mod_quickicon',
			'/administrator/templates/hathor/html/mod_login',
			'/administrator/templates/hathor/html/layouts/plugins/user/profile/fields',
			'/administrator/templates/hathor/html/layouts/plugins/user/profile',
			'/administrator/templates/hathor/html/layouts/plugins/user',
			'/administrator/templates/hathor/html/layouts/plugins',
			'/administrator/templates/hathor/html/layouts/joomla/toolbar',
			'/administrator/templates/hathor/html/layouts/joomla/sidebars',
			'/administrator/templates/hathor/html/layouts/joomla/quickicons',
			'/administrator/templates/hathor/html/layouts/joomla/edit',
			'/administrator/templates/hathor/html/layouts/joomla',
			'/administrator/templates/hathor/html/layouts/com_modules/toolbar',
			'/administrator/templates/hathor/html/layouts/com_modules',
			'/administrator/templates/hathor/html/layouts/com_messages/toolbar',
			'/administrator/templates/hathor/html/layouts/com_messages',
			'/administrator/templates/hathor/html/layouts/com_media/toolbar',
			'/administrator/templates/hathor/html/layouts/com_media',
			'/administrator/templates/hathor/html/layouts',
			'/administrator/templates/hathor/html/com_weblinks/weblinks',
			'/administrator/templates/hathor/html/com_weblinks/weblink',
			'/administrator/templates/hathor/html/com_weblinks',
			'/administrator/templates/hathor/html/com_users/users',
			'/administrator/templates/hathor/html/com_users/user',
			'/administrator/templates/hathor/html/com_users/notes',
			'/administrator/templates/hathor/html/com_users/note',
			'/administrator/templates/hathor/html/com_users/levels',
			'/administrator/templates/hathor/html/com_users/groups',
			'/administrator/templates/hathor/html/com_users/debuguser',
			'/administrator/templates/hathor/html/com_users/debuggroup',
			'/administrator/templates/hathor/html/com_users',
			'/administrator/templates/hathor/html/com_templates/templates',
			'/administrator/templates/hathor/html/com_templates/template',
			'/administrator/templates/hathor/html/com_templates/styles',
			'/administrator/templates/hathor/html/com_templates/style',
			'/administrator/templates/hathor/html/com_templates',
			'/administrator/templates/hathor/html/com_tags/tags',
			'/administrator/templates/hathor/html/com_tags/tag',
			'/administrator/templates/hathor/html/com_tags',
			'/administrator/templates/hathor/html/com_search/searches',
			'/administrator/templates/hathor/html/com_search',
			'/administrator/templates/hathor/html/com_redirect/links',
			'/administrator/templates/hathor/html/com_redirect',
			'/administrator/templates/hathor/html/com_postinstall/messages',
			'/administrator/templates/hathor/html/com_postinstall',
			'/administrator/templates/hathor/html/com_plugins/plugins',
			'/administrator/templates/hathor/html/com_plugins/plugin',
			'/administrator/templates/hathor/html/com_plugins',
			'/administrator/templates/hathor/html/com_newsfeeds/newsfeeds',
			'/administrator/templates/hathor/html/com_newsfeeds/newsfeed',
			'/administrator/templates/hathor/html/com_newsfeeds',
			'/administrator/templates/hathor/html/com_modules/positions',
			'/administrator/templates/hathor/html/com_modules/modules',
			'/administrator/templates/hathor/html/com_modules/module',
			'/administrator/templates/hathor/html/com_modules',
			'/administrator/templates/hathor/html/com_messages/messages',
			'/administrator/templates/hathor/html/com_messages/message',
			'/administrator/templates/hathor/html/com_messages',
			'/administrator/templates/hathor/html/com_menus/menutypes',
			'/administrator/templates/hathor/html/com_menus/menus',
			'/administrator/templates/hathor/html/com_menus/menu',
			'/administrator/templates/hathor/html/com_menus/items',
			'/administrator/templates/hathor/html/com_menus/item',
			'/administrator/templates/hathor/html/com_menus',
			'/administrator/templates/hathor/html/com_languages/overrides',
			'/administrator/templates/hathor/html/com_languages/languages',
			'/administrator/templates/hathor/html/com_languages/installed',
			'/administrator/templates/hathor/html/com_languages',
			'/administrator/templates/hathor/html/com_joomlaupdate/default',
			'/administrator/templates/hathor/html/com_joomlaupdate',
			'/administrator/templates/hathor/html/com_installer/warnings',
			'/administrator/templates/hathor/html/com_installer/update',
			'/administrator/templates/hathor/html/com_installer/manage',
			'/administrator/templates/hathor/html/com_installer/languages',
			'/administrator/templates/hathor/html/com_installer/install',
			'/administrator/templates/hathor/html/com_installer/discover',
			'/administrator/templates/hathor/html/com_installer/default',
			'/administrator/templates/hathor/html/com_installer/database',
			'/administrator/templates/hathor/html/com_installer',
			'/administrator/templates/hathor/html/com_finder/maps',
			'/administrator/templates/hathor/html/com_finder/index',
			'/administrator/templates/hathor/html/com_finder/filters',
			'/administrator/templates/hathor/html/com_finder',
			'/administrator/templates/hathor/html/com_fields/groups',
			'/administrator/templates/hathor/html/com_fields/group',
			'/administrator/templates/hathor/html/com_fields/fields',
			'/administrator/templates/hathor/html/com_fields/field',
			'/administrator/templates/hathor/html/com_fields',
			'/administrator/templates/hathor/html/com_cpanel/cpanel',
			'/administrator/templates/hathor/html/com_cpanel',
			'/administrator/templates/hathor/html/com_contenthistory/history',
			'/administrator/templates/hathor/html/com_contenthistory',
			'/administrator/templates/hathor/html/com_content/featured',
			'/administrator/templates/hathor/html/com_content/articles',
			'/administrator/templates/hathor/html/com_content/article',
			'/administrator/templates/hathor/html/com_content',
			'/administrator/templates/hathor/html/com_contact/contacts',
			'/administrator/templates/hathor/html/com_contact/contact',
			'/administrator/templates/hathor/html/com_contact',
			'/administrator/templates/hathor/html/com_config/component',
			'/administrator/templates/hathor/html/com_config/application',
			'/administrator/templates/hathor/html/com_config',
			'/administrator/templates/hathor/html/com_checkin/checkin',
			'/administrator/templates/hathor/html/com_checkin',
			'/administrator/templates/hathor/html/com_categories/category',
			'/administrator/templates/hathor/html/com_categories/categories',
			'/administrator/templates/hathor/html/com_categories',
			'/administrator/templates/hathor/html/com_cache/purge',
			'/administrator/templates/hathor/html/com_cache/cache',
			'/administrator/templates/hathor/html/com_cache',
			'/administrator/templates/hathor/html/com_banners/tracks',
			'/administrator/templates/hathor/html/com_banners/download',
			'/administrator/templates/hathor/html/com_banners/clients',
			'/administrator/templates/hathor/html/com_banners/client',
			'/administrator/templates/hathor/html/com_banners/banners',
			'/administrator/templates/hathor/html/com_banners/banner',
			'/administrator/templates/hathor/html/com_banners',
			'/administrator/templates/hathor/html/com_associations/associations',
			'/administrator/templates/hathor/html/com_associations',
			'/administrator/templates/hathor/html/com_admin/sysinfo',
			'/administrator/templates/hathor/html/com_admin/profile',
			'/administrator/templates/hathor/html/com_admin/help',
			'/administrator/templates/hathor/html/com_admin',
			'/administrator/templates/hathor/html',
			'/administrator/templates/hathor/css',
			'/administrator/templates/hathor',
			'/administrator/modules/mod_version/language/en-GB',
			'/administrator/modules/mod_version/language',
			'/administrator/modules/mod_status/tmpl',
			'/administrator/modules/mod_status',
			'/administrator/modules/mod_stats_admin/language',
			'/administrator/modules/mod_multilangstatus/language/en-GB',
			'/administrator/modules/mod_multilangstatus/language',
			'/administrator/components/com_users/views/users/tmpl',
			'/administrator/components/com_users/views/users',
			'/administrator/components/com_users/views/user/tmpl',
			'/administrator/components/com_users/views/user',
			'/administrator/components/com_users/views/notes/tmpl',
			'/administrator/components/com_users/views/notes',
			'/administrator/components/com_users/views/note/tmpl',
			'/administrator/components/com_users/views/note',
			'/administrator/components/com_users/views/mail/tmpl',
			'/administrator/components/com_users/views/mail',
			'/administrator/components/com_users/views/levels/tmpl',
			'/administrator/components/com_users/views/levels',
			'/administrator/components/com_users/views/level/tmpl',
			'/administrator/components/com_users/views/level',
			'/administrator/components/com_users/views/groups/tmpl',
			'/administrator/components/com_users/views/groups',
			'/administrator/components/com_users/views/group/tmpl',
			'/administrator/components/com_users/views/group',
			'/administrator/components/com_users/views/debuguser/tmpl',
			'/administrator/components/com_users/views/debuguser',
			'/administrator/components/com_users/views/debuggroup/tmpl',
			'/administrator/components/com_users/views/debuggroup',
			'/administrator/components/com_users/views',
			'/administrator/components/com_users/tables',
			'/administrator/components/com_users/models/forms/fields',
			'/administrator/components/com_users/models/forms',
			'/administrator/components/com_users/models/fields',
			'/administrator/components/com_users/models',
			'/administrator/components/com_users/helpers/html',
			'/administrator/components/com_users/controllers',
			'/administrator/components/com_templates/views/templates/tmpl',
			'/administrator/components/com_templates/views/templates',
			'/administrator/components/com_templates/views/template/tmpl',
			'/administrator/components/com_templates/views/template',
			'/administrator/components/com_templates/views/styles/tmpl',
			'/administrator/components/com_templates/views/styles',
			'/administrator/components/com_templates/views/style/tmpl',
			'/administrator/components/com_templates/views/style',
			'/administrator/components/com_templates/views',
			'/administrator/components/com_templates/tables',
			'/administrator/components/com_templates/models/forms',
			'/administrator/components/com_templates/models/fields',
			'/administrator/components/com_templates/models',
			'/administrator/components/com_templates/helpers/html',
			'/administrator/components/com_templates/controllers',
			'/administrator/components/com_tags/views/tags/tmpl',
			'/administrator/components/com_tags/views/tags',
			'/administrator/components/com_tags/views/tag/tmpl',
			'/administrator/components/com_tags/views/tag',
			'/administrator/components/com_tags/views',
			'/administrator/components/com_tags/tables',
			'/administrator/components/com_tags/models/forms',
			'/administrator/components/com_tags/models',
			'/administrator/components/com_tags/helpers',
			'/administrator/components/com_tags/controllers',
			'/administrator/components/com_redirect/views/links/tmpl',
			'/administrator/components/com_redirect/views/links',
			'/administrator/components/com_redirect/views/link/tmpl',
			'/administrator/components/com_redirect/views/link',
			'/administrator/components/com_redirect/views',
			'/administrator/components/com_redirect/tables',
			'/administrator/components/com_redirect/models/forms',
			'/administrator/components/com_redirect/models/fields',
			'/administrator/components/com_redirect/models',
			'/administrator/components/com_redirect/helpers/html',
			'/administrator/components/com_redirect/controllers',
			'/administrator/components/com_privacy/views/requests/tmpl',
			'/administrator/components/com_privacy/views/requests',
			'/administrator/components/com_privacy/views/request/tmpl',
			'/administrator/components/com_privacy/views/request',
			'/administrator/components/com_privacy/views/export',
			'/administrator/components/com_privacy/views/dashboard/tmpl',
			'/administrator/components/com_privacy/views/dashboard',
			'/administrator/components/com_privacy/views/consents/tmpl',
			'/administrator/components/com_privacy/views/consents',
			'/administrator/components/com_privacy/views/capabilities/tmpl',
			'/administrator/components/com_privacy/views/capabilities',
			'/administrator/components/com_privacy/views',
			'/administrator/components/com_privacy/tables',
			'/administrator/components/com_privacy/models/forms',
			'/administrator/components/com_privacy/models/fields',
			'/administrator/components/com_privacy/models',
			'/administrator/components/com_privacy/helpers/removal',
			'/administrator/components/com_privacy/helpers/html',
			'/administrator/components/com_privacy/helpers/export',
			'/administrator/components/com_privacy/helpers',
			'/administrator/components/com_privacy/controllers',
			'/administrator/components/com_postinstall/views/messages/tmpl',
			'/administrator/components/com_postinstall/views/messages',
			'/administrator/components/com_postinstall/views',
			'/administrator/components/com_postinstall/models',
			'/administrator/components/com_postinstall/controllers',
			'/administrator/components/com_plugins/views/plugins/tmpl',
			'/administrator/components/com_plugins/views/plugins',
			'/administrator/components/com_plugins/views/plugin/tmpl',
			'/administrator/components/com_plugins/views/plugin',
			'/administrator/components/com_plugins/views',
			'/administrator/components/com_plugins/models/forms',
			'/administrator/components/com_plugins/models/fields',
			'/administrator/components/com_plugins/models',
			'/administrator/components/com_plugins/controllers',
			'/administrator/components/com_newsfeeds/views/newsfeeds/tmpl',
			'/administrator/components/com_newsfeeds/views/newsfeeds',
			'/administrator/components/com_newsfeeds/views/newsfeed/tmpl',
			'/administrator/components/com_newsfeeds/views/newsfeed',
			'/administrator/components/com_newsfeeds/views',
			'/administrator/components/com_newsfeeds/tables',
			'/administrator/components/com_newsfeeds/models/forms',
			'/administrator/components/com_newsfeeds/models/fields/modal',
			'/administrator/components/com_newsfeeds/models/fields',
			'/administrator/components/com_newsfeeds/models',
			'/administrator/components/com_newsfeeds/helpers/html',
			'/administrator/components/com_newsfeeds/controllers',
			'/administrator/components/com_modules/views/select/tmpl',
			'/administrator/components/com_modules/views/select',
			'/administrator/components/com_modules/views/preview/tmpl',
			'/administrator/components/com_modules/views/preview',
			'/administrator/components/com_modules/views/positions/tmpl',
			'/administrator/components/com_modules/views/positions',
			'/administrator/components/com_modules/views/modules/tmpl',
			'/administrator/components/com_modules/views/modules',
			'/administrator/components/com_modules/views/module/tmpl',
			'/administrator/components/com_modules/views/module',
			'/administrator/components/com_modules/views',
			'/administrator/components/com_modules/models/forms',
			'/administrator/components/com_modules/models/fields',
			'/administrator/components/com_modules/models',
			'/administrator/components/com_modules/helpers/html',
			'/administrator/components/com_modules/controllers',
			'/administrator/components/com_messages/views/messages/tmpl',
			'/administrator/components/com_messages/views/messages',
			'/administrator/components/com_messages/views/message/tmpl',
			'/administrator/components/com_messages/views/message',
			'/administrator/components/com_messages/views/config/tmpl',
			'/administrator/components/com_messages/views/config',
			'/administrator/components/com_messages/views',
			'/administrator/components/com_messages/tables',
			'/administrator/components/com_messages/models/forms',
			'/administrator/components/com_messages/models/fields',
			'/administrator/components/com_messages/models',
			'/administrator/components/com_messages/helpers/html',
			'/administrator/components/com_messages/helpers',
			'/administrator/components/com_messages/controllers',
			'/administrator/components/com_menus/views/menutypes/tmpl',
			'/administrator/components/com_menus/views/menutypes',
			'/administrator/components/com_menus/views/menus/tmpl',
			'/administrator/components/com_menus/views/menus',
			'/administrator/components/com_menus/views/menu/tmpl',
			'/administrator/components/com_menus/views/menu',
			'/administrator/components/com_menus/views/items/tmpl',
			'/administrator/components/com_menus/views/items',
			'/administrator/components/com_menus/views/item/tmpl',
			'/administrator/components/com_menus/views/item',
			'/administrator/components/com_menus/views',
			'/administrator/components/com_menus/tables',
			'/administrator/components/com_menus/models/forms',
			'/administrator/components/com_menus/models/fields/modal',
			'/administrator/components/com_menus/models/fields',
			'/administrator/components/com_menus/models',
			'/administrator/components/com_menus/layouts/joomla/searchtools/default',
			'/administrator/components/com_menus/helpers/html',
			'/administrator/components/com_menus/controllers',
			'/administrator/components/com_media/views/medialist/tmpl',
			'/administrator/components/com_media/views/medialist',
			'/administrator/components/com_media/views/media/tmpl',
			'/administrator/components/com_media/views/media',
			'/administrator/components/com_media/views/imageslist/tmpl',
			'/administrator/components/com_media/views/imageslist',
			'/administrator/components/com_media/views/images/tmpl',
			'/administrator/components/com_media/views/images',
			'/administrator/components/com_media/views',
			'/administrator/components/com_media/models',
			'/administrator/components/com_media/helpers',
			'/administrator/components/com_media/controllers',
			'/administrator/components/com_login/views/login/tmpl',
			'/administrator/components/com_login/views/login',
			'/administrator/components/com_login/views',
			'/administrator/components/com_login/models',
			'/administrator/components/com_languages/views/overrides/tmpl',
			'/administrator/components/com_languages/views/overrides',
			'/administrator/components/com_languages/views/override/tmpl',
			'/administrator/components/com_languages/views/override',
			'/administrator/components/com_languages/views/multilangstatus/tmpl',
			'/administrator/components/com_languages/views/multilangstatus',
			'/administrator/components/com_languages/views/languages/tmpl',
			'/administrator/components/com_languages/views/languages',
			'/administrator/components/com_languages/views/language/tmpl',
			'/administrator/components/com_languages/views/language',
			'/administrator/components/com_languages/views/installed/tmpl',
			'/administrator/components/com_languages/views/installed',
			'/administrator/components/com_languages/views',
			'/administrator/components/com_languages/models/forms',
			'/administrator/components/com_languages/models/fields',
			'/administrator/components/com_languages/models',
			'/administrator/components/com_languages/layouts/joomla/searchtools/default',
			'/administrator/components/com_languages/layouts/joomla/searchtools',
			'/administrator/components/com_languages/layouts/joomla',
			'/administrator/components/com_languages/layouts',
			'/administrator/components/com_languages/helpers/html',
			'/administrator/components/com_languages/helpers',
			'/administrator/components/com_languages/controllers',
			'/administrator/components/com_joomlaupdate/views/upload/tmpl',
			'/administrator/components/com_joomlaupdate/views/upload',
			'/administrator/components/com_joomlaupdate/views/update/tmpl',
			'/administrator/components/com_joomlaupdate/views/update',
			'/administrator/components/com_joomlaupdate/views/default/tmpl',
			'/administrator/components/com_joomlaupdate/views/default',
			'/administrator/components/com_joomlaupdate/views',
			'/administrator/components/com_joomlaupdate/models',
			'/administrator/components/com_joomlaupdate/helpers',
			'/administrator/components/com_joomlaupdate/controllers',
			'/administrator/components/com_installer/views/warnings/tmpl',
			'/administrator/components/com_installer/views/warnings',
			'/administrator/components/com_installer/views/updatesites/tmpl',
			'/administrator/components/com_installer/views/updatesites',
			'/administrator/components/com_installer/views/update/tmpl',
			'/administrator/components/com_installer/views/update',
			'/administrator/components/com_installer/views/manage/tmpl',
			'/administrator/components/com_installer/views/manage',
			'/administrator/components/com_installer/views/languages/tmpl',
			'/administrator/components/com_installer/views/languages',
			'/administrator/components/com_installer/views/install/tmpl',
			'/administrator/components/com_installer/views/install',
			'/administrator/components/com_installer/views/discover/tmpl',
			'/administrator/components/com_installer/views/discover',
			'/administrator/components/com_installer/views/default/tmpl',
			'/administrator/components/com_installer/views/default',
			'/administrator/components/com_installer/views/database/tmpl',
			'/administrator/components/com_installer/views/database',
			'/administrator/components/com_installer/views',
			'/administrator/components/com_installer/models/forms',
			'/administrator/components/com_installer/models/fields',
			'/administrator/components/com_installer/models',
			'/administrator/components/com_installer/helpers/html',
			'/administrator/components/com_installer/controllers',
			'/administrator/components/com_finder/views/statistics/tmpl',
			'/administrator/components/com_finder/views/statistics',
			'/administrator/components/com_finder/views/maps/tmpl',
			'/administrator/components/com_finder/views/maps',
			'/administrator/components/com_finder/views/indexer/tmpl',
			'/administrator/components/com_finder/views/indexer',
			'/administrator/components/com_finder/views/index/tmpl',
			'/administrator/components/com_finder/views/index',
			'/administrator/components/com_finder/views/filters/tmpl',
			'/administrator/components/com_finder/views/filters',
			'/administrator/components/com_finder/views/filter/tmpl',
			'/administrator/components/com_finder/views/filter',
			'/administrator/components/com_finder/views',
			'/administrator/components/com_finder/tables',
			'/administrator/components/com_finder/models/forms',
			'/administrator/components/com_finder/models/fields',
			'/administrator/components/com_finder/models',
			'/administrator/components/com_finder/helpers/indexer/stemmer',
			'/administrator/components/com_finder/helpers/indexer/parser',
			'/administrator/components/com_finder/helpers/indexer/driver',
			'/administrator/components/com_finder/helpers/html',
			'/administrator/components/com_finder/controllers',
			'/administrator/components/com_fields/views/groups/tmpl',
			'/administrator/components/com_fields/views/groups',
			'/administrator/components/com_fields/views/group/tmpl',
			'/administrator/components/com_fields/views/group',
			'/administrator/components/com_fields/views/fields/tmpl',
			'/administrator/components/com_fields/views/fields',
			'/administrator/components/com_fields/views/field/tmpl',
			'/administrator/components/com_fields/views/field',
			'/administrator/components/com_fields/views',
			'/administrator/components/com_fields/tables',
			'/administrator/components/com_fields/models/forms',
			'/administrator/components/com_fields/models/fields',
			'/administrator/components/com_fields/models',
			'/administrator/components/com_fields/libraries',
			'/administrator/components/com_fields/controllers',
			'/administrator/components/com_cpanel/views/cpanel/tmpl',
			'/administrator/components/com_cpanel/views/cpanel',
			'/administrator/components/com_cpanel/views',
			'/administrator/components/com_contenthistory/views/preview/tmpl',
			'/administrator/components/com_contenthistory/views/preview',
			'/administrator/components/com_contenthistory/views/history/tmpl',
			'/administrator/components/com_contenthistory/views/history',
			'/administrator/components/com_contenthistory/views/compare/tmpl',
			'/administrator/components/com_contenthistory/views/compare',
			'/administrator/components/com_contenthistory/views',
			'/administrator/components/com_contenthistory/models',
			'/administrator/components/com_contenthistory/helpers/html',
			'/administrator/components/com_contenthistory/controllers',
			'/administrator/components/com_content/views/featured/tmpl',
			'/administrator/components/com_content/views/featured',
			'/administrator/components/com_content/views/articles/tmpl',
			'/administrator/components/com_content/views/articles',
			'/administrator/components/com_content/views/article/tmpl',
			'/administrator/components/com_content/views/article',
			'/administrator/components/com_content/views',
			'/administrator/components/com_content/tables',
			'/administrator/components/com_content/models/forms',
			'/administrator/components/com_content/models/fields/modal',
			'/administrator/components/com_content/models/fields',
			'/administrator/components/com_content/models',
			'/administrator/components/com_content/helpers/html',
			'/administrator/components/com_content/controllers',
			'/administrator/components/com_contact/views/contacts/tmpl',
			'/administrator/components/com_contact/views/contacts',
			'/administrator/components/com_contact/views/contact/tmpl',
			'/administrator/components/com_contact/views/contact',
			'/administrator/components/com_contact/views',
			'/administrator/components/com_contact/tables',
			'/administrator/components/com_contact/models/forms/fields',
			'/administrator/components/com_contact/models/forms',
			'/administrator/components/com_contact/models/fields/modal',
			'/administrator/components/com_contact/models/fields',
			'/administrator/components/com_contact/models',
			'/administrator/components/com_contact/helpers/html',
			'/administrator/components/com_contact/controllers',
			'/administrator/components/com_config/view/component/tmpl',
			'/administrator/components/com_config/view/component',
			'/administrator/components/com_config/view/application/tmpl',
			'/administrator/components/com_config/view/application',
			'/administrator/components/com_config/view',
			'/administrator/components/com_config/models',
			'/administrator/components/com_config/model/form',
			'/administrator/components/com_config/model/field',
			'/administrator/components/com_config/model',
			'/administrator/components/com_config/helper',
			'/administrator/components/com_config/controllers',
			'/administrator/components/com_config/controller/component',
			'/administrator/components/com_config/controller/application',
			'/administrator/components/com_config/controller',
			'/administrator/components/com_checkin/views/checkin/tmpl',
			'/administrator/components/com_checkin/views/checkin',
			'/administrator/components/com_checkin/views',
			'/administrator/components/com_checkin/models/forms',
			'/administrator/components/com_checkin/models',
			'/administrator/components/com_categories/views/category/tmpl',
			'/administrator/components/com_categories/views/category',
			'/administrator/components/com_categories/views/categories/tmpl',
			'/administrator/components/com_categories/views/categories',
			'/administrator/components/com_categories/views',
			'/administrator/components/com_categories/tables',
			'/administrator/components/com_categories/models/forms',
			'/administrator/components/com_categories/models/fields/modal',
			'/administrator/components/com_categories/models/fields',
			'/administrator/components/com_categories/models',
			'/administrator/components/com_categories/helpers/html',
			'/administrator/components/com_categories/controllers',
			'/administrator/components/com_cache/views/purge/tmpl',
			'/administrator/components/com_cache/views/purge',
			'/administrator/components/com_cache/views/cache/tmpl',
			'/administrator/components/com_cache/views/cache',
			'/administrator/components/com_cache/views',
			'/administrator/components/com_cache/models/forms',
			'/administrator/components/com_cache/models',
			'/administrator/components/com_cache/helpers',
			'/administrator/components/com_banners/views/tracks/tmpl',
			'/administrator/components/com_banners/views/tracks',
			'/administrator/components/com_banners/views/download/tmpl',
			'/administrator/components/com_banners/views/download',
			'/administrator/components/com_banners/views/clients/tmpl',
			'/administrator/components/com_banners/views/clients',
			'/administrator/components/com_banners/views/client/tmpl',
			'/administrator/components/com_banners/views/client',
			'/administrator/components/com_banners/views/banners/tmpl',
			'/administrator/components/com_banners/views/banners',
			'/administrator/components/com_banners/views/banner/tmpl',
			'/administrator/components/com_banners/views/banner',
			'/administrator/components/com_banners/views',
			'/administrator/components/com_banners/tables',
			'/administrator/components/com_banners/models/forms',
			'/administrator/components/com_banners/models/fields',
			'/administrator/components/com_banners/models',
			'/administrator/components/com_banners/helpers/html',
			'/administrator/components/com_banners/controllers',
			'/administrator/components/com_associations/views/associations/tmpl',
			'/administrator/components/com_associations/views/associations',
			'/administrator/components/com_associations/views/association/tmpl',
			'/administrator/components/com_associations/views/association',
			'/administrator/components/com_associations/views',
			'/administrator/components/com_associations/models/forms',
			'/administrator/components/com_associations/models/fields',
			'/administrator/components/com_associations/models',
			'/administrator/components/com_associations/layouts/joomla/searchtools/default',
			'/administrator/components/com_associations/helpers',
			'/administrator/components/com_associations/controllers',
			'/administrator/components/com_admin/views/sysinfo/tmpl',
			'/administrator/components/com_admin/views/sysinfo',
			'/administrator/components/com_admin/views/profile/tmpl',
			'/administrator/components/com_admin/views/profile',
			'/administrator/components/com_admin/views/help/tmpl',
			'/administrator/components/com_admin/views/help',
			'/administrator/components/com_admin/views',
			'/administrator/components/com_admin/sql/updates/sqlazure',
			'/administrator/components/com_admin/models/forms',
			'/administrator/components/com_admin/models',
			'/administrator/components/com_admin/helpers/html',
			'/administrator/components/com_admin/helpers',
			'/administrator/components/com_admin/controllers',
			'/administrator/components/com_actionlogs/views/actionlogs/tmpl',
			'/administrator/components/com_actionlogs/views/actionlogs',
			'/administrator/components/com_actionlogs/views',
			'/administrator/components/com_actionlogs/models/forms',
			'/administrator/components/com_actionlogs/models/fields',
			'/administrator/components/com_actionlogs/models',
			'/administrator/components/com_actionlogs/libraries',
			'/administrator/components/com_actionlogs/helpers',
			'/administrator/components/com_actionlogs/controllers',
		);

		foreach ($files as $file)
		{
			if (File::exists(JPATH_ROOT . $file) && !File::delete(JPATH_ROOT . $file))
			{
				echo Text::sprintf('FILES_JOOMLA_ERROR_FILE_FOLDER', $file) . '<br>';
			}
		}

		foreach ($folders as $folder)
		{
			if (Folder::exists(JPATH_ROOT . $folder) && !Folder::delete(JPATH_ROOT . $folder))
			{
				echo Text::sprintf('FILES_JOOMLA_ERROR_FILE_FOLDER', $folder) . '<br>';
			}
		}
	}

	/**
	 * Method to create assets for newly installed components
	 *
	 * @param   Installer  $installer  The class calling this method
	 *
	 * @return  boolean
	 *
	 * @since   3.2
	 */
	public function updateAssets($installer)
	{
		// List all components added since 4.0
		$newComponents = array(
			'com_csp',
		);

		foreach ($newComponents as $component)
		{
			/** @var JTableAsset $asset */
			$asset = Table::getInstance('Asset');

			if ($asset->loadByName($component))
			{
				continue;
			}

			$asset->name      = $component;
			$asset->parent_id = 1;
			$asset->rules     = '{}';
			$asset->title     = $component;
			$asset->setLocation(1, 'last-child');

			if (!$asset->store())
			{
				// Install failed, roll back changes
				$installer->abort(Text::sprintf('JLIB_INSTALLER_ABORT_COMP_INSTALL_ROLLBACK', $asset->getError(true)));

				return false;
			}
		}

		return true;
	}

	/**
	 * If we migrated the session from the previous system, flush all the active sessions.
	 * Otherwise users will be logged in, but not able to do anything since they don't have
	 * a valid session
	 *
	 * @return  boolean
	 */
	public function flushSessions()
	{
		/**
		 * The session may have not been started yet (e.g. CLI-based Joomla! update scripts). Let's make sure we do
		 * have a valid session.
		 */
		$session = Factory::getSession();

		/**
		 * Restarting the Session require a new login for the current user so lets check if we have an active session
		 * and only restart it if not.
		 * For B/C reasons we need to use getState as isActive is not available in 2.5
		 */
		if ($session->getState() !== 'active')
		{
			$session->restart();
		}

		// If $_SESSION['__default'] is no longer set we do not have a migrated session, therefore we can quit.
		if (!isset($_SESSION['__default']))
		{
			return true;
		}

		$db = Factory::getDbo();

		try
		{
			switch ($db->getServerType())
			{
				// MySQL database, use TRUNCATE (faster, more resilient)
				case 'mysql':
					$db->truncateTable('#__session');
					break;

				// Non-MySQL databases, use a simple DELETE FROM query
				default:
					$query = $db->getQuery(true)
						->delete($db->quoteName('#__session'));
					$db->setQuery($query)->execute();
					break;
			}
		}
		catch (Exception $e)
		{
			echo Text::sprintf('JLIB_DATABASE_ERROR_FUNCTION_FAILED', $e->getCode(), $e->getMessage()) . '<br>';

			return false;
		}

		return true;
	}

	/**
	 * Converts the site's database tables to support UTF-8 Multibyte.
	 *
	 * @param   boolean  $doDbFixMsg  Flag if message to be shown to check db fix
	 *
	 * @return  void
	 *
	 * @since   3.5
	 */
	public function convertTablesToUtf8mb4($doDbFixMsg = false)
	{
		$db = Factory::getDbo();

		if ($db->getServerType() !== 'mysql')
		{
			return;
		}

		// Check if the #__utf8_conversion table exists
		$db->setQuery('SHOW TABLES LIKE ' . $db->quote($db->getPrefix() . 'utf8_conversion'));

		try
		{
			$rows = $db->loadRowList(0);
		}
		catch (Exception $e)
		{
			// Render the error message from the Exception object
			Factory::getApplication()->enqueueMessage($e->getMessage(), 'error');

			if ($doDbFixMsg)
			{
				// Show an error message telling to check database problems
				Factory::getApplication()->enqueueMessage(Text::_('JLIB_DATABASE_ERROR_DATABASE_UPGRADE_FAILED'), 'error');
			}

			return;
		}

		// Nothing to do if the table doesn't exist because the CMS has never been updated from a pre-4.0 version
		if (\count($rows) === 0)
		{
			return;
		}

		// Set required conversion status
		$converted = 5;

		// Check conversion status in database
		$db->setQuery('SELECT ' . $db->quoteName('converted')
			. ' FROM ' . $db->quoteName('#__utf8_conversion')
		);

		try
		{
			$convertedDB = $db->loadResult();
		}
		catch (Exception $e)
		{
			// Render the error message from the Exception object
			Factory::getApplication()->enqueueMessage($e->getMessage(), 'error');

			if ($doDbFixMsg)
			{
				// Show an error message telling to check database problems
				Factory::getApplication()->enqueueMessage(Text::_('JLIB_DATABASE_ERROR_DATABASE_UPGRADE_FAILED'), 'error');
			}

			return;
		}

		// If conversion status from DB is equal to required final status, try to drop the #__utf8_conversion table
		if ($convertedDB === $converted)
		{
			$this->dropUtf8ConversionTable();

			return;
		}

		// Perform the required conversions of core tables if not done already in a previous step
		if ($convertedDB !== 99)
		{
			$fileName1 = JPATH_ROOT . '/administrator/components/com_admin/sql/others/mysql/utf8mb4-conversion.sql';

			if (is_file($fileName1))
			{
				$fileContents1 = @file_get_contents($fileName1);
				$queries1      = $db->splitSql($fileContents1);

				if (!empty($queries1))
				{
					foreach ($queries1 as $query1)
					{
						try
						{
							$db->setQuery($query1)->execute();
						}
						catch (Exception $e)
						{
							$converted = $convertedDB;

							// Still render the error message from the Exception object
							Factory::getApplication()->enqueueMessage($e->getMessage(), 'error');
						}
					}
				}
			}
		}

		// If no error before, perform the optional conversions of tables which might or might not exist
		if ($converted === 5)
		{
			$fileName2 = JPATH_ROOT . '/administrator/components/com_admin/sql/others/mysql/utf8mb4-conversion_optional.sql';

			if (is_file($fileName2))
			{
				$fileContents2 = @file_get_contents($fileName2);
				$queries2      = $db->splitSql($fileContents2);

				if (!empty($queries2))
				{
					foreach ($queries2 as $query2)
					{
						// Get table name from query
						if (preg_match('/^ALTER\s+TABLE\s+([^\s]+)\s+/i', $query2, $matches) === 1)
						{
							$tableName = str_replace('`', '', $matches[1]);
							$tableName = str_replace('#__', $db->getPrefix(), $tableName);

							// Check if the table exists and if yes, run the query
							try
							{
								$db->setQuery('SHOW TABLES LIKE ' . $db->quote($tableName));

								$rows = $db->loadRowList(0);

								if (\count($rows) > 0)
								{
									$db->setQuery($query2)->execute();
								}
							}
							catch (Exception $e)
							{
								$converted = 99;

								// Still render the error message from the Exception object
								Factory::getApplication()->enqueueMessage($e->getMessage(), 'error');
							}
						}
					}
				}
			}
		}

		if ($doDbFixMsg && $converted !== 5)
		{
			// Show an error message telling to check database problems
			Factory::getApplication()->enqueueMessage(Text::_('JLIB_DATABASE_ERROR_DATABASE_UPGRADE_FAILED'), 'error');
		}

		// If the conversion was successful try to drop the #__utf8_conversion table
		if ($converted === 5 && $this->dropUtf8ConversionTable())
		{
			// Table successfully dropped
			return;
		}

		// Set flag in database if the conversion status has changed.
		if ($converted !== $convertedDB)
		{
			$db->setQuery('UPDATE ' . $db->quoteName('#__utf8_conversion')
				. ' SET ' . $db->quoteName('converted') . ' = ' . $converted . ';'
			)->execute();
		}
	}

	/**
	 * This method clean the Joomla Cache using the method `clean` from the com_cache model
	 *
	 * @return  void
	 *
	 * @since   3.5.1
	 */
	private function cleanJoomlaCache()
	{
		$model = new \Joomla\Component\Cache\Administrator\Model\CacheModel;

		// Clean frontend cache
		$model->clean();

		// Clean admin cache
		$model->setState('client_id', 1);
		$model->clean();
	}

	/**
	 * This method drops the #__utf8_conversion table
	 *
	 * @return  boolean  True on success
	 *
	 * @since   4.0.0
	 */
	private function dropUtf8ConversionTable()
	{
		$db = Factory::getDbo();

		try
		{
			$db->setQuery('DROP TABLE ' . $db->quoteName('#__utf8_conversion') . ';'
			)->execute();
		}
		catch (Exception $e)
		{
			return false;
		}

		return true;
	}

	/**
	 * Called after any type of action
	 *
	 * @param   string     $action     Which action is happening (install|uninstall|discover_install|update)
	 * @param   Installer  $installer  The class calling this method
	 *
	 * @return  boolean  True on success
	 *
	 * @since   4.0.0
	 */
	public function postflight($action, $installer)
	{
		if ($action !== 'update')
		{
			return true;
		}

		if (empty($this->fromVersion) || version_compare($this->fromVersion, '4.0.0', 'ge'))
		{
			return true;
		}

		// Update UCM content types.
		$this->updateContentTypes();

		$db = Factory::getDbo();
		Table::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_menus/Table/');

		$tableItem   = new \Joomla\Component\Menus\Administrator\Table\MenuTable($db);

		$contactItems = $this->contactItems($tableItem);
		$finderItems  = $this->finderItems($tableItem);

		$menuItems = array_merge($contactItems, $finderItems);

		foreach ($menuItems as $menuItem)
		{
			// Check an existing record
			$keys = [
				'menutype'  => $menuItem['menutype'],
				'type'      => $menuItem['type'],
				'title'     => $menuItem['title'],
				'parent_id' => $menuItem['parent_id'],
				'client_id' => $menuItem['client_id'],
			];

			if ($tableItem->load($keys))
			{
				continue;
			}

			$newTableItem = new \Joomla\Component\Menus\Administrator\Table\MenuTable($db);

			// Bind the data.
			if (!$newTableItem->bind($menuItem))
			{
				return false;
			}

			$newTableItem->setLocation($menuItem['parent_id'], 'last-child');

			// Check the data.
			if (!$newTableItem->check())
			{
				return false;
			}

			// Store the data.
			if (!$newTableItem->store())
			{
				return false;
			}

			// Rebuild the tree path.
			if (!$newTableItem->rebuildPath($newTableItem->id))
			{
				return false;
			}
		}

		return true;
	}

	/**
	 * Prepare the contact menu items
	 *
	 * @return  array  Menu items
	 *
	 * @since   4.0.0
	 */
	private function contactItems(Table $tableItem): array
	{
		// Check for the Contact parent Id Menu Item
		$keys = [
			'menutype'  => 'main',
			'type'      => 'component',
			'title'     => 'com_contact',
			'parent_id' => 1,
			'client_id' => 1,
		];

		$contactMenuitem = $tableItem->load($keys);

		if (!$contactMenuitem)
		{
			return [];
		}

		$parentId    = $tableItem->id;
		$componentId = ExtensionHelper::getExtensionRecord('com_fields', 'component')->extension_id;

		// Add Contact Fields Menu Items.
		$menuItems = [
			[
				'menutype'          => 'main',
				'title'             => '-',
				'alias'             => microtime(true),
				'note'              => '',
				'path'              => '',
				'link'              => '#',
				'type'              => 'separator',
				'published'         => 1,
				'parent_id'         => $parentId,
				'level'             => 2,
				'component_id'      => $componentId,
				'checked_out'       => null,
				'checked_out_time'  => null,
				'browserNav'        => 0,
				'access'            => 0,
				'img'               => '',
				'template_style_id' => 0,
				'params'            => '{}',
				'home'              => 0,
				'language'          => '*',
				'client_id'         => 1,
				'publish_up'        => null,
				'publish_down'      => null,
			],
			[
				'menutype'          => 'main',
				'title'             => 'mod_menu_fields',
				'alias'             => 'Contact Custom Fields',
				'note'              => '',
				'path'              => 'contact/Custom Fields',
				'link'              => 'index.php?option=com_fields&context=com_contact.contact',
				'type'              => 'component',
				'published'         => 1,
				'parent_id'         => $parentId,
				'level'             => 2,
				'component_id'      => $componentId,
				'checked_out'       => null,
				'checked_out_time'  => null,
				'browserNav'        => 0,
				'access'            => 0,
				'img'               => '',
				'template_style_id' => 0,
				'params'            => '{}',
				'home'              => 0,
				'language'          => '*',
				'client_id'         => 1,
				'publish_up'        => null,
				'publish_down'      => null,
			],
			[
				'menutype'          => 'main',
				'title'             => 'mod_menu_fields_group',
				'alias'             => 'Contact Custom Fields Group',
				'note'              => '',
				'path'              => 'contact/Custom Fields Group',
				'link'              => 'index.php?option=com_fields&view=groups&context=com_contact.contact',
				'type'              => 'component',
				'published'         => 1,
				'parent_id'         => $parentId,
				'level'             => 2,
				'component_id'      => $componentId,
				'checked_out'       => null,
				'checked_out_time'  => null,
				'browserNav'        => 0,
				'access'            => 0,
				'img'               => '',
				'template_style_id' => 0,
				'params'            => '{}',
				'home'              => 0,
				'language'          => '*',
				'client_id'         => 1,
				'publish_up'        => null,
				'publish_down'      => null,
			]
		];

		return $menuItems;
	}

	/**
	 * Prepare the finder menu items
	 *
	 * @return  array  Menu items
	 *
	 * @since   4.0.0
	 */
	private function finderItems(Table $tableItem): array
	{
		// Check for the Finder parent Id Menu Item
		$keys = [
			'menutype'  => 'main',
			'type'      => 'component',
			'title'     => 'com_finder',
			'parent_id' => 1,
			'client_id' => 1,
		];

		$finderMenuitem = $tableItem->load($keys);

		if (!$finderMenuitem)
		{
			return [];
		}

		$parentId    = $tableItem->id;
		$componentId = ExtensionHelper::getExtensionRecord('com_finder', 'component')->extension_id;

		// Add Finder Fields Menu Items.
		$menuItems = [
			[
				'menutype'          => 'main',
				'title'             => '-',
				'alias'             => microtime(true),
				'note'              => '',
				'path'              => '',
				'link'              => '#',
				'type'              => 'separator',
				'published'         => 1,
				'parent_id'         => $parentId,
				'level'             => 2,
				'component_id'      => $componentId,
				'checked_out'       => null,
				'checked_out_time'  => null,
				'browserNav'        => 0,
				'access'            => 0,
				'img'               => '',
				'template_style_id' => 0,
				'params'            => '{}',
				'home'              => 0,
				'language'          => '*',
				'client_id'         => 1,
				'publish_up'        => null,
				'publish_down'      => null,
			],
			[
				'menutype'          => 'main',
				'title'             => 'com_finder_index',
				'alias'             => 'Smart-Search-Index',
				'note'              => '',
				'path'              => 'Smart Search/Index',
				'link'              => 'index.php?option=com_finder&view=index',
				'type'              => 'component',
				'published'         => 1,
				'parent_id'         => $parentId,
				'level'             => 2,
				'component_id'      => $componentId,
				'checked_out'       => null,
				'checked_out_time'  => null,
				'browserNav'        => 0,
				'access'            => 0,
				'img'               => '',
				'template_style_id' => 0,
				'params'            => '{}',
				'home'              => 0,
				'language'          => '*',
				'client_id'         => 1,
				'publish_up'        => null,
				'publish_down'      => null,
			],
			[
				'menutype'          => 'main',
				'title'             => 'com_finder_maps',
				'alias'             => 'Smart-Search-Maps',
				'note'              => '',
				'path'              => 'Smart Search/Maps',
				'link'              => 'index.php?option=com_finder&view=maps',
				'type'              => 'component',
				'published'         => 1,
				'parent_id'         => $parentId,
				'level'             => 2,
				'component_id'      => $componentId,
				'checked_out'       => null,
				'checked_out_time'  => null,
				'browserNav'        => 0,
				'access'            => 0,
				'img'               => '',
				'template_style_id' => 0,
				'params'            => '{}',
				'home'              => 0,
				'language'          => '*',
				'client_id'         => 1,
				'publish_up'        => null,
				'publish_down'      => null,
			],
			[
				'menutype'          => 'main',
				'title'             => 'com_finder_filters',
				'alias'             => 'Smart-Search-Filters',
				'note'              => '',
				'path'              => 'Smart Search/Filters',
				'link'              => 'index.php?option=com_finder&view=filters',
				'type'              => 'component',
				'published'         => 1,
				'parent_id'         => $parentId,
				'level'             => 2,
				'component_id'      => $componentId,
				'checked_out'       => null,
				'checked_out_time'  => null,
				'browserNav'        => 0,
				'access'            => 0,
				'img'               => '',
				'template_style_id' => 0,
				'params'            => '{}',
				'home'              => 0,
				'language'          => '*',
				'client_id'         => 1,
				'publish_up'        => null,
				'publish_down'      => null,
			],
			[
				'menutype'          => 'main',
				'title'             => 'com_finder_searches',
				'alias'             => 'Smart-Search-Searches',
				'note'              => '',
				'path'              => 'Smart Search/Searches',
				'link'              => 'index.php?option=com_finder&view=searches',
				'type'              => 'component',
				'published'         => 1,
				'parent_id'         => $parentId,
				'level'             => 2,
				'component_id'      => $componentId,
				'checked_out'       => null,
				'checked_out_time'  => null,
				'browserNav'        => 0,
				'access'            => 0,
				'img'               => '',
				'template_style_id' => 0,
				'params'            => '{}',
				'home'              => 0,
				'language'          => '*',
				'client_id'         => 1,
				'publish_up'        => null,
				'publish_down'      => null,
			]
		];

		return $menuItems;
	}

	/**
	 * Updates content type table classes.
	 *
	 * @return  void
	 *
	 * @since   4.0.0
	 */
	private function updateContentTypes(): void
	{
		// Content types to update.
		$contentTypes = [
			'com_content.article',
			'com_contact.contact',
			'com_newsfeeds.newsfeed',
			'com_tags.tag',
			'com_banners.banner',
			'com_banners.client',
			'com_users.note',
			'com_content.category',
			'com_contact.category',
			'com_newsfeeds.category',
			'com_banners.category',
			'com_users.category',
			'com_users.user',
		];

		// Get table definitions.
		$db = Factory::getDbo();
		$query = $db->getQuery(true)
			->select(
				[
					$db->quoteName('type_alias'),
					$db->quoteName('table'),
				]
			)
			->from($db->quoteName('#__content_types'))
			->whereIn($db->quoteName('type_alias'), $contentTypes, ParameterType::STRING);

		$db->setQuery($query);
		$contentTypes = $db->loadObjectList();

		// Prepare the update query.
		$query = $db->getQuery(true)
			->update($db->quoteName('#__content_types'))
			->set($db->quoteName('table') . ' = :table')
			->where($db->quoteName('type_alias') . ' = :typeAlias')
			->bind(':table', $table)
			->bind(':typeAlias', $typeAlias);

		$db->setQuery($query);

		foreach ($contentTypes as $contentType)
		{
			list($component, $tableType) = explode('.', $contentType->type_alias);

			// Special case for core table classes.
			if ($contentType->type_alias === 'com_users.users' || $tableType === 'category')
			{
				$tablePrefix = 'Joomla\\CMS\Table\\';
				$tableType   = ucfirst($tableType);
			}
			else
			{
				$tablePrefix = 'Joomla\\Component\\' . ucfirst(substr($component, 4)) . '\\Administrator\\Table\\';
				$tableType   = ucfirst($tableType) . 'Table';
			}

			// Bind type alias.
			$typeAlias = $contentType->type_alias;

			$table = json_decode($contentType->table);

			// Update table definitions.
			$table->special->type   = $tableType;
			$table->special->prefix = $tablePrefix;

			// Some content types don't have this property.
			if (!empty($table->common->prefix))
			{
				$table->common->prefix  = 'Joomla\\CMS\\Table\\';
			}

			$table = json_encode($table);

			// Execute the query.
			$db->execute();
		}
	}
}
